// Type definitions for roosterjs (Version 8.57.0)
// Generated by dts tool from roosterjs
// Project: https://github.com/Microsoft/roosterjs

declare namespace roosterjs {
    /**
 * Create an editor instance with most common options
 * @param contentDiv The html div element needed for creating the editor
 * @param additionalPlugins The additional user defined plugins. Currently the default plugins that are already included are
 * ContentEdit, HyperLink and Paste, user don't need to add those.
 * @param initialContent The initial content to show in editor. It can't be removed by undo, user need to manually remove it if needed.
 * @returns The editor instance
 */
function createEditor(contentDiv: HTMLDivElement, additionalPlugins?: EditorPlugin[], initialContent?: string): IEditor;

    /**
 * This produces a block element from a a node
 * It needs to account for various HTML structure. Examples:
 * 1) &lt;root&gt;&lt;div&gt;abc&lt;/div&gt;&lt;/root&gt;
 *   This is most common the case, user passes in a node pointing to abc, and get back a block representing &lt;div&gt;abc&lt;/div&gt;
 * 2) &lt;root&gt;&lt;p&gt;&lt;br&gt;&lt;/p&gt;&lt;/root&gt;
 *   Common content for empty block, user passes node pointing to &lt;br&gt;, and get back a block representing &lt;p&gt;&lt;br&gt;&lt;/p&gt;
 * 3) &lt;root&gt;abc&lt;/root&gt;
 *   Not common, but does happen. It is still a block in user's view. User passes in abc, and get back a start-end block representing abc
 *   NOTE: abc could be just one node. However, since it is not a html block, it is more appropriate to use start-end block although they point to same node
 * 4) &lt;root&gt;&lt;div&gt;abc&lt;br&gt;123&lt;/div&gt;&lt;/root&gt;
 *   A bit tricky, but can happen when user use Ctrl+Enter which simply inserts a &lt;BR&gt; to create a link break. There're two blocks:
 *   block1: 1) abc&lt;br&gt; block2: 123
 * 5) &lt;root&gt;&lt;div&gt;abc&lt;div&gt;123&lt;/div&gt;&lt;/div&gt;&lt;/root&gt;
 *   Nesting div and there is text node in same level as a DIV. Two blocks: 1) abc 2) &lt;div&gt;123&lt;/div&gt;
 * 6) &lt;root&gt;&lt;div&gt;abc&lt;span&gt;123&lt;br&gt;456&lt;/span&gt;&lt;/div&gt;&lt;/root&gt;
 *   This is really tricky. Essentially there is a &lt;BR&gt; in middle of a span breaking the span into two blocks;
 *   block1: abc&lt;span&gt;123&lt;br&gt; block2: 456
 * In summary, given any arbitrary node (leaf), to identify the head and tail of the block, following rules need to be followed:
 * 1) to identify the head, it needs to crawl DOM tre left/up till a block node or BR is encountered
 * 2) same for identifying tail
 * 3) should also apply a block ceiling, meaning as it crawls up, it should stop at a block node
 * @param rootNode Root node of the scope, the block element will be inside of this node
 * @param node The node to get BlockElement start from
 */
function getBlockElementAtNode(rootNode: Node, node: Node | null): BlockElement | null;

    /**
 * Get the first/last BlockElement of under the root node.
 * If no suitable BlockElement found, returns null
 * @param rootNode The root node to get BlockElement from
 * @param isFirst True to get first BlockElement, false to get last BlockElement
 */
function getFirstLastBlockElement(rootNode: Node, isFirst: boolean): BlockElement | null;

    /**
 * The provides traversing of content inside editor.
 * There are two ways to traverse, block by block, or inline element by inline element
 * Block and inline traversing is independent from each other, meaning if you traverse block by block, it does not change
 * the current inline element position
 */
class ContentTraverser implements IContentTraverser  {
    private scoper;
    private skipTags?;
    private currentInline;
    private currentBlock;
    /**
     * Create a content traverser for the whole body of given root node
     * @param scoper Traversing scoper object to help scope the traversing
     * @param skipTags (Optional) tags that child elements will be skipped
     */
    private constructor();
    /**
     * Create a content traverser for the whole body of given root node
     * @param rootNode The root node to traverse in
     * @param startNode The node to start from. If not passed, it will start from the beginning of the body
     * @param skipTags (Optional) tags that child elements will be skipped
     */
    static createBodyTraverser(rootNode: Node, startNode?: Node, skipTags?: string[]): IContentTraverser;
    /**
     * Create a content traverser for the given selection
     * @param rootNode The root node to traverse in
     * @param range The selection range to scope the traversing
     * @param skipTags (Optional) tags that child elements will be skipped
     */
    static createSelectionTraverser(rootNode: Node, range: Range, skipTags?: string[]): IContentTraverser;
    /**
     * Create a content traverser for a block element which contains the given position
     * @param rootNode The root node to traverse in
     * @param position A position inside a block, traversing will be scoped within this block.
     * If passing a range, the start position of this range will be used
     * @param startFrom Start position of traversing. The value can be Begin, End, SelectionStart
     * @param skipTags (Optional) tags that child elements will be skipped
     */
    static createBlockTraverser(rootNode: Node, position: NodePosition | Range, start?: ContentPosition | CompatibleContentPosition, skipTags?: string[]): IContentTraverser;
    /**
     * Get current block
     */
    get currentBlockElement(): BlockElement | null;
    /**
     * Get next block element
     */
    getNextBlockElement(): BlockElement | null;
    /**
     * Get previous block element
     */
    getPreviousBlockElement(): BlockElement | null;
    private getPreviousNextBlockElement;
    /**
     * Current inline element getter
     */
    get currentInlineElement(): InlineElement | null;
    /**
     * Get next inline element
     */
    getNextInlineElement(): InlineElement | null;
    /**
     * Get previous inline element
     */
    getPreviousInlineElement(): InlineElement | null;
    private getPreviousNextInlineElement;
}

    /**
 * The class that helps search content around a position
 */
class PositionContentSearcher implements IPositionContentSearcher  {
    private rootNode;
    private position;
    private text;
    private word;
    private inlineBefore;
    private inlineAfter;
    private traverser;
    private traversingComplete;
    private inlineElements;
    private nearestNonTextInlineElement;
    /**
     * Create a new CursorData instance
     * @param rootNode Root node of the whole scope
     * @param position Start position
     */
    constructor(rootNode: Node, position: NodePosition);
    /**
     * Get the word before position. The word is determined by scanning backwards till the first white space, the portion
     * between position and the white space is the word before position
     * @returns The word before position
     */
    getWordBefore(): string;
    /**
     * Get the inline element before position
     * @returns The inlineElement before position
     */
    getInlineElementBefore(): InlineElement | null;
    /**
     * Get the inline element after position
     * @returns The inline element after position
     */
    getInlineElementAfter(): InlineElement | null;
    /**
     * Get X number of chars before position
     * The actual returned chars may be less than what is requested.
     * @param length The length of string user want to get, the string always ends at the position,
     * so this length determines the start position of the string
     * @returns The actual string we get as a sub string, or the whole string before position when
     * there is not enough chars in the string
     */
    getSubStringBefore(length: number): string;
    /**
     * Try to get a range matches the given text before the position
     * @param text The text to match against
     * @param exactMatch Whether it is an exact match
     * @returns The range for the matched text, null if unable to find a match
     */
    getRangeFromText(text: string, exactMatch: boolean): Range | null;
    /**
     * Get text section before position till stop condition is met.
     * This offers consumers to retrieve text section by section
     * The section essentially is just an inline element which has Container element
     * so that the consumer can remember it for anchoring popup or verification purpose
     * when position moves out of context etc.
     * @param stopFunc The callback stop function
     */
    forEachTextInlineElement(callback: (textInlineElement: InlineElement) => any): void;
    /**
     * Get first non textual inline element before position
     * @returns First non textual inline element before position or null if no such element exists
     */
    getNearestNonTextInlineElement(): InlineElement | null;
    /**
     * Continue traversing backward till stop condition is met or begin of block is reached
     */
    private traverse;
}

    /**
 * Adds delimiters to the element provided. If the delimiters already exists, will not be added
 * @param node the node to add the delimiters
 */
function addDelimiters(node: Element): Element[];

    /**
 * Adds delimiter after the element provided.
 * @param element element to use
 */
function addDelimiterAfter(element: Element): HTMLElement;

    /**
 * Adds delimiter before the element provided.
 * @param element element to use
 */
function addDelimiterBefore(element: Element): HTMLElement;

    /**
 * Retrieves Delimiter information from a provided element.
 * @param element element to try to retrieve a delimiter
 * @returns delimiter info if it is a Delimiter, else null
 */
function getDelimiterFromElement(element: Node | null | undefined): Element | null;

    /**
 * Get the inline element at a node
 * @param rootNode The root node of current scope
 * @param node The node to get InlineElement from
 */
function getInlineElementAtNode(rootNode: Node, node: Node | null): InlineElement;

    /**
 * Get the inline element at a node
 * @param parentBlock Parent BlockElement of this node
 * @param node The node to get InlineElement from
 */
function getInlineElementAtNode(parentBlock: BlockElement, node: Node | null): InlineElement;

    /**
 * This is an inline element representing an Html image
 */
class ImageInlineElement extends NodeInlineElement  {
    constructor(containerNode: Node, parentBlock: BlockElement);
}

    /**
 * This is inline element presenting an html hyperlink
 */
class LinkInlineElement extends NodeInlineElement  {
    constructor(containerNode: Node, parentBlock: BlockElement);
}

    /**
 * This presents an inline element that can be represented by a single html node.
 * This serves as base for most inline element as it contains most implementation
 * of all operations that can happen on an inline element. Other sub inline elements mostly
 * just identify themselves for a certain type
 */
class NodeInlineElement implements InlineElement  {
    private containerNode;
    private parentBlock;
    constructor(containerNode: Node, parentBlock: BlockElement);
    /**
     * The text content for this inline element
     */
    getTextContent(): string;
    /**
     * Get the container node
     */
    getContainerNode(): Node;
    getParentBlock(): BlockElement;
    /**
     * Get the start position of the inline element
     */
    getStartPosition(): NodePosition;
    /**
     * Get the end position of the inline element
     */
    getEndPosition(): NodePosition;
    /**
     * Checks if this inline element is a textual inline element
     */
    isTextualInlineElement(): boolean;
    /**
     * Checks if an inline element is after the current inline element
     */
    isAfter(inlineElement: InlineElement): boolean;
    /**
     * Checks if the given position is contained in the inline element
     */
    contains(pos: NodePosition): boolean;
    /**
     * Apply inline style to an inline element
     */
    applyStyle(styler: (element: HTMLElement, isInnerNode?: boolean) => any): void;
}

    /**
 * This is a special version of inline element that identifies a section of an inline element
 * We often have the need to cut an inline element in half and perform some operation only on half of an inline element
 * i.e. users select only some text of a text node and apply format, in that case, format has to happen on partial of an inline element
 * PartialInlineElement is implemented in a way that decorate another full inline element with its own override on methods like isAfter
 * It also offers some special methods that others don't have, i.e. nextInlineElement etc.
 */
class PartialInlineElement implements InlineElement  {
    private inlineElement;
    private start;
    private end;
    constructor(inlineElement: InlineElement, start?: NodePosition | null, end?: NodePosition | null);
    /**
     * Get the full inline element that this partial inline decorates
     */
    getDecoratedInline(): InlineElement;
    /**
     * Gets the container node
     */
    getContainerNode(): Node;
    /**
     * Gets the parent block
     */
    getParentBlock(): BlockElement;
    /**
     * Gets the text content
     */
    getTextContent(): string;
    /**
     * Get start position of this inline element.
     */
    getStartPosition(): NodePosition;
    /**
     * Get end position of this inline element.
     */
    getEndPosition(): NodePosition;
    /**
     * Get next partial inline element if it is not at the end boundary yet
     */
    get nextInlineElement(): PartialInlineElement | null;
    /**
     * Get previous partial inline element if it is not at the begin boundary yet
     */
    get previousInlineElement(): PartialInlineElement | null;
    /**
     * Checks if it contains a position
     */
    contains(pos: NodePosition): boolean;
    /**
     * Checks if this inline element is a textual inline element
     */
    isTextualInlineElement(): boolean;
    /**
     * Check if this inline element is after the other inline element
     */
    isAfter(inlineElement: InlineElement): boolean;
    /**
     * apply style
     */
    applyStyle(styler: (element: HTMLElement, isInnerNode?: boolean) => any): void;
}

    /**
 * Apply style using a styler function to the given container node in the given range
 * @param container The container node to apply style to
 * @param styler The styler function
 * @param fromPosition From position
 * @param toPosition To position
 */
function applyTextStyle(container: Node, styler: (node: HTMLElement, isInnerNode?: boolean) => any, from?: NodePosition, to?: NodePosition): void;

    /**
 * @deprecated Use extractClipboardItems and extractClipboardItemsForIE instead
 * Extract a Clipboard event
 * @param event The paste event
 * @param callback Callback function when data is ready
 * @param options Options to retrieve more items from the event, including HTML string and other customized items
 * @param rangeBeforePaste Optional range to be removed when pasting in Android
 * @returns An object with the following properties:
 *  types: Available types from the clipboard event
 *  text: Plain text from the clipboard event
 *  image: Image file from the clipboard event
 *  html: Html string from the clipboard event. When set to null, it means there's no HTML found from the event.
 *   When set to undefined, it means can't retrieve HTML string, there may be HTML string but direct retrieving is
 *   not supported by browser.
 */
function extractClipboardEvent(event: ClipboardEvent, callback: (clipboardData: ClipboardData) => void, options?: ExtractClipboardEventOption, rangeBeforePaste?: Range): void;

    /**
 * Extract clipboard items to be a ClipboardData object for IE
 * @param items The clipboard items retrieve from a DataTransfer object
 * @param callback Callback function when data is ready
 * @returns An object with the following properties:
 *  types: Available types from the clipboard event
 *  text: Plain text from the clipboard event
 *  image: Image file from the clipboard event
 *  html: Html string from the clipboard event. When set to null, it means there's no HTML found from the event.
 *   When set to undefined, it means can't retrieve HTML string, there may be HTML string but direct retrieving is
 *   not supported by browser.
 */
function extractClipboardItems(items: DataTransferItem[], options?: ExtractClipboardItemsOption, pasteNativeEvent?: boolean): Promise<ClipboardData>;

    /**
 * Extract clipboard items to be a ClipboardData object for IE
 * @param dataTransfer The clipboard items retrieve from a DataTransfer object
 * @param callback Callback function when data is ready
 * @returns An object with the following properties:
 *  types: Available types from the clipboard event
 *  text: Plain text from the clipboard event
 *  image: Image file from the clipboard event
 *  html: Html string from the clipboard event. When set to null, it means there's no HTML found from the event.
 *   When set to undefined, it means can't retrieve HTML string, there may be HTML string but direct retrieving is
 *   not supported by browser.
 */
function extractClipboardItemsForIE(dataTransfer: DataTransfer, callback: (data: ClipboardData) => void, options?: ExtractClipboardItemsForIEOptions): void;

    /**
 * Handles the content when using the Image Paste Option
 * @param imageDataUri the image uri to use for the image
 * @param fragment fragment that will contain the content to paste.
 */
function handleImagePaste(imageDataUri: string, fragment: DocumentFragment): void;

    /**
 * handle the content when using the text only option
 * @param text Text from clipboard
 * @param position current position of the clipboard
 * @param fragment fragment that contains the paste content.
 */
function handleTextPaste(text: string, position: NodePosition | null, fragment: DocumentFragment): void;

    /**
 * Retrieves the metadata from the content inside of the clipboard
 * @param doc Document parsed from the clipboard
 * @param event Before Paste event
 * @param trustedHTMLHandler the trusted html handler to sanitize the content.
 */
function retrieveMetadataFromClipboard(doc: Document | undefined, event: BeforePasteEvent, trustedHTMLHandler: TrustedHTMLHandler): void;

    /**
 * Sanitize the content from the pasted content
 * @param event The before paste event
 * @param position the position of the cursor
 */
function sanitizePasteContent(event: BeforePasteEvent, position: NodePosition | null): void;

    /**
 * Get the paste type that will be used corresponding to the configuration
 * @param pasteAsText Whether to paste as Text
 * @param applyCurrentStyle Whether to apply the current format to the content
 * @param pasteAsImage Whether to only paste the image
 * @returns
 */
function getPasteType(pasteAsText: boolean, applyCurrentStyle: boolean, pasteAsImage: boolean): PasteType;

    /**
 * Browser object contains browser and operating system information of current environment
 */
const Browser: BrowserInfo;

    /**
 * Get current browser information from user agent string
 * @param userAgent The userAgent string of a browser
 * @param appVersion The appVersion string of a browser
 * @param vendor The vendor string of a browser
 * @returns The BrowserInfo object calculated from the given userAgent and appVersion
 */
function getBrowserInfo(userAgent: string, appVersion: string, vendor?: string): BrowserInfo;

    /**
 * Apply format to an HTML element
 * @param element The HTML element to apply format to
 * @param format The format to apply
 * @param isDarkMode Whether the content should be formatted in dark mode
 * @param darkColorHandler An optional dark color handler object. When it is passed, we will use this handler to do variable-based dark color instead of original dataset base dark color
 */
function applyFormat(element: HTMLElement, format: DefaultFormat, isDarkMode?: boolean, darkColorHandler?: DarkColorHandler | null): void;

    /**
 * Change tag of an HTML Element to a new one, and replace it from DOM tree
 * @param element The element to change tag
 * @param newTag New tag to change to
 * @returns The new element with new tag
 */
function changeElementTag<K extends keyof HTMLElementTagNameMap>(element: HTMLElement, newTag: K): HTMLElementTagNameMap[K];

    /**
 * Change tag of an HTML Element to a new one, and replace it from DOM tree
 * @param element The element to change tag
 * @param newTag New tag to change to
 * @returns The new element with new tag
 */
function changeElementTag(element: HTMLElement, newTag: string): HTMLElement | null;

    /**
 * Collapse nodes within the given start and end nodes to their common ancestor node,
 * split parent nodes if necessary
 * @param root The root node of the scope
 * @param start The start node
 * @param end The end node
 * @param canSplitParent True to allow split parent node there are nodes before start or after end under the same parent
 * and the returned nodes will be all nodes from start through end after splitting
 * False to disallow split parent
 * @returns When canSplitParent is true, returns all node from start through end after splitting,
 * otherwise just return start and end
 */
function collapseNodes(root: Node, start: Node, end: Node, canSplitParent: boolean): Node[];

    /**
 * Test if a node contains another node
 * @param container The container node
 * @param contained The node to check if it is inside container
 * @param treatSameNodeAsContain When container and contained are the same node,
 * return true if this param is set to true, otherwise return false. Default value is false
 * @returns True if contained is inside container, or they are the same node when treatSameNodeAsContain is true.
 * Otherwise false.
 */
function contains(container: Node | null | undefined, contained: Node | null | undefined, treatSameNodeAsContain?: boolean): boolean;

    /**
 * Test if a node contains a given range
 * @param container The container node
 * @param contained The range to check if it is inside container
 * @returns True if contained is inside container, otherwise false
 */
function contains(container: Node | null | undefined, contained: Range | null | undefined): boolean;

    /**
 * Find closest element ancestor start from the given node which matches the given selector
 * @param node Find ancestor start from this node
 * @param root Root node where the search should stop at. The return value can never be this node
 * @param selector The expected selector. If null, return the first HTML Element found from start node
 * @returns An HTML element which matches the given selector. If the given start node matches the selector,
 * returns the given node
 */
function findClosestElementAncestor(node: Node | null, root?: Node, selector?: string): HTMLElement | null;

    /**
 * @deprecated
 * Creates an HTML node array from html
 * @param html the html string to create HTML elements from
 * @param ownerDocument Owner document of the result HTML elements
 * @returns An HTML node array to represent the given html string
 */
function fromHtml(html: string, ownerDocument: HTMLDocument): Node[];

    /**
 * Get computed styles of a node
 * @param node The node to get computed styles from
 * @param styleNames Names of style to get, can be a single name or an array.
 * Default value is font-family, font-size, color, background-color
 * @returns An array of the computed styles
 */
function getComputedStyles(node: Node, styleNames?: string | string[]): string[];

    /**
 * A shortcut for getComputedStyles() when only one style is to be retrieved
 * @param node The node to get style from
 * @param styleName The style name
 * @returns The style value
 */
function getComputedStyle(node: Node, styleName: string): string;

    /**
 * Get Pendable Format State at cursor.
 * @param document The HTML Document to get format state from
 * @returns A PendableFormatState object which contains the values of pendable format states
 */
function getPendableFormatState(document: Document): PendableFormatState;

    /**
 * A map from pendable format name to document command
 */
const PendableFormatCommandMap: {
    [key in PendableFormatNames]: DocumentCommand;
};

    /**
 * Names of Pendable formats
 */
type PendableFormatNames = keyof PendableFormatState;

    /**
 * Get the html tag of a node, or empty if it is not an element
 * @param node The node to get tag of
 * @returns Tag name in upper case if the given node is an Element, or empty string otherwise
 */
function getTagOfNode(node: Node | null): string;

    /**
 * Checks if the node is a block like element. Block like element are usually those P, DIV, LI, TD etc.
 * @param node The node to check
 * @returns True if the node is a block element, otherwise false
 */
function isBlockElement(node: Node): node is HTMLElement;

    /**
 * Check if a given node has no visible content
 * @param node The node to check
 * @param trimContent Whether trim the text content so that spaces will be treated as empty.
 * Default value is false
 * @returns True if there isn't any visible element inside node, otherwise false
 */
function isNodeEmpty(node: Node, trimContent?: boolean, shouldCountBrAsVisible?: boolean): boolean;

    /**
 * Check if the given node is html void element. Void element cannot have children
 * @param node The node to check
 */
function isVoidHtmlElement(node: Node): boolean;

    /**
 * Try to match a given string with link match rules, return matched link
 * @param url Input url to match
 * @param option Link match option, exact or partial. If it is exact match, we need
 * to check the length of matched link and url
 * @param rules Optional link match rules, if not passed, only the default link match
 * rules will be applied
 * @returns The matched link data, or null if no match found.
 * The link data includes an original url and a normalized url
 */
function matchLink(url: string): LinkData | null;

    /**
 * Query HTML elements in the container by a selector string
 * @param container Container element to query from
 * @param selector Selector string to query
 * @param forEachCallback An optional callback to be invoked on each node in query result
 * @param scope The scope of the query, default value is QueryScope.Body
 * @param range The selection range to query with. This is required when scope is not Body
 * @returns HTML Element array of the query result
 */
function queryElements(container: ParentNode, selector: string, forEachCallback?: ((node: HTMLElement) => any) | null, scope?: QueryScope | CompatibleQueryScope, range?: Range): HTMLElement[];

    /**
 * Split parent node of the given node before/after the given node.
 * When a parent node contains [A,B,C] and pass B as the given node,
 * If split before, the new nodes will be [A][B,C] and returns [A];
 * otherwise, it will be [A,B][C] and returns [C].
 * @param node The node to split before/after
 * @param splitBefore Whether split before or after
 * @param removeEmptyNewNode If the new node is empty (even then only child is space or ZER_WIDTH_SPACE),
 * we remove it. @default false
 * @returns The new parent node
 */
function splitParentNode(node: Node, splitBefore: boolean): Node | null;

    /**
 * Split parent node by a balanced node range
 * @param nodes The nodes to split from. If only one node is passed, split it from all its siblings.
 * If two or nodes are passed, will split before the first one and after the last one, all other nodes will be ignored
 * @returns The parent node of the given node range if the given nodes are balanced, otherwise null
 */
function splitBalancedNodeRange(nodes: Node | Node[]): Node | null;

    /**
 * Removes the node and keep all children in place, return the parentNode where the children are attached
 * @param node the node to remove
 */
function unwrap(node: Node): Node | null;

    /**
 * Wrap all the node with html and return the wrapped node, and put the wrapper node under the parent of the first node
 * @param nodes The node or node array to wrap
 * @param wrapper The wrapper HTML tag name
 * @returns The wrapper element
 */
function wrap<T extends keyof HTMLElementTagNameMap>(nodes: Node | Node[], wrapper?: T): HTMLElementTagNameMap[T];

    /**
 * @deprecated
 * Wrap all the nodes with html and return the wrapped node, and put the wrapper node under the parent of the first node
 * @param nodes The node or node array to wrap
 * @param wrapper The wrapper HTML string, default value is DIV
 * @returns The wrapper element
 */
function wrap(nodes: Node | Node[], wrapper?: string): HTMLElement;

    /**
 * Wrap all the nodes with html and return the wrapped node, and put the wrapper node under the parent of the first node
 * @param nodes The node or node array to wrap
 * @param wrapper The wrapper HTML element, default value is a new DIV element
 * @returns The wrapper element
 */
function wrap(nodes: Node | Node[], wrapper?: HTMLElement): HTMLElement;

    /**
 * Wraps all the nodes with CreateElementData or an index of a known CreateElementData
 * @param nodes The nodes to wrap
 * @param wrapper The CreateElementData or an index of a known CreateElementData
 */
function wrap(nodes: Node | Node[], wrapper?: CreateElementData | KnownCreateElementDataIndex | CompatibleKnownCreateElementDataIndex): HTMLElement;

    /**
 * This walks forwards DOM tree to get next meaningful node
 * @param rootNode Root node to scope the leaf sibling node
 * @param startNode current node to get sibling node from
 * @param skipTags (Optional) tags that child elements will be skipped
 */
function getNextLeafSibling(rootNode: Node, startNode: Node, skipTags?: string[]): Node | null;

    /**
 * This walks backwards DOM tree to get next meaningful node
 * @param rootNode Root node to scope the leaf sibling node
 * @param startNode current node to get sibling node from
 * @param skipTags (Optional) tags that child elements will be skipped
 */
function getPreviousLeafSibling(rootNode: Node, startNode: Node, skipTags?: string[]): Node | null;

    /**
 * Get the first meaningful leaf node
 * @param rootNode Root node to get leaf node from
 */
function getFirstLeafNode(rootNode: Node): Node | null;

    /**
 * Get the last meaningful leaf node
 * @param rootNode Root node to get leaf node from
 */
function getLastLeafNode(rootNode: Node): Node | null;

    /**
 * Split a text node into two parts by an offset number, and return one of them
 * @param textNode The text node to split
 * @param offset The offset number to split at
 * @param returnFirstPart True to return the first part, then the passed in textNode will become the second part.
 * Otherwise return the second part, and the passed in textNode will become the first part
 */
function splitTextNode(textNode: Text, offset: number, returnFirstPart: boolean): Text;

    /**
 * A ClientRect of all 0 is possible. i.e. chrome returns a ClientRect of 0 when the cursor is on an empty p
 * We validate that and only return a rect when the passed in ClientRect is valid
 */
function normalizeRect(clientRect: DOMRect): Rect | null;

    /**
 * Check if the given object is instance of the target type
 * @param obj Object to check
 * @param typeName Target type name
 */
function safeInstanceOf<T extends keyof W, W extends TargetWindow = TargetWindow>(obj: any, typeName: T): obj is W[T];

    /**
 * Read a file object and invoke a callback function with the data url of this file
 * @param file The file to read
 * @param callback the callback to invoke with data url of the file.
 * If fail to read, dataUrl will be null
 */
function readFile(file: File, callback: (dataUrl: string | null) => void): void;

    /**
 * Get innerHTML of the given node
 * @param node The DOM node to get innerHTML from
 */
function getInnerHTML(node: HTMLElement | DocumentFragment): string;

    /**
 * Set text color or background color to the given element
 * @param element The element to set color to
 * @param color The color to set, it can be a string of color name/value or a ModeIndependentColor object
 * @param isBackgroundColor Whether set background color or text color
 * @param isDarkMode Whether current mode is dark mode. @default false
 * @param shouldAdaptTheFontColor Whether the font color needs to be adapted to be visible in a dark or bright background color. @default false
 * @param darkColorHandler A dark color handler object. This is now required.
 * We keep it optional only for backward compatibility. If it is not passed, color will not be set.
 */
function setColor(element: HTMLElement, color: string | ModeIndependentColor, isBackgroundColor: boolean, isDarkMode?: boolean, shouldAdaptTheFontColor?: boolean, darkColorHandler?: DarkColorHandler | null): void;

    /**
 * A wrapper function of Element.matches
 * @param element The element to match
 * @param selector The selector to match
 */
function matchesSelector(element: Node, selector: string): boolean;

    /**
 * Create DOM element from the given CreateElementData
 * @param elementData The CreateElementData or an index of a known CreateElementData used for creating this element
 * @param document The document to create the element from
 * @returns The root DOM element just created
 */
function createElement(elementData: CreateElementData | KnownCreateElementDataIndex | CompatibleKnownCreateElementDataIndex, document: Document): Element | null;

    /**
 * All known CreateElementData used by roosterjs to create elements
 */
const KnownCreateElementData: Record<KnownCreateElementDataIndex, CreateElementData>;

    /**
 * Replace all child nodes of the given target node to the child nodes of source node.
 * @param target Target node, all child nodes of this node will be removed if keepExistingChildren is not set to true
 * @param source (Optional) source node, all child nodes of this node will be move to target node
 * @param keepExistingChildren (Optional) When set to true, all existing child nodes of target will be kept
 */
function moveChildNodes(target: Node, source?: Node, keepExistingChildren?: boolean): void;

    /**
 * Get the intersected Rect of elements provided
 *
 * @example
 * The result of the following Elements Rects would be:
    {
        top: Element2.top,
        bottom: Element1.bottom,
        left: Element2.left,
        right: Element2.right
    }
    +-------------------------+
    | Element 1               |
    |   +-----------------+   |
    |   | Element2        |   |
    |   |                 |   |
    |   |                 |   |
    +-------------------------+
        |                 |
        +-----------------+

 * @param elements Elements to use.
 * @param additionalRects additional rects to use
 * @returns If the Rect is valid return the rect, if not, return null.
 */
function getIntersectedRect(elements: HTMLElement[], additionalRects?: Rect[]): Rect | null;

    /**
 * Checks if node1 is after node2
 * @param node1 The node to check if it is after another node
 * @param node2 The node to check if another node is after this one
 * @returns True if node1 is after node2, otherwise false
 */
function isNodeAfter(node1: Node, node2: Node): boolean;

    /**
 * Parse color string to r/g/b value.
 * If the given color is not in a recognized format, return null
 */
function parseColor(color: string): [number, number, number] | null;

    /**
 * A virtual table class, represent an HTML table, by expand all merged cells to each separated cells
 */
class VTable {
    /**
     * The HTML table object
     */
    table: HTMLTableElement;
    /**
     * Virtual cells
     */
    cells: VCell[][] | null;
    /**
     * Current row index
     */
    row: number | undefined;
    /**
     * Current column index
     */
    col: number | undefined;
    /**
     * Current format of the table
     */
    formatInfo: Required<TableFormat> | null;
    private trs;
    private tableSelection;
    /**
     * Create a new instance of VTable object using HTML TABLE or TD node
     * @param node The HTML Table or TD node
     * @param normalizeSize Whether table size needs to be normalized
     * @param zoomScale When the table is under a zoomed container, pass in the zoom scale here
     */
    constructor(node: HTMLTableElement | HTMLTableCellElement, normalizeSize?: boolean, zoomScale?: number | SizeTransformer);
    /**
     * Selected range of cells with the coordinates of the first and last cell selected.
     */
    get selection(): TableSelection | null;
    set selection(value: TableSelection | null);
    /**
     * Write the virtual table back to DOM tree to represent the change of VTable
     * @param skipApplyFormat Do not reapply table format when write back. Only use this parameter when you are pretty sure there is no format or table structure change during the process.
     * @param darkColorHandler An object to handle dark background colors, if not passed the cell background color will not be set
     */
    writeBack(skipApplyFormat?: boolean, darkColorHandler?: DarkColorHandler | null): void;
    private recalculateCellHeight;
    /**
     * Apply the given table format to this virtual table
     * @param format Table format to apply
     */
    applyFormat(format: Partial<TableFormat>): void;
    /**
     * Remove the cellShade dataset to apply a new style format at the cell.
     * @param cells
     */
    private deleteCellShadeDataset;
    /**
     * Edit table with given operation.
     * @param operation Table operation
     */
    edit(operation: TableOperation | CompatibleTableOperation): void;
    setAlignmentToSelectedCells(firstRow: number, lastRow: number, firstColumn: number, lastColumn: number, alignmentType: string, isVertical?: boolean): void;
    private mergeCells;
    private isEmptyCell;
    private mergeCellContents;
    /**
     * Loop each cell of current column and invoke a callback function
     * @param callback The callback function to invoke
     */
    forEachCellOfCurrentColumn(callback: (cell: VCell, row: VCell[], i: number) => any): void;
    /**
     * Loop each table cell and get all the cells that share the same border from one side
     * The result is an array of table cell elements
     * @param borderPos The position of the border
     * @param getLeftCells Get left-hand-side or right-hand-side cells of the border
     *
     * Example, consider having a 3 by 4 table as below with merged and split cells
     *
     *     | 1 | 4 | 7 | 8 |
     *     |   5   |   9   |
     *     |   3   |   10  |
     *
     *  input => borderPos: the 3rd border, getLeftCells: true
     *  output => [4, 5, 3]
     *
     *  input => borderPos: the 3rd border, getLeftCells: false
     *  output => [7, 9, 10]
     *
     *  input => borderPos: the 2nd border, getLeftCells: true
     *  output => [1]
     *
     *  input => borderPos: the 2nd border, getLeftCells: false
     *  output => [4]
     */
    getCellsWithBorder(borderPos: number, getLeftCells: boolean): HTMLTableCellElement[];
    /**
     * Loop each cell of current row and invoke a callback function
     * @param callback The callback function to invoke
     */
    forEachCellOfCurrentRow(callback: (cell: VCell, i: number) => any): void;
    /**
     * Get a table cell using its row and column index. This function will always return an object
     * even if the given indexes don't exist in table.
     * @param row The row index
     * @param col The column index
     */
    getCell(row: number, col: number): VCell;
    /**
     * Get current HTML table cell object. If the current table cell is a virtual expanded cell, return its root cell
     */
    getCurrentTd(): HTMLTableCellElement | null;
    /**
     * Get the Table Cell in a provided coordinate
     * @param row row of the cell
     * @param col column of the cell
     */
    getTd(row: number | undefined, col: number | undefined): HTMLTableCellElement | null;
    private forEachCellOfColumn;
    private forEachCellOfRow;
    private recalculateSpans;
    private countSpanLeft;
    private countSpanAbove;
    private normalizeEmptyTableCells;
    normalizeTableCellSize(zoomScale?: number | SizeTransformer): void;
    private normalizeSize;
}

    /**
 * Check if the whole table is selected
 * @param vTable VTable to check whether all cells are selected
 * @param selection Table selection with first cell selected and last cell selected coordinates.
 * @returns
 */
function isWholeTableSelected(vTable: VTable, selection: TableSelection): boolean;

    /**
 * Represent a bullet or a numbering list
 *
 * @example
 * A VList is a logical representation of list items, it contains an item array with node and list type stack.
 * e.g. We have a list like this
 * ```html
 * <ol>
 *   <li>item 1</li>
 *   <li>item 2</li>
 *   <ul>
 *     <li>item 2.1</li>
 *     <li>item 2.2</li>
 *   <ul>
 * </ol>
 * ```
 *
 * A VList of this list will be like this:
 * ```javascript
 * {
 *   rootList: (OL node),
 *   items: [{
 *       node: (LI node with 'item 1'),
 *       listTypes: [null, OL],
 *     }, {
 *       node: (LI node with 'item 2'),
 *       listTypes: [null, OL],
 *     }, {
 *       node: (LI node with 'item 2.1),
 *       listTypes: [null, OL, UL],
 *     }, {
 *       node: (LI node with 'item 2.2'),
 *       listTypes: [null, OL, UL],
 *     }
 *   ]
 * }
 * ```
 *
 * When we want to outdent item 2.1, we just need to remove the last "UL" from listTypes of item 2.1, then
 * the writeBack() function will handle everything related to DOM change
 */
class VList {
    rootList: HTMLOListElement | HTMLUListElement;
    readonly items: VListItem[];
    /**
     * Create a new instance of VList class
     * @param rootList The root list element, can be either OL or UL tag
     */
    constructor(rootList: HTMLOListElement | HTMLUListElement);
    /**
     * Check if this list contains the given node
     * @param node The node to check
     */
    contains(node: Node): boolean;
    /**
     * Get list number of the last item in this VList.
     * If there is no order list item, result will be undefined
     */
    getLastItemNumber(): number | undefined;
    /**
     * Write the result back into DOM tree
     * After that, this VList becomes unavailable because we set this.rootList to null
     *
     * @param shouldReuseAllAncestorListElements Optional - defaults to false.
     * @param disableListChain Whether we want to disable list chain functionality. @default false
     */
    writeBack(shouldReuseAllAncestorListElements?: boolean, disableListChain?: boolean): void;
    /**
     * Sets the New List Start Property, that is going to be used to create a new List in the WriteBack function
     * @param separator The HTML element that indicates when to split the VList
     * @param startNumber The start number of the new List
     */
    split(separator: HTMLElement, startNumber: number): void;
    /**
     * Set indentation of the given range of this list
     * @param start Start position to operate from
     * @param end End position to operate to
     * @param indentation Indent or outdent
     */
    setIndentation(start: NodePosition, end: NodePosition, indentation: Indentation | CompatibleIndentation): void;
    /**
     * Outdent the give range of this list
     * @param start Start position to operate from
     * @param end End position to operate to
     * @param indentation Specify to outdent
     * @param softOutdent (Optional) True to make the item to by dummy (no bullet or number) if the item is not dummy,
     * otherwise outdent the item
     * @param preventItemRemoval (Optional) True to prevent the indentation to remove the bullet when outdenting a first
     * level list item, by default is false
     */
    setIndentation(start: NodePosition, end: NodePosition, indentation: Indentation.Decrease | CompatibleIndentation.Decrease, softOutdent?: boolean, preventItemRemoval?: boolean): void;
    /**
     * Set alignment of the given range of this list
     * @param start Start position to operate from
     * @param end End position to operate to
     * @param alignment Align items left, center or right
     */
    setAlignment(start: NodePosition, end: NodePosition, alignment: Alignment | CompatibleAlignment): void;
    /**
     * Change list type of the given range of this list.
     * If some of the items are not real list item yet, this will make them to be list item with given type
     * If all items in the given range are already in the type to change to, this becomes an outdent operation
     * @param start Start position to operate from
     * @param end End position to operate to
     * @param targetType Target list type
     */
    changeListType(start: NodePosition, end: NodePosition, targetType: ListType | CompatibleListType): void;
    /**
     * Change list style of the given range of this list.
     * If some of the items are not real list item yet, this will make them to be list item with given style
     * @param orderedStyle The style of ordered list
     * @param unorderedStyle The style of unordered list
     */
    setListStyleType(orderedStyle?: NumberingListType | CompatibleNumberingListType, unorderedStyle?: BulletListType | CompatibleBulletListType): void;
    /**
     * Append a new item to this VList
     * @param node node of the item to append. If it is not wrapped with LI tag, it will be wrapped
     * @param type Type of this list item, can be ListType.None
     */
    appendItem(node: Node, type: ListType | CompatibleListType): void;
    /**
     * Merge the given VList into current VList.
     * - All list items will be removed from the given VList and added into this list.
     * - The root node of the given VList will be removed from DOM tree
     * - If there are orphan items in the given VList, they will be merged into the last item
     *   of this list if any.
     * @param list The vList to merge from
     */
    mergeVList(list: VList): void;
    /**
     * Get the index of the List Item in the current List
     * If the root list is:
     * Ordered list, the listIndex start count is going to be the start property of the OL - 1,
     * @example For example if we want to find the index of Item 2 in the list below, the returned index is going to be 6
     *  * ```html
     * <ol start="5">
     *   <li>item 1</li>
     *   <li>item 2</li> <!-- Node to find -->
     *   <li>item 3</li>
     * </ol>
     * ```
     * Unordered list, the listIndex start count starts from 0
     * @example For example if we want to find the index of Item 2 in the list below, the returned index is going to be 2
     * ```html
     * <ul>
     *   <li>item 1</li>
     *   <li>item 2</li> <!-- Node to find -->
     *   <li>item 3</li>
     * </ul>
     * ```
     * @param input List item to find in the root list
     */
    getListItemIndex(input: Node): number;
    /**
     * Get the Start property of the root list of this VList
     * @returns Start number of the list
     */
    getStart(): number | undefined;
    private findListItems;
    private populateItems;
}

    /**
 * !!! Never directly create instance of this class. It should be created within VList class !!!
 *
 * Represent a list item.
 *
 * A list item is normally wrapped using a LI tag. But this class is only a logical item,
 * it can be a LI tag, or another other type of node which means it is actually not a list item.
 * That can happen after we do "outdent" on a 1-level list item, then it becomes not a list item.
 */
class VListItem {
    private listTypes;
    private node;
    private dummy;
    private newListStart;
    /**
     * Construct a new instance of VListItem class
     * @param node The DOM node for this item
     * @param listTypes An array represents list types of all parent and current level.
     * Skip this parameter for a non-list item.
     */
    constructor(node: Node, ...listTypes: (ListType.Ordered | ListType.Unordered | CompatibleListType.Ordered | CompatibleListType.Unordered)[]);
    /**
     * Get type of current list item
     */
    getListType(): ListType | CompatibleListType;
    /**
     * Get the levels of this list item.
     */
    getLevel(): number;
    /**
     * Get DOM node of this list item
     */
    getNode(): HTMLLIElement;
    /**
     * Get the Start Number of the new List
     */
    getNewListStart(): number | undefined;
    /**
     * Check if a given node is contained by this list item
     * @param node The node to check
     */
    contains(node: Node): boolean;
    /**
     * Check if this item is a dummy item.
     * A dummy item is also represented by LI tag, but it won't render a bullet (for Unordered list) or a number (for Ordered list)
     * normally it has CSS style display set to a value other than "list-item"
     */
    isDummy(): boolean;
    /**
     * @deprecated Always return false
     */
    isOrphanItem(): boolean;
    /**
     * @deprecated
     */
    canMerge(item: VListItem): boolean;
    /**
     * @deprecated
     */
    mergeItems(items: VListItem[]): void;
    /**
     * Indent this item
     * If this is not an list item, it will be no op
     */
    indent(): void;
    /**
     * Outdent this item
     * If this item is already not an list item, it will be no op
     * @param preventItemRemoval Whether prevent the list item to be removed for the listItem by default false
     */
    outdent(preventItemRemoval?: boolean): void;
    /**
     * Add negative margin to the List item
     */
    addNegativeMargins(): void;
    /**
     * Change list type of this item
     * @param targetType The target list type to change to
     */
    changeListType(targetType: ListType | CompatibleListType): void;
    /**
     * Set whether the item is a dummy item
     * @param isDummy Whether the item is a dummy item
     */
    setIsDummy(isDummy: boolean): void;
    /**
     * Set the start Number of the new list
     * @param isDummy Whether the item is a dummy item
     */
    setNewListStart(startNumber: number): void;
    /**
     * Apply the list style type
     * @param rootList the vList that receives the style
     * @param index the list item index
     */
    applyListStyle(rootList: HTMLOListElement | HTMLUListElement, index: number): void;
    /**
     * Write the change result back into DOM
     * @param listStack current stack of list elements
     * @param originalRoot Original list root element. It will be reused when write back if possible
     * @param shouldReuseAllAncestorListElements Optional - defaults to false. If true, only make
     *              sure the direct parent of this list matches the list types when writing back.
     */
    writeBack(listStack: Node[], originalRoot?: HTMLOListElement | HTMLUListElement, shouldReuseAllAncestorListElements?: boolean): void;
    /**
     * Get the index of how deep is the current node parent list inside of the original root list.
     * @example In the following structure this function would return 2
     * ```html
     *  <ol> <!-- original Root -->
     *      <ol>
     *          <ol>
     *              <li></li> <!-- this.node  -->
     *          </ol>
     *      </ol>
     *  </ol>
     * ```
     * @param originalRoot The root list
     * @returns -1  if the node does not have parent element or if original root was not provided,
     *              else, how deep is the parent element inside of the original root.
     */
    private getDeepChildIndex;
}

    /**
 * Create a VList object from the given region.
 * @param region The region to get VList from
 * @param includeSiblingLists True to also try get lists before and after the selection and merge them together,
 * false to only include the list for the selected blocks
 * @param startNode (Optional) When specified, try get VList which will contain this node.
 * If not specified, get VList from selection of this region
 */
function createVListFromRegion(region: Region, includeSiblingLists?: boolean, startNode?: Node): VList | null;

    /**
 * Represent a chain of list nodes.
 * A chain of lists is a virtual link of lists that have continuous numbers, when editor one of them,
 * all others should also be updated in order to main the list number to be continuous.
 */
class VListChain {
    private region;
    private name;
    private lastNumber;
    private lastNumberBeforeCursor;
    /**
     * Create an array of VListChain from current region in editor
     * @param region The region to create VListChain from
     * @param currentNode Optional current node, used for mark lists that are after this node
     * @param nameGenerator Used by test code only
     */
    static createListChains(region: RegionBase | RegionBase[], currentNode?: Node, nameGenerator?: () => string): VListChain[];
    /**
     * Check if a list with the given start number can be appended next to the last list before cursor
     * @param startNumber The start number of the new list
     */
    canAppendAtCursor(startNumber: number): boolean;
    /**
     * Create a VList to wrap the block of the given node, and append to current chain
     * @param container The container node to create list at
     * @param startNumber Start number of the new list
     */
    createVListAtBlock(container: Node, startNumber: number): VList | null;
    /**
     * After change the lists, commit the change to all lists in this chain to update the list number,
     * and clear the temporary dataset values added to list node
     * @param shouldReuseAllAncestorListElements Whether we can parent list item (OL/UL) even if its list type does not match the previous one. @default false
     * @param disableListChain Whether we want to disable list chain functionality, so splitted list will always restart its number from 1 @default false
     */
    commit(shouldReuseAllAncestorListElements?: boolean, disableListChain?: boolean): void;
    /**
     * Construct a new instance of VListChain class
     * @param editor Editor object
     */
    private constructor();
    /**
     * Check if the given list node is can be appended into current list chain
     * @param list The list node to check
     */
    private canAppendToTail;
    /**
     * Append the given list node into this VListChain
     * @param list The list node to append
     * @param isAfterCurrentNode Whether this list is after current node
     */
    private append;
    private applyChainName;
    private getLists;
}

    /**
 * Convert decimal numbers into english alphabet letters
 * @param decimal The decimal number that needs to be converted
 * @param isLowerCase if true the roman value will appear in lower case
 * @returns
 */
function convertDecimalsToAlpha(decimal: number, isLowerCase?: boolean): string;

    /**
 * Convert decimal numbers into roman numbers
 * @param decimal The decimal number that needs to be converted
 * @param isLowerCase if true the roman value will appear in lower case
 * @returns
 */
function convertDecimalsToRoman(decimal: number, isLowerCase?: boolean): string;

    /**
 * Set the Style of a List Item provided, with the styles that the inline child elements have
 * If the child inline elements have different styles, it will not modify the styles of the list item
 * @param element the LI Element to set the styles
 * @param styles The styles that should be applied to the element.
 * @param isCssStyle True means the given styles are CSS style names, false means they are HTML attributes @default true
 */
function setListItemStyle(element: HTMLLIElement, styles: string[], isCssStyle?: boolean): void;

    /**
 * Get the format info of a table
 * If the table does not have a info saved, it will be retrieved from the css styles
 * @param table The table that has the info
 */
function getTableFormatInfo(table: HTMLTableElement): Required<TableFormat> | null;

    /**
 * Add metadata to a cell
 * @param cell The table cell to add the metadata
 * @param format The format of the table
 */
function saveTableCellMetadata(cell: HTMLTableCellElement, format: TableCellMetadataFormat): void;

    /**
 * Get regions impacted by the given range under the root node
 * @param root Root node to get regions from
 * @param range A selection range. Regions will be created according to this range. Each region will be
 * fully or partially covered by this range.
 * @param type Type of region. Currently we only support TABLE region.
 */
function getRegionsFromRange(root: HTMLElement, range: Range, type: RegionType | CompatibleRegionType): Region[];

    /**
 * Get all block elements covered by the selection under this region
 * @param regionBase The region to get block elements from
 * @param createBlockIfEmpty When set to true, a new empty block element will be created if there is not
 * any blocks in the region. Default value is false
 * @param deprecated Deprecated parameter, not used
 */
function getSelectedBlockElementsInRegion(regionBase: RegionBase, createBlockIfEmpty?: boolean, deprecated?: boolean): BlockElement[];

    /**
 * Collapse nodes within this region to their common ancestor node under this region
 * @param region The region to collapse nodes in.
 * @param nodesOrBlockElements Nodes or block elements to collapse. When take BlockElement[] as input,
 * start node of the first BlockElement and end node of the last BlockElement will be used as the nodes.
 * All nodes not contained by the given region will be ignored.
 */
function collapseNodesInRegion(region: RegionBase, nodesOrBlockElements: Node[] | BlockElement[]): Node[];

    /**
 * Check if a given node is contained by the given region
 * @param region The region to check from
 * @param node The node or block element to check
 */
function isNodeInRegion(region: RegionBase, node: Node): boolean;

    /**
 * Get the selection range in the given region.
 * The original range can cover multiple regions, this function will narrow the original selection
 * of a region into current region
 * @param regionBase The region to get range from
 */
function getSelectionRangeInRegion(regionBase: RegionBase): Range | null;

    /**
 * Merge a BlockElement of given node after another node
 * @param region Region to operate in
 * @param refNode The node to merge after
 * @param targetNode The node of target block element
 */
function mergeBlocksInRegion(region: RegionBase, refNode: Node, targetNode: Node): void;

    /**
 * Represent a position in DOM tree by the node and its offset index
 */
class Position implements NodePosition  {
    private readonly isFromEndOfRange?;
    readonly node: Node;
    readonly element: HTMLElement;
    readonly offset: number;
    readonly isAtEnd: boolean;
    /**
     * Clone and validate a position from existing position.
     * If the given position has invalid offset, this function will return a corrected value.
     * @param position The original position to clone from
     */
    constructor(position: NodePosition);
    /**
     * Create a Position from node and an offset number
     * @param node The node of this position
     * @param offset Offset of this position
     * @param isFromEndOfRange Whether this position is created from end of a range. An position
     * created from end of range has different behavior when normalize, it will use the child node
     * before current position if any as a deeper level node and set isAtEnd to true.
     */
    constructor(node: Node, offset: number, isFromEndOfRange?: boolean);
    /**
     * Create a Position from node and a type of position
     * @param node The node of this position
     * @param positionType Type of the position, can be Begin, End, Before, After
     */
    constructor(node: Node, positionType: PositionType | CompatiblePositionType);
    /**
     * Normalize this position to the leaf node, return the normalize result.
     * If current position is already using leaf node, return this position object itself
     */
    normalize(): NodePosition;
    /**
     * Check if this position is equal to the given position
     * @param position The position to check
     */
    equalTo(position: NodePosition): boolean;
    /**
     * Checks if this position is after the given position
     */
    isAfter(position: NodePosition): boolean;
    /**
     * Move this position with offset, returns a new position with a valid offset in the same node
     * @param offset Offset to move with
     */
    move(offset: number): Position;
    /**
     * Get start position of the given Range
     * @param range The range to get position from
     */
    static getStart(range: Range): Position;
    /**
     * Get end position of the given Range
     * @param range The range to get position from
     */
    static getEnd(range: Range): Position;
}

    /**
 * Create a range around the given node(s)
 * @param startNode The start node to create range from
 * @param endNode The end node to create range from. If specified, the range will start before startNode and
 * end after endNode, otherwise, the range will start before and end after the start node
 * @returns A range start before the given node and end after the given node
 */
function createRange(startNode: Node, endNode?: Node): Range;

    /**
 * Create a collapsed range at the given node and offset
 * @param node The container node of the range
 * @param offset The offset of the range, can be a number or value of PositionType
 * @returns A range at the given node and offset
 */
function createRange(node: Node, offset: number | PositionType): Range;

    /**
 * Create a range with the given start/end container node and offset
 * @param startNode The start container node of the range
 * @param startOffset The start offset of the range
 * @param endNode The end container node of the range
 * @param endOffset The end offset of the range
 * @returns A range at the given start/end container node and offset
 */
function createRange(startNode: Node, startOffset: number | PositionType, endNode: Node, endOffset: number | PositionType): Range;

    /**
 * Create a range under the given rootNode with start and end selection paths
 * @param rootNode The root node that the selection paths start from
 * @param startPath The selection path of the start position of the range
 * @param endPath The selection path of the end position of the range
 * @returns A range with the given start and end selection paths
 */
function createRange(rootNode: Node, startPath: number[], endPath?: number[]): Range;

    /**
 * Create a range with the start and end position
 * @param startPosition The start position of the range
 * @param endPosition The end position of the range, if not specified, the range will be collapsed at start position
 * @returns A range start at startPosition, end at endPosition, or startPosition when endPosition is not specified
 */
function createRange(startPosition: NodePosition, endPosition?: NodePosition): Range;

    /**
 * Get bounding rect of this position
 * @param position The position to get rect from
 */
function getPositionRect(position: NodePosition): Rect | null;

    /**
 * Check if this position is at beginning of the given node.
 * This will return true if all nodes between the beginning of target node and the position are empty.
 * @param position The position to check
 * @param targetNode The node to check
 * @returns True if position is at beginning of the node, otherwise false
 */
function isPositionAtBeginningOf(position: NodePosition, targetNode: Node | null): boolean;

    /**
 * Get path of the given selection range related to the given rootNode
 * @param rootNode The root node where the path start from
 * @param range The range of selection
 */
function getSelectionPath(rootNode: Node, range: Range | null): SelectionPath | null;

    /**
 * Get inner Html of a root node with a selection path which can be used for restore selection.
 * The result string can be used by setHtmlWithSelectionPath() to restore the HTML and selection.
 * @param rootNode Root node to get inner Html from
 * @param range The range of selection. If pass null, no selection path will be added
 * @returns Inner HTML of the root node, followed by HTML comment contains selection path if the given range is valid
 */
function getHtmlWithSelectionPath(rootNode: HTMLElement | DocumentFragment, range: Range | null): string;

    /**
 * @deprecated Use setHtmlWithMetadata instead
 * Restore inner HTML of a root element from given html string. If the string contains selection path,
 * remove the selection path and return a range represented by the path
 * @param root The root element
 * @param html The HTML to restore
 * @param trustedHTMLHandler An optional trusted HTML handler to convert HTML string to security string
 * @returns A selection range if the html contains a valid selection path, otherwise null
 */
function setHtmlWithSelectionPath(rootNode: HTMLElement, html: string, trustedHTMLHandler?: TrustedHTMLHandler): Range | null;

    /**
 * Restore inner HTML of a root element from given html string. If the string contains metadata,
 * remove it from DOM tree and return the metadata
 * @param root The root element
 * @param html The HTML to restore
 * @param trustedHTMLHandler An optional trusted HTML handler to convert HTML string to security string
 * @returns Content metadata if any, or undefined
 */
function setHtmlWithMetadata(rootNode: HTMLElement, html: string, trustedHTMLHandler?: TrustedHTMLHandler): ContentMetadata | undefined;

    /**
 * Extract content metadata from DOM tree
 * @param rootNode Root of the DOM tree
 * @returns If there is a valid content metadata node in the give DOM tree, return this metadata object, otherwise undefined
 */
function extractContentMetadata(rootNode: HTMLElement): ContentMetadata | undefined;

    /**
 * Add the given range into selection of the given document
 * @param range The range to select
 * @param skipSameRange When set to true, do nothing if the given range is the same with current selection,
 * otherwise it will always remove current selection range and set to the given one.
 * This parameter is always treat as true in Edge to avoid some weird runtime exception.
 */
function addRangeToSelection(range: Range, skipSameRange?: boolean): void;

    /**
 * Add a new snapshot to the given snapshots data structure
 * @param snapshots The snapshots data structure to add new snapshot into
 * @param html The snapshot HTML to add
 * @param isAutoCompleteSnapshot Whether this is a snapshot before auto complete action
 */
function addSnapshot(snapshots: Snapshots<string>, html: string, isAutoCompleteSnapshot: boolean): void;

    /**
 * Add a new snapshot to the given snapshots data structure
 * @param snapshots The snapshots data structure to add new snapshot into
 * @param snapshot The generic snapshot object to add
 * @param isAutoCompleteSnapshot Whether this is a snapshot before auto complete action
 * @param getLength A callback function to calculate length of the snapshot
 * @param isSame A callback function to check if the given snapshots are the same
 */
function addSnapshot<T>(snapshots: Snapshots<T>, snapshot: T, isAutoCompleteSnapshot: boolean, getLength: (snapshot: T) => number, isSame: (snapshot1: T, snapshot2: T) => boolean): void;

    /**
 * Add a new snapshot to the given snapshots data structure
 * @param snapshots The snapshots data structure to add new snapshot into
 * @param snapshot The snapshot object to add
 * @param isAutoCompleteSnapshot Whether this is a snapshot before auto complete action
 */
function addSnapshotV2(snapshots: Snapshots<Snapshot>, snapshot: Snapshot, isAutoCompleteSnapshot: boolean): void;

    /**
 * Check whether can move current snapshot with the given step
 * @param snapshots The snapshots data structure to check
 * @param step The step to check, can be positive, negative or 0
 * @returns True if can move current snapshot with the given step, otherwise false
 */
function canMoveCurrentSnapshot<T = string>(snapshots: Snapshots<T>, step: number): boolean;

    /**
 * Clear all snapshots after the current one
 * @param snapshots The snapshots data structure to clear
 */
function clearProceedingSnapshots(snapshots: Snapshots<string>): void;

    /**
 * Clear all snapshots after the current one
 * @param snapshots The snapshots data structure to clear
 */
function clearProceedingSnapshots<T>(snapshots: Snapshots<T>, getLength: (snapshot: T) => number): void;

    /**
 * Clear all snapshots after the current one
 * @param snapshots The snapshots data structure to clear
 */
function clearProceedingSnapshotsV2(snapshots: Snapshots<Snapshot>): void;

    /**
 * Move current snapshot with the given step if can move this step. Otherwise no action and return null
 * @param snapshots The snapshots data structure to move
 * @param step The step to move
 * @returns If can move with the given step, returns the snapshot after move, otherwise null
 */
function moveCurrentSnapshot<T = string>(snapshots: Snapshots<T>, step: number): T | null;

    /**
 * @deprecated
 * For backward compatibility only
 */
const moveCurrentSnapsnot: typeof moveCurrentSnapshot;

    /**
 * Create initial snapshots
 * @param maxSize max size of all snapshots
 */
function createSnapshots<T = string>(maxSize: number): Snapshots<T>;

    /**
 * Whether there is a snapshot added before auto complete and it can be undone now
 */
function canUndoAutoComplete<T = string>(snapshots: Snapshots<T>): boolean;

    /**
 * HTML sanitizer class provides two features:
 * 1. Convert global CSS to inline CSS
 * 2. Sanitize an HTML document, remove unnecessary/dangerous attribute/nodes
 */
class HtmlSanitizer {
    /**
     * @deprecated Use new HtmlSanitizer().convertGlobalCssToInlineCss() instead
     * Convert global CSS to inline CSS if any
     * @param html HTML source
     * @param additionalStyleNodes (Optional) additional HTML STYLE elements used as global CSS
     */
    static convertInlineCss(html: string, additionalStyleNodes?: HTMLStyleElement[]): string;
    /**
     * @deprecated Use new HtmlSanitizer().sanitize() instead
     * Sanitize HTML string, remove any unused HTML node/attribute/CSS.
     * @param html HTML source string
     * @param options Options used for this sanitizing process
     */
    static sanitizeHtml(html: string, options?: SanitizeHtmlOptions): string;
    private elementCallbacks;
    private styleCallbacks;
    private attributeCallbacks;
    private tagReplacements;
    private allowedAttributes;
    private allowedCssClassesRegex;
    private defaultStyleValues;
    private additionalPredefinedCssForElement;
    private additionalGlobalStyleNodes;
    private preserveHtmlComments;
    private unknownTagReplacement;
    /**
     * Construct a new instance of HtmlSanitizer
     * @param options Options for HtmlSanitizer
     */
    constructor(options?: HtmlSanitizerOptions);
    /**
     * @deprecated Use HtmlSanitizer.convertGlobalCssToInlineCss() and HtmlSanitizer.sanitize() instead
     * Sanitize HTML string
     * This function will do the following work:
     * 1. Convert global CSS into inline CSS
     * 2. Remove dangerous HTML tags and attributes
     * 3. Remove useless CSS properties
     * @param html The input HTML
     * @param convertInlineCssOnly Whether only convert inline css and skip html content sanitizing
     * @param currentStyles Current inheritable CSS styles
     */
    exec(html: string, convertCssOnly?: boolean, currentStyles?: StringMap): string;
    /**
     * Sanitize an HTML element, remove unnecessary or dangerous elements/attribute/CSS rules
     * @param rootNode Root node to sanitize
     * @param currentStyles Current CSS styles. Inheritable styles in the given node which has
     * the same value with current styles will be ignored.
     */
    sanitize(rootNode: Node, currentStyles?: StringMap): "" | undefined;
    /**
     * Convert global CSS into inline CSS
     * @param rootNode The HTML Document
     */
    convertGlobalCssToInlineCss(rootNode: ParentNode): void;
    private processNode;
    private preprocessCss;
    private processCss;
    private processAttributes;
    private processCssClass;
}

    /**
 * Get inheritable CSS style values from the given element
 * @param element The element to get style from
 */
function getInheritableStyles(element: HTMLElement | null): StringMap;

    /**
 * Create default value of HtmlSanitizerOptions with every property set
 */
function createDefaultHtmlSanitizerOptions(): Required<HtmlSanitizerOptions>;

    /**
 * Chain all callback for an attribute together
 * @param map The source callback map
 * @param name Name of the property to chain
 * @param newCallback A new callback to process the given name on the given map.
 * If the same property got multiple callbacks, the final return value will be the return
 * value of the latest callback
 */
function chainSanitizerCallback<TOriginalArgs extends any[], TChainedFn extends (...args: TOriginalArgs) => R, R>(map: Record<string, (...args: TOriginalArgs) => R>, name: string, newCallback: TChainedFn): void;

    /**
 * Commit information of an entity (type, isReadonly, id) into the wrapper node as CSS Classes
 * @param wrapper The entity wrapper element
 * @param type Entity type
 * @param isReadonly Whether this is a readonly entity
 * @param id Optional Id of the entity
 */
function commitEntity(wrapper: HTMLElement, type: string, isReadonly: boolean, id?: string): void;

    /**
 * Get Entity object from an entity root element
 * @param element The entity root element. If this element is not an entity root element,
 * it will return null
 */
function getEntityFromElement(element: HTMLElement): Entity | null;

    /**
 * Get a selector string for specified entity type and id
 * @param type (Optional) Type of entity
 * @param id (Optional) Id of entity
 */
function getEntitySelector(type?: string, id?: string): string;

    /**
 * @deprecated
 * Create a placeholder comment node for entity
 * @param entity The entity to create placeholder from
 * @returns A placeholder comment node as
 */
function createEntityPlaceholder(entity: Entity): HTMLElement;

    /**
 * Move content from a container into a new Document fragment, and try keep entities to be reusable by creating placeholder
 * for them in the document fragment.
 * If an entity is directly under root container, the whole entity can be reused and no need to move it at all.
 * If an entity is not directly under root container, it is still reusable, but it may need some movement.
 * In any case, entities will be replaced with a placeholder in the target document fragment.
 * We will use an entity map (the "entities" parameter) to save the map from entity id to its wrapper element.
 * @param root The root element
 * @param entities A map from entity id to entity wrapper element
 * @returns A new document fragment contains all the content and entity placeholders
 */
function moveContentWithEntityPlaceholders(root: HTMLDivElement, entities: Record<string, HTMLElement>): DocumentFragment;

    /**
 * Restore HTML content from a document fragment that may contain entity placeholders.
 * @param source Source document fragment that contains HTML content and entity placeholders
 * @param target Target container, usually to be editor root container
 * @param entities A map from entity id to entity wrapper, used for reusing existing DOM structure for entity
 * @param insertClonedNode When pass true, merge with a cloned copy of the nodes from source fragment rather than the nodes themselves @default false
 */
function restoreContentWithEntityPlaceholder(source: ParentNode, target: HTMLElement, entities: Record<string, HTMLElement | KnownEntityItem> | null, insertClonedNode?: boolean): void;

    /**
 * Gets the cached event data by cache key from event object if there is already one.
 * Otherwise, call getter function to create one, and cache it.
 * @param event The event object
 * @param key Cache key string, need to be unique
 * @param getter Getter function to get the object when it is not in cache yet
 */
function cacheGetEventData<T>(event: PluginEvent | null, key: string, getter: () => T): T;

    /**
 * Clear a cached object by its key from an event object
 * @param event The event object
 * @param key The cache key
 */
function clearEventDataCache(event: PluginEvent, key?: string): void;

    /**
 * Returns true when the event was fired from a modifier key, otherwise false
 * @param event The keyboard event object
 */
function isModifierKey(event: KeyboardEvent): boolean;

    /**
 * Returns true when the event was fired from a key that produces a character value, otherwise false
 * This detection is not 100% accurate. event.key is not fully supported by all browsers, and in some browsers (e.g. IE),
 * event.key is longer than 1 for num pad input. But here we just want to improve performance as much as possible.
 * So if we missed some case here it is still acceptable.
 * @param event The keyboard event object
 */
function isCharacterValue(event: KeyboardEvent): boolean;

    /**
 * Check if Ctrl key (Windows) or Meta key (Mac) is pressed for the given Event
 * @param event A Keyboard event or Mouse event object
 * @returns True if Ctrl key is pressed on Windows or Meta key is pressed on Mac
 */
function isCtrlOrMetaPressed(event: KeyboardEvent | MouseEvent): boolean;

    /**
 * Get CSS styles of a given element in name-value pair format
 * @param element The element to get styles from
 */
function getStyles(element: HTMLElement): Record<string, string>;

    /**
 * Set styles to an HTML element. If styles are empty, remove 'style' attribute
 * @param element The element to set styles
 * @param styles The styles to set, in name-value pair format
 */
function setStyles(element: HTMLElement, styles: Record<string, string>): void;

    /**
 * Removes the css important rule from some css properties
 * @param element The HTMLElement
 * @param styleProperties The css properties that important must be removed. Ex: ['background-color', 'background']
 */
function removeImportantStyleRule(element: HTMLElement, styleProperties: string[]): void;

    /**
 * Add global css styles
 * @param doc The document object
 * @param cssRule The css rule that must added to the selection
 * @param styleId The id of the style tag
 */
function setGlobalCssStyles(doc: Document, cssRule: string, styleId: string): void;

    /**
 * Remove a css rule style from a style sheet
 * @param doc The document object
 * @param styleId the ID of the style tag
 */
function removeGlobalCssStyle(doc: Document, styleId: string): void;

    /**
 *
 * @param root the contentDiv of the ditor
 * @param nodeToInsert the node to be inserted
 * @param position the position of the node to be inserted
 * @param range the range current or cached range of the editor
 * @returns the adjusted position of the inserted node
 */
function adjustInsertPosition(root: HTMLElement, nodeToInsert: Node, position: NodePosition, range: Range): NodePosition;

    /**
 * Delete selected content, and return the new position to select
 * @param core The EditorCore object.
 * @param range The range to delete
 */
function deleteSelectedContent(root: HTMLElement, range: Range): NodePosition | null;

    /**
 * get block element's text content.
 * @param rootNode Root node that the get the textContent of.
 * @returns text content of given text content.
 */
function getTextContent(rootNode: Node): string;

    /**
 * Validate the given object with a type definition object
 * @param input The object to validate
 * @param def The type definition object used for validation
 * @returns True if the object passed the validation, otherwise false
 */
function validate<T>(input: any, def: Definition<T>): input is T;

    /**
 * Create a number definition
 * @param isOptional Whether this property is optional
 * @param value Optional value of the number
 * @param minValue Optional minimum value
 * @param maxValue Optional maximum value
 * @param allowNull Allow the property to be null
 * @returns The number definition object
 */
function createNumberDefinition(isOptional?: boolean, value?: number, minValue?: number, maxValue?: number, allowNull?: boolean): NumberDefinition;

    /**
 * Create a boolean definition
 * @param isOptional  Whether this property is optional
 * @param value Optional expected boolean value
 * @param allowNull Allow the property to be null
 * @returns  The boolean definition object
 */
function createBooleanDefinition(isOptional?: boolean, value?: boolean, allowNull?: boolean): BooleanDefinition;

    /**
 * Create a string definition
 * @param isOptional  Whether this property is optional
 * @param value Optional expected string value
 * @param allowNull Allow the property to be null
 * @returns  The string definition object
 */
function createStringDefinition(isOptional?: boolean, value?: string, allowNull?: boolean): StringDefinition;

    /**
 * Create an array definition
 * @param itemDef Definition of each item of the related array
 * @param isOptional  Whether this property is optional
 * @param allowNull Allow the property to be null
 * @returns  The array definition object
 */
function createArrayDefinition<T>(itemDef: Definition<T>, isOptional?: boolean, minLength?: number, maxLength?: number, allowNull?: boolean): ArrayDefinition<T[]>;

    /**
 * Create an object definition
 * @param propertyDef Definition of each property of the related object
 * @param isOptional  Whether this property is optional
 * @param allowNull Allow the property to be null
 * @returns  The object definition object
 */
function createObjectDefinition<T extends Object>(propertyDef: ObjectPropertyDefinition<T>, isOptional?: boolean, allowNull?: boolean): ObjectDefinition<T>;

    /**
 * Get metadata object from an HTML element
 * @param element The HTML element to get metadata object from
 * @param definition The type definition of this metadata used for validate this metadata object.
 * If not specified, no validation will be performed and always return whatever we get from the element
 * @param defaultValue The default value to return if the retrieved object cannot pass the validation,
 * or there is no metadata object at all
 * @returns The strong-type metadata object if it can be validated, or null
 */
function getMetadata<T>(element: HTMLElement, definition?: Definition<T>, defaultValue?: T): T | null;

    /**
 * Set metadata object into an HTML element
 * @param element The HTML element to set metadata object to
 * @param metadata The metadata object to set
 * @param def An optional type definition object used for validate this metadata object.
 * If not specified, metadata will be set without validation
 * @returns True if metadata is set, otherwise false
 */
function setMetadata<T>(element: HTMLElement, metadata: T, def?: Definition<T>): boolean;

    /**
 * Remove metadata from the given element if any
 * @param element The element to remove metadata from
 * @param metadataKey The metadata key to remove, if none provided it will delete all metadata
 */
function removeMetadata<T>(element: HTMLElement, metadataKey?: string): void;

    /**
 * A type-safe wrapper for Array.prototype.push.apply()
 * @param mainArray The main array to push items into
 * @param itemsArray The items to push to main array
 */
function arrayPush<T>(mainArray: T[], itemsArray: T[]): void;

    /**
 * Provide a strong-typed version of Object.keys()
 * @param obj The source object
 * @returns Array of keys
 */
function getObjectKeys<T extends string | number | symbol>(obj: Record<T, any> | Partial<Record<T, any>>): T[];

    /**
 * Convert a named node map to an array
 * @param collection The map to convert
 */
function toArray(collection: NamedNodeMap): Attr[];

    /**
 * Convert a named node map to an array
 * @param collection The map to convert
 */
function toArray(collection: DataTransferItemList): DataTransferItem[];

    /**
 * Convert a collection to an array
 * @param collection The collection to convert
 */
function toArray<T extends Node>(collection: NodeListOf<T>): T[];

    /**
 * Convert a collection to an array
 * @param collection The collection to convert
 */
function toArray<T extends Element>(collection: HTMLCollectionOf<T>): T[];

    /**
 * Convert an array to an array.
 * This is to satisfy typescript compiler. For some cases the object can be a collection at runtime,
 * but the declaration is an array. e.g. ClipboardData.types
 * @param array The array to convert
 */
function toArray<T>(array: readonly T[]): T[];

    /**
 * This function tries to get the source of the Pasted content
 * @param event the before paste event
 * @param shouldConvertSingleImage Whether convert single image is enabled.
 * @returns The Type of pasted content, if no type found will return {KnownSourceType.Default}
 */
function getPasteSource(event: BeforePasteEvent, shouldConvertSingleImage: boolean): KnownPasteSourceType;

    /**
 * RoosterJs core editor class
 */
class Editor extends EditorBase<EditorCore, EditorOptions> {
    /**
     * Creates an instance of EditorBase
     * @param contentDiv The DIV HTML element which will be the container element of editor
     * @param options An optional options object to customize the editor
     */
    constructor(contentDiv: HTMLDivElement, options?: EditorOptions);
}

    /**
 * Base class of editor
 */
class EditorBase<TEditorCore extends EditorCore, TEditorOptions extends EditorOptions> implements IEditor  {
    private core;
    /**
     * Creates an instance of EditorBase
     * @param contentDiv The DIV HTML element which will be the container element of editor
     * @param options An optional options object to customize the editor
     */
    constructor(contentDiv: HTMLDivElement, options: TEditorOptions, coreCreator: CoreCreator<TEditorCore, TEditorOptions>);
    /**
     * Dispose this editor, dispose all plugins and custom data
     */
    dispose(): void;
    /**
     * Get whether this editor is disposed
     * @returns True if editor is disposed, otherwise false
     */
    isDisposed(): boolean;
    /**
     * Insert node into editor
     * @param node The node to insert
     * @param option Insert options. Default value is:
     *  position: ContentPosition.SelectionStart
     *  updateCursor: true
     *  replaceSelection: true
     *  insertOnNewLine: false
     * @returns true if node is inserted. Otherwise false
     */
    insertNode(node: Node, option?: InsertOption): boolean;
    /**
     * Delete a node from editor content
     * @param node The node to delete
     * @returns true if node is deleted. Otherwise false
     */
    deleteNode(node: Node): boolean;
    /**
     * Replace a node in editor content with another node
     * @param existingNode The existing node to be replaced
     * @param toNode node to replace to
     * @param transformColorForDarkMode (optional) Whether to transform new node to dark mode. Default is false
     * @returns true if node is replaced. Otherwise false
     */
    replaceNode(existingNode: Node, toNode: Node, transformColorForDarkMode?: boolean): boolean;
    /**
     * Get BlockElement at given node
     * @param node The node to create InlineElement
     * @returns The BlockElement result
     */
    getBlockElementAtNode(node: Node): BlockElement | null;
    contains(arg: Node | Range | null): boolean;
    queryElements(selector: string, scopeOrCallback?: QueryScope | CompatibleQueryScope | ((node: Node) => any), callback?: (node: Node) => any): HTMLElement[];
    /**
     * Collapse nodes within the given start and end nodes to their common ancestor node,
     * split parent nodes if necessary
     * @param start The start node
     * @param end The end node
     * @param canSplitParent True to allow split parent node there are nodes before start or after end under the same parent
     * and the returned nodes will be all nodes from start through end after splitting
     * False to disallow split parent
     * @returns When canSplitParent is true, returns all node from start through end after splitting,
     * otherwise just return start and end
     */
    collapseNodes(start: Node, end: Node, canSplitParent: boolean): Node[];
    /**
     * Check whether the editor contains any visible content
     * @param trim Whether trim the content string before check. Default is false
     * @returns True if there's no visible content, otherwise false
     */
    isEmpty(trim?: boolean): boolean;
    /**
     * Get current editor content as HTML string
     * @param mode specify what kind of HTML content to retrieve
     * @returns HTML string representing current editor content
     */
    getContent(mode?: GetContentMode | CompatibleGetContentMode): string;
    /**
     * Set HTML content to this editor. All existing content will be replaced. A ContentChanged event will be triggered
     * @param content HTML content to set in
     * @param triggerContentChangedEvent True to trigger a ContentChanged event. Default value is true
     */
    setContent(content: string, triggerContentChangedEvent?: boolean): void;
    /**
     * Insert HTML content into editor
     * @param HTML content to insert
     * @param option Insert options. Default value is:
     *  position: ContentPosition.SelectionStart
     *  updateCursor: true
     *  replaceSelection: true
     *  insertOnNewLine: false
     */
    insertContent(content: string, option?: InsertOption): void;
    /**
     * Delete selected content
     */
    deleteSelectedContent(): NodePosition | null;
    /**
     * Paste into editor using a clipboardData object
     * @param clipboardData Clipboard data retrieved from clipboard
     * @param pasteAsText Force pasting as plain text. Default value is false
     * @param applyCurrentStyle True if apply format of current selection to the pasted content,
     * false to keep original format.  Default value is false. When pasteAsText is true, this parameter is ignored
     * @param pasteAsImage: When set to true, if the clipboardData contains a imageDataUri will paste the image to the editor
     */
    paste(clipboardData: ClipboardData, pasteAsText?: boolean, applyCurrentFormat?: boolean, pasteAsImage?: boolean): void;
    /**
     * Get current selection range from Editor.
     * It does a live pull on the selection, if nothing retrieved, return whatever we have in cache.
     * @param tryGetFromCache Set to true to retrieve the selection range from cache if editor doesn't own the focus now.
     * Default value is true
     * @returns current selection range, or null if editor never got focus before
     */
    getSelectionRange(tryGetFromCache?: boolean): Range | null;
    /**
     * Get current selection range from Editor.
     * It does a live pull on the selection, if nothing retrieved, return whatever we have in cache.
     * @param tryGetFromCache Set to true to retrieve the selection range from cache if editor doesn't own the focus now.
     * Default value is true
     * @returns current selection range, or null if editor never got focus before
     */
    getSelectionRangeEx(): SelectionRangeEx;
    /**
     * Get current selection in a serializable format
     * It does a live pull on the selection, if nothing retrieved, return whatever we have in cache.
     * @returns current selection path, or null if editor never got focus before
     */
    getSelectionPath(): SelectionPath | null;
    /**
     * Check if focus is in editor now
     * @returns true if focus is in editor, otherwise false
     */
    hasFocus(): boolean;
    /**
     * Focus to this editor, the selection was restored to where it was before, no unexpected scroll.
     */
    focus(): void;
    select(arg1: Range | SelectionRangeEx | NodePosition | Node | SelectionPath | null, arg2?: NodePosition | number | PositionType | TableSelection | null, arg3?: Node, arg4?: number | PositionType): boolean;
    /**
     * Get current focused position. Return null if editor doesn't have focus at this time.
     */
    getFocusedPosition(): NodePosition | null;
    /**
     * Get an HTML element from current cursor position.
     * When expectedTags is not specified, return value is the current node (if it is HTML element)
     * or its parent node (if current node is a Text node).
     * When expectedTags is specified, return value is the first ancestor of current node which has
     * one of the expected tags.
     * If no element found within editor by the given tag, return null.
     * @param selector Optional, an HTML selector to find HTML element with.
     * @param startFrom Start search from this node. If not specified, start from current focused position
     * @param event Optional, if specified, editor will try to get cached result from the event object first.
     * If it is not cached before, query from DOM and cache the result into the event object
     */
    getElementAtCursor(selector?: string, startFrom?: Node, event?: PluginEvent): HTMLElement | null;
    /**
     * Check if this position is at beginning of the editor.
     * This will return true if all nodes between the beginning of target node and the position are empty.
     * @param position The position to check
     * @returns True if position is at beginning of the editor, otherwise false
     */
    isPositionAtBeginning(position: NodePosition): boolean;
    /**
     * Get impacted regions from selection
     */
    getSelectedRegions(type?: RegionType | CompatibleRegionType): Region[];
    addDomEventHandler(nameOrMap: string | Record<string, DOMEventHandler>, handler?: DOMEventHandler): () => void;
    /**
     * Trigger an event to be dispatched to all plugins
     * @param eventType Type of the event
     * @param data data of the event with given type, this is the rest part of PluginEvent with the given type
     * @param broadcast indicates if the event needs to be dispatched to all plugins
     * True means to all, false means to allow exclusive handling from one plugin unless no one wants that
     * @returns the event object which is really passed into plugins. Some plugin may modify the event object so
     * the result of this function provides a chance to read the modified result
     */
    triggerPluginEvent<T extends PluginEventType | CompatiblePluginEventType>(eventType: T, data: PluginEventData<T>, broadcast?: boolean): PluginEventFromType<T>;
    /**
     * Trigger a ContentChangedEvent
     * @param source Source of this event, by default is 'SetContent'
     * @param data additional data for this event
     */
    triggerContentChangedEvent(source?: ChangeSource | CompatibleChangeSource | string, data?: any): void;
    /**
     * Undo last edit operation
     */
    undo(): void;
    /**
     * Redo next edit operation
     */
    redo(): void;
    /**
     * Add undo snapshot, and execute a format callback function, then add another undo snapshot, then trigger
     * ContentChangedEvent with given change source.
     * If this function is called nested, undo snapshot will only be added in the outside one
     * @param callback The callback function to perform formatting, returns a data object which will be used as
     * the data field in ContentChangedEvent if changeSource is not null.
     * @param changeSource The change source to use when fire ContentChangedEvent. When the value is not null,
     * a ContentChangedEvent will be fired with change source equal to this value
     * @param canUndoByBackspace True if this action can be undone when user press Backspace key (aka Auto Complete).
     */
    addUndoSnapshot(callback?: (start: NodePosition | null, end: NodePosition | null) => any, changeSource?: ChangeSource | CompatibleChangeSource | string, canUndoByBackspace?: boolean, additionalData?: ContentChangedData): void;
    /**
     * Whether there is an available undo/redo snapshot
     */
    getUndoState(): EditorUndoState;
    /**
     * Get document which contains this editor
     * @returns The HTML document which contains this editor
     */
    getDocument(): Document;
    /**
     * Get the scroll container of the editor
     */
    getScrollContainer(): HTMLElement;
    /**
     * Get custom data related to this editor
     * @param key Key of the custom data
     * @param getter Getter function. If custom data for the given key doesn't exist,
     * call this function to get one and store it if it is specified. Otherwise return undefined
     * @param disposer An optional disposer function to dispose this custom data when
     * dispose editor.
     */
    getCustomData<T>(key: string, getter?: () => T, disposer?: (value: T) => void): T;
    /**
     * Check if editor is in IME input sequence
     * @returns True if editor is in IME input sequence, otherwise false
     */
    isInIME(): boolean;
    /**
     * Get default format of this editor
     * @returns Default format object of this editor
     */
    getDefaultFormat(): DefaultFormat;
    /**
     * Get a content traverser for the whole editor
     * @param startNode The node to start from. If not passed, it will start from the beginning of the body
     */
    getBodyTraverser(startNode?: Node): IContentTraverser;
    /**
     * Get a content traverser for current selection
     * @returns A content traverser, or null if editor never got focus before
     */
    getSelectionTraverser(range?: Range): IContentTraverser | null;
    /**
     * Get a content traverser for current block element start from specified position
     * @param startFrom Start position of the traverser. Default value is ContentPosition.SelectionStart
     * @returns A content traverser, or null if editor never got focus before
     */
    getBlockTraverser(startFrom?: ContentPosition | CompatibleContentPosition): IContentTraverser | null;
    /**
     * Get a text traverser of current selection
     * @param event Optional, if specified, editor will try to get cached result from the event object first.
     * If it is not cached before, query from DOM and cache the result into the event object
     * @returns A content traverser, or null if editor never got focus before
     */
    getContentSearcherOfCursor(event?: PluginEvent): IPositionContentSearcher | null;
    /**
     * Run a callback function asynchronously
     * @param callback The callback function to run
     * @returns a function to cancel this async run
     */
    runAsync(callback: (editor: IEditor) => void): () => void;
    /**
     * Set DOM attribute of editor content DIV
     * @param name Name of the attribute
     * @param value Value of the attribute
     */
    setEditorDomAttribute(name: string, value: string | null): void;
    /**
     * Get DOM attribute of editor content DIV, null if there is no such attribute.
     * @param name Name of the attribute
     */
    getEditorDomAttribute(name: string): string | null;
    /**
     * @deprecated Use getVisibleViewport() instead.
     *
     * Get current relative distance from top-left corner of the given element to top-left corner of editor content DIV.
     * @param element The element to calculate from. If the given element is not in editor, return value will be null
     * @param addScroll When pass true, The return value will also add scrollLeft and scrollTop if any. So the value
     * may be different than what user is seeing from the view. When pass false, scroll position will be ignored.
     * @returns An [x, y] array which contains the left and top distances, or null if the given element is not in editor.
     */
    getRelativeDistanceToEditor(element: HTMLElement, addScroll?: boolean): number[] | null;
    /**
     * Add a Content Edit feature.
     * @param feature The feature to add
     */
    addContentEditFeature(feature: GenericContentEditFeature<PluginEvent>): void;
    /**
     * Remove a Content Edit feature.
     * @param feature The feature to remove
     */
    removeContentEditFeature(feature: GenericContentEditFeature<PluginEvent>): void;
    /**
     * Get style based format state from current selection, including font name/size and colors
     */
    getStyleBasedFormatState(node?: Node): StyleBasedFormatState;
    /**
     * Get the pendable format such as underline and bold
     * @param forceGetStateFromDOM If set to true, will force get the format state from DOM tree.
     * @returns The pending format state
     */
    getPendableFormatState(forceGetStateFromDOM?: boolean): PendableFormatState;
    /**
     * Ensure user will type into a container element rather than into the editor content DIV directly
     * @param position The position that user is about to type to
     * @param keyboardEvent Optional keyboard event object
     */
    ensureTypeInContainer(position: NodePosition, keyboardEvent?: KeyboardEvent): void;
    /**
     * Set the dark mode state and transforms the content to match the new state.
     * @param nextDarkMode The next status of dark mode. True if the editor should be in dark mode, false if not.
     */
    setDarkModeState(nextDarkMode?: boolean): void;
    /**
     * Check if the editor is in dark mode
     * @returns True if the editor is in dark mode, otherwise false
     */
    isDarkMode(): boolean;
    /**
     * Transform the given node and all its child nodes to dark mode color if editor is in dark mode
     * @param node The node to transform
     * @param direction The transform direction. @default ColorTransformDirection.LightToDark
     */
    transformToDarkColor(node: Node, direction?: ColorTransformDirection | CompatibleColorTransformDirection): void;
    /**
     * Get a darkColorHandler object for this editor.
     */
    getDarkColorHandler(): DarkColorHandler;
    /**
     * Make the editor in "Shadow Edit" mode.
     * In Shadow Edit mode, all format change will finally be ignored.
     * This can be used for building a live preview feature for format button, to allow user
     * see format result without really apply it.
     * This function can be called repeated. If editor is already in shadow edit mode, we can still
     * use this function to do more shadow edit operation.
     */
    startShadowEdit(): void;
    /**
     * Leave "Shadow Edit" mode, all changes made during shadow edit will be discarded
     */
    stopShadowEdit(): void;
    /**
     * Check if editor is in Shadow Edit mode
     */
    isInShadowEdit(): boolean;
    /**
     * Check if the given experimental feature is enabled
     * @param feature The feature to check
     */
    isFeatureEnabled(feature: ExperimentalFeatures | CompatibleExperimentalFeatures): boolean;
    /**
     * Get a function to convert HTML string to trusted HTML string.
     * By default it will just return the input HTML directly. To override this behavior,
     * pass your own trusted HTML handler to EditorOptions.trustedHTMLHandler
     * See https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy/trusted-types
     */
    getTrustedHTMLHandler(): TrustedHTMLHandler;
    /**
     * @deprecated Use getZoomScale() instead
     */
    getSizeTransformer(): SizeTransformer;
    /**
     * Get current zoom scale, default value is 1
     * When editor is put under a zoomed container, need to pass the zoom scale number using EditorOptions.zoomScale
     * to let editor behave correctly especially for those mouse drag/drop behaviors
     * @returns current zoom scale number
     */
    getZoomScale(): number;
    /**
     * Set current zoom scale, default value is 1
     * When editor is put under a zoomed container, need to pass the zoom scale number using EditorOptions.zoomScale
     * to let editor behave correctly especially for those mouse drag/drop behaviors
     * @param scale The new scale number to set. It should be positive number and no greater than 10, otherwise it will be ignored.
     */
    setZoomScale(scale: number): void;
    /**
     * Retrieves the rect of the visible viewport of the editor.
     */
    getVisibleViewport(): Rect | null;
    /**
     * @returns the current EditorCore object
     * @throws a standard Error if there's no core object
     */
    protected getCore(): TEditorCore;
}

    /**
 * Check if the given experimental feature is enabled
 * @param featureSet All enabled features
 * @param feature The feature to check
 * @returns True if the given feature is enabled, otherwise false
 */
function isFeatureEnabled(featureSet: (ExperimentalFeatures | CompatibleExperimentalFeatures)[] | undefined, feature: ExperimentalFeatures | CompatibleExperimentalFeatures): boolean;

    /**
 * Create a new instance of Editor Core
 * @param contentDiv The DIV HTML element which will be the container element of editor
 * @param options An optional options object to customize the editor
 */
const createEditorCore: CoreCreator<EditorCore, EditorOptions>;

    /**
 * Increase or decrease font size in selection
 * @param editor The editor instance
 * @param change Whether increase or decrease font size
 * @param fontSizes A sorted font size array, in pt. Default value is FONT_SIZES
 */
function changeFontSize(editor: IEditor, change: FontSizeChange | CompatibleFontSizeChange, fontSizes?: number[]): void;

    /**
 * Default font size sequence, in pt. Suggest editor UI use this sequence as your font size list,
 * So that when increase/decrease font size, the font size can match the sequence of your font size picker
 */
const FONT_SIZES: number[];

    /**
 * @deprecated Use clearFormat instead and pass the ClearFormatMode.Block as parameter
 * @param editor The editor instance
 */
function clearBlockFormat(editor: IEditor): void;

    /**
 * Clear the format in current selection, after cleaning, the format will be
 * changed to default format. The format that get cleaned include B/I/U/font name/
 * font size/text color/background color/align left/align right/align center/superscript/subscript
 * @param editor The editor instance
 * @param formatType type of format to apply
 */
function clearFormat(editor: IEditor, formatType?: ClearFormatMode | CompatibleClearFormatMode): void;

    /**
 * Insert a hyperlink at cursor.
 * When there is a selection, hyperlink will be applied to the selection,
 * otherwise a hyperlink will be inserted to the cursor position.
 * @param editor Editor object
 * @param link Link address, can be http(s), mailto, notes, file, unc, ftp, news, telnet, gopher, wais.
 * When protocol is not specified, a best matched protocol will be predicted.
 * @param altText Optional alt text of the link, will be shown when hover on the link
 * @param displayText Optional display text for the link.
 * @param target Optional display target for the link ("_blank"|"_self"|"_parent"|"_top"|"{framename}")
 * If specified, the display text of link will be replaced with this text.
 * If not specified and there wasn't a link, the link url will be used as display text.
 */
function createLink(editor: IEditor, link: string, altText?: string, displayText?: string, target?: string): void;

    /**
 * Get format state at cursor
 * A format state is a collection of all format related states, e.g.,
 * bold, italic, underline, font name, font size, etc.
 * @param editor The editor instance
 * @param event (Optional) The plugin event, it stores the event cached data for looking up.
 * In this function the event cache is used to get list state and heading level. If not passed,
 * it will query the node within selection to get the info
 * @returns The format state at cursor
 */
function getFormatState(editor: IEditor, event?: PluginEvent): FormatState;

    /**
 * Get element based Format State at cursor
 * @param editor The editor instance
 * @param event (Optional) The plugin event, it stores the event cached data for looking up.
 * In this function the event cache is used to get list state and heading level. If not passed,
 * it will query the node within selection to get the info
 * @returns An ElementBasedFormatState object
 */
function getElementBasedFormatState(editor: IEditor, event?: PluginEvent): ElementBasedFormatState;

    /**
 * Insert an entity into editor.
 * @param editor The editor to insert entity into.
 * @param type Type of the entity
 * @param contentNode Root element of the entity
 * @param isBlock Whether the entity will be shown as a block
 * @param isReadonly Whether the entity will be a readonly entity
 * @param position @optional The position to insert into. If not specified, current position will be used.
 * If isBlock is true, entity will be insert below this position
 * @param insertToRegionRoot @optional When pass true, insert the entity at the root level of current region.
 * Parent nodes will be split if need
 * @param focusAfterEntity @optional When pass true, focus will be moved next to the entity. For inline entity,
 * focus will be after right after the entity (and the delimiter if exist). For block entity, focus will be in
 * the new empty line below the entity
 */
function insertEntity(editor: IEditor, type: string, contentNode: Node, isBlock: boolean, isReadonly: boolean, position?: NodePosition | ContentPosition.Begin | ContentPosition.End | ContentPosition.DomEnd | CompatibleContentPosition.Begin | CompatibleContentPosition.End | CompatibleContentPosition.DomEnd, insertToRegionRoot?: boolean, focusAfterEntity?: boolean): Entity;

    /**
 * Insert an image to editor at current selection
 * @param editor The editor instance
 * @param imageFileOrSrc Either the image file blob or source string of the image.
 * @param attributes Optional image element attributes
 */
function insertImage(editor: IEditor, imageFileOrSrc: File | string, attributes?: Record<string, string>): void;

    /**
 * Insert table into editor at current selection
 * @param editor The editor instance
 * @param columns Number of columns in table, it also controls the default table cell width:
 * if columns &lt;= 4, width = 120px; if columns &lt;= 6, width = 100px; else width = 70px
 * @param rows Number of rows in table
 * @param format (Optional) The table format. If not passed, the default format will be applied:
 * background color: #FFF; border color: #ABABAB
 */
function insertTable(editor: IEditor, columns: number, rows: number, format?: TableFormat): void;

    /**
 * Edit table with given operation. If there is no table at cursor then no op.
 * @param editor The editor instance
 * @param operation Table operation
 */
function editTable(editor: IEditor, operation: TableOperation | CompatibleTableOperation): void;

    /**
 * Format table
 * @param editor The editor which contains the table to format
 * @param format A TableFormat object contains format information we want to apply to the table
 * @param table The table to format. This is optional. When not passed, the current table (if any) will be formatted
 */
function formatTable(editor: IEditor, format: TableFormat, table?: HTMLTableElement): void;

    /**
 * Remove link at selection. If no links at selection, do nothing.
 * If selection contains multiple links, all of the link styles will be removed.
 * If only part of a link is selected, the whole link style will be removed.
 * @param editor The editor instance
 */
function removeLink(editor: IEditor): void;

    /**
 * Replace text before current selection with a node, current selection will be kept if possible
 * @param editor The editor instance
 * @param text The text for matching. We will try to match the text with the text before cursor
 * @param node The node to replace the text with
 * @param exactMatch True if the text must appear exactly before selection,
 * otherwise there can be some text between the target text and selection
 * @param searcher Optional PositionContentSearcher of current selection to help search text
 */
function replaceWithNode(editor: IEditor, text: string, node: Node, exactMatch: boolean, searcher?: IPositionContentSearcher): boolean;

    /**
 * Replace a given range with a node, current selection will be kept if possible
 * @param editor The editor instance
 * @param range The range to replace from
 * @param node The node to replace the text with
 * @param exactMatch True if the text must appear exactly before selection,
 * otherwise there can be some text between the target text and selection
 */
function replaceWithNode(editor: IEditor, range: Range, node: Node, exactMatch: boolean): boolean;

    /**
 * Rotate an element visually
 * @param editor The editor instance
 * @param element The element that should be rotated
 * @param angle The degree at which to rotate the element from it's center
 */
function rotateElement(editor: IEditor, element: HTMLElement, angle: number): void;

    /**
 * Set content alignment
 * @param editor The editor instance
 * @param alignment The alignment option:
 * Alignment.Center, Alignment.Left, Alignment.Right
 */
function setAlignment(editor: IEditor, alignment: Alignment | CompatibleAlignment): void;

    /**
 * Set background color at current selection
 * @param editor The editor instance
 * @param color One of two options:
 * The color string, can be any of the predefined color names (e.g, 'red')
 * or hexadecimal color string (e.g, '#FF0000') or rgb value (e.g, 'rgb(255, 0, 0)') supported by browser.
 * Currently there's no validation to the string, if the passed string is invalid, it won't take effect
 * Alternatively, you can pass a @typedef ModeIndependentColor. If in light mode, the lightModeColor property will be used.
 * If in dark mode, the darkModeColor will be used and the lightModeColor will be used when converting back to light mode.
 **/
function setBackgroundColor(editor: IEditor, color: string | ModeIndependentColor): void;

    /**
 * Set text color at selection
 * @param editor The editor instance
 * @param color One of two options:
 * The color string, can be any of the predefined color names (e.g, 'red')
 * or hexadecimal color string (e.g, '#FF0000') or rgb value (e.g, 'rgb(255, 0, 0)') supported by browser.
 * Currently there's no validation to the string, if the passed string is invalid, it won't take affect
 * Alternatively, you can pass a @typedef ModeIndependentColor. If in light mode, the lightModeColor property will be used.
 * If in dark mode, the darkModeColor will be used and the lightModeColor will be used when converting back to light mode.
 * @param shouldApplyInlineStyle Optional callback function to be invoked to verify if the current element should have the inline Style applied
 */
function setTextColor(editor: IEditor, color: string | ModeIndependentColor, shouldApplyInlineStyle?: (element: HTMLElement) => boolean): void;

    /**
 * Change direction for the blocks/paragraph at selection
 * @param editor The editor instance
 * @param direction The direction option:
 * Direction.LeftToRight refers to 'ltr', Direction.RightToLeft refers to 'rtl'
 */
function setDirection(editor: IEditor, direction: Direction | CompatibleDirection): void;

    /**
 * Set font name at selection
 * @param editor The editor instance
 * @param fontName The fontName string, should be a valid CSS font-family style.
 * Currently there's no validation to the string, if the passed string is invalid, it won't take affect
 */
function setFontName(editor: IEditor, fontName: string): void;

    /**
 * Set font size at selection
 * @param editor The editor instance
 * @param fontSize The fontSize string, should be a valid CSS font-size style.
 * Currently there's no validation to the string, if the passed string is invalid, it won't take affect
 */
function setFontSize(editor: IEditor, fontSize: string): void;

    /**
 * Set image alt text for all selected images at selection. If no images is contained
 * in selection, do nothing.
 * The alt attribute provides alternative information for an image if a user for some reason
 * cannot view it (because of slow connection, an error in the src attribute, or if the user
 * uses a screen reader). See https://www.w3schools.com/tags/att_img_alt.asp
 * @param editor The editor instance
 * @param altText The image alt text
 */
function setImageAltText(editor: IEditor, altText: string): void;

    /**
 * Set indentation at selection
 * If selection contains bullet/numbering list, increase/decrease indentation will
 * increase/decrease the list level by one.
 * @param editor The editor instance
 * @param indentation The indentation option:
 * Indentation.Increase to increase indentation or Indentation.Decrease to decrease indentation
 */
function setIndentation(editor: IEditor, indentation: Indentation | CompatibleIndentation): void;

    /**
 * Change the capitalization of text in the selection
 * @param editor The editor instance
 * @param capitalization The case option
 * @param language Optional parameter for language string that should comply to "IETF BCP 47 Tags for
 * Identifying Languages". For example: 'en' or 'en-US' for English, 'tr' for Turkish.
 * Default is the host environment’s current locale.
 */
function changeCapitalization(editor: IEditor, capitalization: Capitalization | CompatibleCapitalization, language?: string): void;

    /**
 * Toggle bold at selection
 * If selection is collapsed, it will only affect the following input after caret
 * If selection contains only bold text, the bold style will be removed
 * If selection contains only normal text, bold style will be added to the whole selected text
 * If selection contains both bold and normal text, bold style will be added to the whole selected text
 * @param editor The editor instance
 */
function toggleBold(editor: IEditor): void;

    /**
 * Toggle bullet at selection
 * If selection contains bullet in deep level, toggle bullet will decrease the bullet level by one
 * If selection contains number list, toggle bullet will convert the number list into bullet list
 * If selection contains both bullet/numbering and normal text, the behavior is decided by corresponding
 * browser execCommand API
 * @param editor The editor instance
 * @param listStyle (Optional) the style of the bullet list. If not defined, the style will be set to disc.
 * @param apiNameOverride (Optional) Set a new api name, if empty the api name will be 'toggleListType'.
 */
function toggleBullet(editor: IEditor, listStyle?: BulletListType | CompatibleBulletListType, apiNameOverride?: string): void;

    /**
 * Toggle italic at selection
 * If selection is collapsed, it will only affect the input after caret
 * If selection contains only italic text, the italic style will be removed
 * If selection contains only normal text, italic style will be added to the whole selected text
 * If selection contains both italic and normal text, italic style will be added to the whole selected text
 * @param editor The editor instance
 */
function toggleItalic(editor: IEditor): void;

    /**
 * Toggle numbering at selection
 * If selection contains numbering in deep level, toggle numbering will decrease the numbering level by one
 * If selection contains bullet list, toggle numbering will convert the bullet list into number list
 * If selection contains both bullet/numbering and normal text, the behavior is decided by corresponding
 * realization of browser execCommand API
 * @param editor The editor instance
 * @param startNumber (Optional) Start number of the list
 * @param listStyle (Optional) The style of the numbering list. If not defined, the style will be set to decimal.
 * @param apiNameOverride (Optional) Set a new api name, if empty the api name will be 'toggleListType'.
 */
function toggleNumbering(editor: IEditor, startNumber?: number, listStyle?: NumberingListType | CompatibleNumberingListType, apiNameOverride?: string): void;

    /**
 * Resets Ordered List Numbering back to the value of the parameter startNumber
 * @param editor The editor instance
 * @param separator The HTML element that indicates when to split the VList
 * @param startNumber The number of that the splitted list should start
 */
function setOrderedListNumbering(editor: IEditor, separator: HTMLLIElement, startNumber?: number): void;

    /**
 * Toggle blockquote at selection, if selection already contains any blockquote elements,
 * the blockquote elements will be unquote and other elements will take no effect
 * @param editor The editor instance
 * @param styler (Optional) The custom styler for setting the style for the blockquote element
 */
function toggleBlockQuote(editor: IEditor, styler?: (element: HTMLElement) => void): void;

    /**
 * Toggle code block at selection, if selection already contains any code blocked elements,
 * the code block elements will be no longer be code blocked and other elements will take no affect
 * @param editor The editor instance
 * @param styler (Optional) The custom styler for setting the style for the code block element
 */
function toggleCodeBlock(editor: IEditor, styler?: (element: HTMLElement) => void): void;

    /**
 * Toggle strikethrough at selection
 * If selection is collapsed, it will only affect the input after caret
 * If selection contains only strikethrough text, the strikethrough style will be removed
 * If selection contains only normal text, strikethrough style will be added to the whole selected text
 * If selection contains both strikethrough and normal text, strikethrough style will be added to the whole selected text
 * @param editor The editor instance
 */
function toggleStrikethrough(editor: IEditor): void;

    /**
 * Toggle subscript at selection
 * If selection is collapsed, it will only affect the input after caret
 * If selection contains only subscript text, the subscript style will be removed
 * If selection contains only normal text, subscript style will be added to the whole selected text
 * If selection contains both subscript and normal text, the subscript style will be removed from whole selected text
 * If selection contains any superscript text, the behavior is determined by corresponding realization of browser
 * execCommand API
 * @param editor The editor instance
 */
function toggleSubscript(editor: IEditor): void;

    /**
 * Toggle superscript at selection
 * If selection is collapsed, it will only affect the input after caret
 * If selection contains only superscript text, the superscript style will be removed
 * If selection contains only normal text, superscript style will be added to the whole selected text
 * If selection contains both superscript and normal text, the superscript style will be removed from whole selected text
 * If selection contains any subscript text, the behavior is determined by corresponding realization of browser
 * execCommand API
 * @param editor The editor instance
 */
function toggleSuperscript(editor: IEditor): void;

    /**
 * Toggle underline at selection
 * If selection is collapsed, it will only affect the input after caret
 * If selection contains only underlined text, the underline style will be removed
 * If selection contains only normal text, underline style will be added to the whole selected text
 * If selection contains both underlined and normal text, the underline style will be added to the whole selected text
 * @param editor The editor instance
 */
function toggleUnderline(editor: IEditor): void;

    /**
 * Set heading level at selection
 * @param editor The editor instance
 * @param level The heading level, can be a number from 0 to 6, in which 1 ~ 6 refers to
 * the HTML heading element &lt;H1&gt; to &lt;H6&gt;, 0 means no heading
 * if passed in param is outside the range, will be rounded to nearest number in the range
 */
function setHeadingLevel(editor: IEditor, level: number): void;

    /**
 * @deprecated Use setHeadingLevel instead
 * Keep this for compatibility only, will be removed in next major release
 */
const toggleHeader: typeof setHeadingLevel;

    /**
 * Set background color of cells.
 * @param editor The editor instance
 * @param color One of two options:
 **/
function applyCellShading(editor: IEditor, color: string | ModeIndependentColor): void;

    /**
 * Toggle List Type at selection
 * If ListType Provided is Ordered:
 *      If selection contains numbering in deep level, toggle numbering will decrease the numbering level by one
 *      If selection contains bullet list, toggle numbering will convert the bullet list into number list
 *      If selection contains both bullet/numbering and normal text, the behavior is decided by corresponding
 *       realization of browser execCommand API
 * If ListType Provided is Unordered:
 *      If selection contains bullet in deep level, toggle bullet will decrease the bullet level by one
 *      If selection contains number list, toggle bullet will convert the number list into bullet list
 *      If selection contains both bullet/numbering and normal text, the behavior is decided by corresponding
 *      browser execCommand API
 * @param editor The editor instance
 * @param listType The list type to toggle
 * @param startNumber (Optional) Start number of the list
 * @param includeSiblingLists Sets wether the operation should include Sibling Lists, by default true
 * @param orderedStyle (Optional) the style of an ordered. If not defined, the style will be set to decimal.
 * @param unorderedStyle (Optional) the style of an unordered list. If not defined, the style will be set to disc.
 * @param apiNameOverride (Optional) Set a new api name, if empty the api name will be 'toggleListType'.
 */
function toggleListType(editor: IEditor, listType: ListType | CompatibleListType, startNumber?: number, includeSiblingLists?: boolean, orderedStyle?: NumberingListType | CompatibleNumberingListType, unorderedStyle?: BulletListType | CompatibleBulletListType, apiNameOverride?: string): void;

    /**
 * Split selection into regions, and perform a block-wise formatting action for each region.
 */
function blockFormat(editor: IEditor, callback: (region: Region, start: NodePosition | null, end: NodePosition | null, chains: VListChain[]) => void, beforeRunCallback?: () => boolean, apiName?: string): void;

    /**
 * Commit changes of all list changes when experiment features are allowed
 * @param editor The Editor object
 * @param chains List chains to commit
 */
function commitListChains(editor: IEditor, chains: VListChain[]): void;

    /**
 * @deprecated
 * Same with commitListChains, keep this export just for backward compatibility
 */
const experimentCommitListChains: typeof commitListChains;

    /**
 * Get dark mode color for a given color
 * @param color The color to calculate from
 * @param baseLValue The Light value for base dark color in LAB format. @default the Light value for #333333
 */
function getDarkColor(color: string, baseLValue?: number): string;

    /**
 * Information of current OS and web browser
 */
interface BrowserInfo {
    /**
     * Wether current OS is Mac
     */
    readonly isMac?: boolean;
    /**
     * Whether current OS is Windows
     */
    readonly isWin?: boolean;
    /**
     * Whether current browser is using webkit kernel
     */
    readonly isWebKit?: boolean;
    /**
     * Whether current browser is Internet Explorer
     */
    readonly isIE?: boolean;
    /**
     * Whether current browser is IE10 or IE11
     */
    readonly isIE11OrGreater?: boolean;
    /**
     * Whether current browser is Safari
     */
    readonly isSafari?: boolean;
    /**
     * Whether current browser is Chrome
     */
    readonly isChrome?: boolean;
    /**
     * Whether current browser is Firefox
     */
    readonly isFirefox?: boolean;
    /**
     * Whether current browser is Edge
     */
    readonly isEdge?: boolean;
    /**
     * Whether current browser is IE/Edge
     */
    readonly isIEOrEdge?: boolean;
    /**
     * Whether current OS is Android
     */
    readonly isAndroid?: boolean;
    /**
     * Whether current browser is on mobile or a tablet
     */
    readonly isMobileOrTablet?: boolean;
}

    /**
 * The link preview pasted info provided by Edge
 */
interface EdgeLinkPreview {
    /**
     * Domain of the source page
     */
    domain: string;
    /**
     * Preferred paste format
     */
    preferred_format: string;
    /**
     * Title of the source page
     */
    title: string;
    /**
     * Type of the paste content
     */
    type: string;
    /**
     * Url of the source page
     */
    url: string;
}

    /**
 * The state object for DOMEventPlugin
 */
interface DOMEventPluginState {
    /**
     * Whether editor is in IME input sequence
     */
    isInIME: boolean;
    /**
     * Scroll container of editor
     */
    scrollContainer: HTMLElement;
    /**
     * Cached selection range
     */
    selectionRange: Range | null;
    /**
     * Table selection range
     */
    tableSelectionRange: TableSelectionRange | null;
    /**
     * stop propagation of a printable keyboard event
     */
    stopPrintableKeyboardEventPropagation: boolean;
    /**
     * Context menu providers, that can provide context menu items
     */
    contextMenuProviders: ContextMenuProvider<any>[];
    /**
     * Image selection range
     */
    imageSelectionRange: ImageSelectionRange | null;
    /**
     * When set to true, onFocus event will not trigger reselect cached range
     */
    skipReselectOnFocus?: boolean;
}

    /**
 * The state object for EditPlugin
 */
interface EditPluginState {
    /**
     * All content edit features sorted by their keys
     */
    features: Record<number, GenericContentEditFeature<PluginEvent>[]>;
}

    /**
 * The state object for EntityPlugin
 */
interface EntityPluginState {
    /**
     * @deprecated
     * Last clicking point when mouse down event happens
     */
    clickingPoint?: {
        pageX: number;
        pageY: number;
    };
    /**
     * @deprecated
     * All known entity elements
     */
    knownEntityElements?: HTMLElement[];
    /**
     * @deprecated
     */
    shadowEntityCache?: Record<string, HTMLElement>;
    /**
     * Entities cached for undo snapshot
     */
    entityMap: Record<string, KnownEntityItem>;
}

    /**
 * The state object for LifecyclePlugin
 */
interface LifecyclePluginState {
    /**
     * Custom data of this editor
     */
    customData: Record<string, CustomData>;
    /**
     * Default format of this editor
     */
    defaultFormat: DefaultFormat | null;
    /**
     * Whether editor is in dark mode
     */
    isDarkMode: boolean;
    /**
     * Calculate dark mode color from light mode color
     */
    getDarkColor: (lightColor: string) => string;
    /**
     * External content transform function to help do color transform for existing content
     */
    onExternalContentTransform: ((element: HTMLElement, fromDarkMode: boolean, toDarkMode: boolean, darkColorHandler: DarkColorHandler) => void) | null;
    /**
     * Enabled experimental features
     */
    experimentalFeatures: (ExperimentalFeatures | CompatibleExperimentalFeatures)[];
    /**
     * Cached document fragment for original content
     */
    shadowEditFragment: DocumentFragment | null;
    /**
     * Cached entity pairs for original content
     */
    shadowEditEntities: Record<string, HTMLElement> | null;
    /**
     * Cached selection path for original content
     */
    shadowEditSelectionPath: SelectionPath | null;
    /**
     * Cached table selection path for original content
     */
    shadowEditTableSelectionPath: SelectionPath[] | null;
    /**
     * Cached image selection path for original content
     */
    shadowEditImageSelectionPath: SelectionPath[] | null;
}

    /**
 * The state object for PendingFormatStatePlugin
 */
interface PendingFormatStatePluginState {
    /**
     * Current pending format state
     */
    pendableFormatState: PendableFormatState | null;
    /**
     * The position of last pendable format state changing
     */
    pendableFormatPosition: NodePosition | null;
    /**
     * A temporary SPAN element to hold pending format change. it will be inserted into content when user type something,
     * or discard if focus position is moved
     */
    pendableFormatSpan: HTMLElement | null;
}

    /**
 * The state object for UndoPlugin
 */
interface UndoPluginState {
    /**
     * Snapshot service for undo, it helps handle snapshot add, remove and retrieve
     */
    snapshotsService: UndoSnapshotsService<Snapshot>;
    /**
     * Whether restoring of undo snapshot is in progress.
     */
    isRestoring: boolean;
    /**
     * Whether there is new content change after last undo snapshot is taken
     */
    hasNewContent: boolean;
    /**
     * If addUndoSnapshot() is called nested in another one, this will be true
     */
    isNested: boolean;
    /**
     * Position after last auto complete. Undo autoComplete only works if the current position matches this one
     */
    autoCompletePosition: NodePosition | null;
}

    /**
 * The state object for CopyPastePlugin
 */
interface CopyPastePluginState {
    /**
     * Allowed custom content type when paste besides text/plain, text/html and images
     * Only text types are supported, and do not add "text/" prefix to the type values
     */
    allowedCustomPasteType: string[];
}

    /**
 * Known announce strings
 */
const enum KnownAnnounceStrings {
    /**
     * String announced for a list item in a OL List
     * @example
     * Auto corrected, &lcub;0&rcub;
     * Where &lcub0&rcub is the new list item bullet
     */
    AnnounceListItemNumbering = 1,
    /**
     * String announced for a list item in a UL List
     * @example
     * Auto corrected bullet
     */
    AnnounceListItemBullet = 2,
    /**
     * String announced when cursor is moved to the last cell in a table
     */
    AnnounceOnFocusLastCell = 3
}

    /**
 * Command strings for Document.execCommand() API
 * https://developer.mozilla.org/en-US/docs/Web/API/Document/execCommand
 */
const enum DocumentCommand {
    /**
     * Changes the browser auto-link behavior (Internet Explorer only)
     */
    AutoUrlDetect = "AutoUrlDetect",
    /**
     * Changes the document background color. In styleWithCss mode, it affects the background color of the containing block instead.
     * This requires a &lt;color&gt; value string to be passed in as a value argument. Note that Internet Explorer uses this to set the
     * text background color.
     */
    BackColor = "backColor",
    /**
     * Toggles bold on/off for the selection or at the insertion point. Internet Explorer uses the &lt;strong&gt; tag instead of &lt;b&gt;.
     */
    Bold = "bold",
    /**
     * Clears all authentication credentials from the cache.
     */
    ClearAuthenticationCache = "ClearAuthenticationCache",
    /**
     * Makes the content document either read-only or editable. This requires a boolean true/false as the value argument.
     * (Not supported by Internet Explorer.)
     */
    ContentReadOnly = "contentReadOnly",
    /**
     * Copies the current selection to the clipboard. Conditions of having this behavior enabled vary from one browser to another,
     * and have evolved over time. Check the compatibility table to determine if you can use it in your case.
     */
    Copy = "copy",
    /**
     * Creates an hyperlink from the selection, but only if there is a selection. Requires a URI string as a value argument for the
     * hyperlink's href. The URI must contain at least a single character, which may be whitespace.
     * (Internet Explorer will create a link with a null value.)
     */
    CreateLink = "createLink",
    /**
     * Removes the current selection and copies it to the clipboard. When this behavior is enabled varies between browsers,
     * and its conditions have evolved over time. Check the compatibility table for usage details.
     */
    Cut = "cut",
    /**
     * Adds a &lt;small&gt; tag around the selection or at the insertion point. (Not supported by Internet Explorer.)
     */
    DecreaseFontSize = "decreaseFontSize",
    /**
     * Changes the paragraph separator used when new paragraphs are created in editable text regions. See Differences in markup
     * generation for more details.
     */
    DefaultParagraphSeparator = "defaultParagraphSeparator",
    /**
     * Deletes the current selection.
     */
    Delete = "delete",
    /**
     * Enables or disables the table row/column insertion and deletion controls. (Not supported by Internet Explorer.)
     */
    EnableInlineTableEditing = "enableInlineTableEditing",
    /**
     * Enables or disables the resize handles on images and other resizable objects. (Not supported by Internet Explorer.)
     */
    EnableObjectResizing = "enableObjectResizing",
    /**
     * Changes the font name for the selection or at the insertion point. This requires a font name string (like "Arial")
     * as a value argument.
     */
    FontName = "fontName",
    /**
     * Changes the font size for the selection or at the insertion point. This requires an integer from 1-7 as a value argument.
     */
    FontSize = "fontSize",
    /**
     * Changes a font color for the selection or at the insertion point. This requires a hexadecimal color value string
     * as a value argument.
     */
    ForeColor = "foreColor",
    /**
     * Adds an HTML block-level element around the line containing the current selection, replacing the block element containing
     * the line if one exists (in Firefox, &lt;blockquote&gt; is the exception — it will wrap any containing block element).
     * Requires a tag-name string as a value argument. Virtually all block-level elements can be used.
     * (Internet Explorer supports only heading tags H1–H6, ADDRESS, and PRE, which must be wrapped in angle brackets, such as "&lt;H1&gt;".)
     */
    FormatBlock = "formatBlock",
    /**
     * Deletes the character ahead of the cursor's position, identical to hitting the Delete key on a Windows keyboard.
     */
    ForwardDelete = "forwardDelete",
    /**
     * Adds a heading element around a selection or insertion point line. Requires the tag-name strings a value argument (i.e. "H1", "H6").
     * (Not supported by Internet Explorer and Safari.)
     */
    Heading = "heading",
    /**
     * Changes the background color for the selection or at the insertion point. Requires a color value string as a value argument.
     * useCSS must be true for this to function. (Not supported by Internet Explorer.)
     */
    HiliteColor = "hiliteColor",
    /**
     * Adds a &lt;big&gt; tag around the selection or at the insertion point. (Not supported by Internet Explorer.)
     */
    IncreaseFontSize = "increaseFontSize",
    /**
     * Indents the line containing the selection or insertion point. In Firefox, if the selection spans multiple lines at different
     * levels of indentation, only the least indented lines in the selection will be indented.
     */
    Indent = "indent",
    /**
     * Controls whether the Enter key inserts a &lt;br&gt; element, or splits the current block element into two.
     * (Not supported by Internet Explorer.)
     */
    InsertBrOnReturn = "insertBrOnReturn",
    /**
     * Inserts a &lt;hr&gt; element at the insertion point, or replaces the selection with it.
     */
    InsertHorizontalRule = "insertHorizontalRule",
    /**
     * Inserts an HTML string at the insertion point (deletes selection). Requires a valid HTML string as a value argument.
     * (Not supported by Internet Explorer.)
     */
    InsertHTML = "insertHTML",
    /**
     * Inserts an image at the insertion point (deletes selection). Requires a URL string for the image's src as a value argument.
     * The requirements for this string are the same as createLink.
     */
    InsertImage = "insertImage",
    /**
     * Creates a numbered ordered list for the selection or at the insertion point.
     */
    InsertOrderedList = "insertOrderedList",
    /**
     * Creates a bulleted unordered list for the selection or at the insertion point.
     */
    InsertUnorderedList = "insertUnorderedList",
    /**
     * Inserts a paragraph around the selection or the current line.
     * (Internet Explorer inserts a paragraph at the insertion point and deletes the selection.)
     */
    InsertParagraph = "insertParagraph",
    /**
     * Inserts the given plain text at the insertion point (deletes selection).
     */
    InsertText = "insertText",
    /**
     * Toggles italics on/off for the selection or at the insertion point.
     * (Internet Explorer uses the &lt;em&gt; element instead of &lt;i&gt;.)
     */
    Italic = "italic",
    /**
     * Centers the selection or insertion point.
     */
    JustifyCenter = "justifyCenter",
    /**
     * Justifies the selection or insertion point.
     */
    JustifyFull = "justifyFull",
    /**
     * Justifies the selection or insertion point to the left.
     */
    JustifyLeft = "justifyLeft",
    /**
     * Right-justifies the selection or the insertion point.
     */
    JustifyRight = "justifyRight",
    /**
     * Outdents the line containing the selection or insertion point.
     */
    Outdent = "outdent",
    /**
     * Pastes the clipboard contents at the insertion point (replaces current selection). Disabled for web content. See [1].
     */
    Paste = "paste",
    /**
     * Redoes the previous undo command.
     */
    Redo = "redo",
    /**
     * Removes all formatting from the current selection.
     */
    RemoveFormat = "removeFormat",
    /**
     * Selects all of the content of the editable region.
     */
    SelectAll = "selectAll",
    /**
     * Toggles strikethrough on/off for the selection or at the insertion point.
     */
    StrikeThrough = "strikeThrough",
    /**
     * Toggles subscript on/off for the selection or at the insertion point.
     */
    Subscript = "subscript",
    /**
     * Toggles superscript on/off for the selection or at the insertion point.
     */
    Superscript = "superscript",
    /**
     * Toggles underline on/off for the selection or at the insertion point.
     */
    Underline = "underline",
    /**
     * Undoes the last executed command.
     */
    Undo = "undo",
    /**
     * Removes the anchor element from a selected hyperlink.
     */
    Unlink = "unlink",
    /**
     * Replaces the useCSS command. true modifies/generates style attributes in markup, false generates presentational elements.
     */
    StyleWithCSS = "styleWithCSS"
}

    /**
 * The is essentially an enum representing result from browser compareDocumentPosition API
 * https://developer.mozilla.org/en-US/docs/Web/API/Node/compareDocumentPosition
 */
const enum DocumentPosition {
    /**
     * Same node
     */
    Same = 0,
    /**
     * Node is disconnected from document
     */
    Disconnected = 1,
    /**
     * Node is preceding the comparing node
     */
    Preceding = 2,
    /**
     * Node is following the comparing node
     */
    Following = 4,
    /**
     * Node contains the comparing node
     */
    Contains = 8,
    /**
     * Node is contained by the comparing node
     */
    ContainedBy = 16
}

    /**
 * Key numbers common used keys
 */
const enum Keys {
    NULL = 0,
    BACKSPACE = 8,
    TAB = 9,
    ENTER = 13,
    SHIFT = 16,
    CTRL_LEFT = 17,
    ALT = 18,
    ESCAPE = 27,
    SPACE = 32,
    PAGEUP = 33,
    END = 35,
    HOME = 36,
    LEFT = 37,
    UP = 38,
    RIGHT = 39,
    DOWN = 40,
    PRINT_SCREEN = 44,
    DELETE = 46,
    /**
     * @deprecated Just for backward compatibility
     */
    EIGHT_ASTIRISK = 56,
    EIGHT_ASTERISK = 56,
    B = 66,
    I = 73,
    U = 85,
    Y = 89,
    Z = 90,
    META_LEFT = 91,
    COMMA = 188,
    DASH_UNDERSCORE = 189,
    PERIOD = 190,
    /**
     * @deprecated Just for backward compatibility
     */
    FORWARDSLASH = 191,
    FORWARD_SLASH = 191,
    GRAVE_TILDE = 192,
    /**
     * Keys below are non-standard, and should be used in ContentEditFeatures only
     */
    CONTENTCHANGED = 257,
    RANGE = 258,
    Ctrl = 4096,
    Meta = 8192,
    Shift = 16384
}

    /**
 * The is essentially an enum represents the type of the node
 * https://developer.mozilla.org/en-US/docs/Web/API/Node/nodeType
 * Values not listed here are deprecated.
 */
const enum NodeType {
    /**
     * An Element node such as &lt;p&gt; or &lt;div&gt;.
     */
    Element = 1,
    /**
     * An Attribute node such as name="value".
     */
    Attribute = 2,
    /**
     * The actual Text of Element or Attr.
     */
    Text = 3,
    /**
     * A ProcessingInstruction of an XML document such as &lt;?xml-stylesheet ... ?&gt; declaration.
     */
    ProcessingInstruction = 7,
    /**
     * A Comment node.
     */
    Comment = 8,
    /**
     * A Document node.
     */
    Document = 9,
    /**
     * A DocumentType node e.g. &lt;!DOCTYPE html&gt; for HTML5 documents.
     */
    DocumentType = 10,
    /**
     * A DocumentFragment node.
     */
    DocumentFragment = 11
}

    /**
 * Prefix of content types
 */
const enum ContentTypePrefix {
    /**
     * Text type prefix
     */
    Text = "text/",
    /**
     * Image type prefix
     */
    Image = "image/"
}

    /**
 * Known content types
 */
const enum ContentType {
    /**
     * Plain text content type
     */
    PlainText = "text/plain",
    /**
     * HTML content type
     */
    HTML = "text/html"
}

    /**
 * enum for setting block alignment, used by setAlignment API
 */
const enum Alignment {
    /**
     * Align left
     */
    Left = 0,
    /**
     * Align center
     */
    Center = 1,
    /**
     * Align right
     */
    Right = 2
}

    /**
 * Possible change sources. Here are the predefined sources.
 * It can also be other string if the change source can't fall into these sources.
 */
const enum ChangeSource {
    /**
     * Content changed by auto link
     */
    AutoLink = "AutoLink",
    /**
     * Content changed by create link
     */
    CreateLink = "CreateLink",
    /**
     * Content changed by format
     */
    Format = "Format",
    /**
     * Content changed by image resize
     */
    ImageResize = "ImageResize",
    /**
     * Content changed by paste
     */
    Paste = "Paste",
    /**
     * Content changed by setContent API
     */
    SetContent = "SetContent",
    /**
     * Content changed by cut operation
     */
    Cut = "Cut",
    /**
     * Content changed by drag & drop operation
     */
    Drop = "Drop",
    /**
     * Insert a new entity into editor
     */
    InsertEntity = "InsertEntity",
    /**
     * Editor is switched to dark mode, content color is changed
     */
    SwitchToDarkMode = "SwitchToDarkMode",
    /**
     * Editor is switched to light mode, content color is changed
     */
    SwitchToLightMode = "SwitchToLightMode",
    /**
     * List chain reorganized numbers of lists
     */
    ListChain = "ListChain",
    /**
     * Keyboard event, used by Content Model.
     * Data of this event will be the key code number
     */
    Keyboard = "Keyboard"
}

    /**
 * Represents the mode of color transformation
 */
const enum ColorTransformDirection {
    /**
     * Transform from light to dark
     */
    LightToDark = 0,
    /**
     * Transform from dark to light
     */
    DarkToLight = 1
}

    /**
 * The position. Mostly used for content insertion and traversing
 * On insertion, we will need to specify where we want the content to be placed (begin, end, selection or outside)
 * On content traversing, we will need to specify the start position of traversing
 */
const enum ContentPosition {
    /**
     * Begin of the container
     */
    Begin = 0,
    /**
     * End of the container
     */
    End = 1,
    /**
     * End of the content div domain.
     */
    DomEnd = 2,
    /**
     * Selection start
     */
    SelectionStart = 3,
    /**
     * Outside of editor
     */
    Outside = 4,
    /**
     * Manually defined range
     */
    Range = 5
}

    /**
 * @deprecated
 * Constants string for dataset names used by dark mode
 */
const enum DarkModeDatasetNames {
    /**
     * Original style text color
     */
    OriginalStyleColor = "ogsc",
    /**
     * Original style background color
     */
    OriginalStyleBackgroundColor = "ogsb",
    /**
     * Original attribute text color
     */
    OriginalAttributeColor = "ogac",
    /**
     * Original attribute background color
     */
    OriginalAttributeBackgroundColor = "ogab"
}

    /**
 * Class names for Delimiter
 */
const enum DelimiterClasses {
    /**
     * Class name to specify this delimiter is before an entity
     */
    DELIMITER_BEFORE = "entityDelimiterBefore",
    /**
     *  Class name to specify this delimiter is after an entity
     */
    DELIMITER_AFTER = "entityDelimiterAfter"
}

    /**
 * enum for setting block direction, used by setDirection API
 */
const enum Direction {
    /**
     * Left to right
     */
    LeftToRight = 0,
    /**
     * Right to left
     */
    RightToLeft = 1
}

    /**
 * CSS Class names for Entity
 */
const enum EntityClasses {
    /**
     * Class name to specify this is an entity
     */
    ENTITY_INFO_NAME = "_Entity",
    /**
     * Class name to specify the type of an entity
     */
    ENTITY_TYPE_PREFIX = "_EType_",
    /**
     * Class name to specify the ID of an entity
     */
    ENTITY_ID_PREFIX = "_EId_",
    /**
     * Class name to specify if the entity is readonly
     */
    ENTITY_READONLY_PREFIX = "_EReadonly_"
}

    /**
 * Define possible operations to an entity
 */
const enum EntityOperation {
    /**
     * Notify plugins that there is a new plugin was added into editor.
     * Plugin can handle this event to entity hydration.
     * This event will be only fired once for each entity DOM node.
     * After undo, or copy/paste, since new DOM nodes were added, this event will be fired
     * for those entities represented by newly added nodes.
     */
    NewEntity = 0,
    /**
     * Notify plugins that user is clicking target to an entity
     */
    Click = 1,
    /**
     * Notify plugins that user is triggering context menu to an entity
     */
    ContextMenu = 2,
    /**
     * Notify plugins that user is escaping from an entity by pressing ESC key
     */
    Escape = 3,
    /**
     * Notify plugins that user is removing an entity from its start position using DELETE key
     */
    RemoveFromStart = 4,
    /**
     * Notify plugins that user is remove an entity from its end position using BACKSPACE key
     */
    RemoveFromEnd = 5,
    /**
     * Notify plugins that an entity is being overwritten.
     * This can be caused by key in, cut, paste, delete, backspace ... on a selection
     * which contains some entities.
     */
    Overwrite = 6,
    /**
     * Notify plugins that an entity is being partially overwritten.
     * This happens when user selected part of the entity then do key press, or cut, paste, delete, backspace, ...
     */
    PartialOverwrite = 7,
    /**
     * Notify plugins that editor is generating HTML content for save.
     * Plugin should use this event to remove any temporary content, and only leave DOM nodes that
     * should be saved as HTML string.
     * This event will provide a cloned DOM tree for each entity, do NOT compare the DOM nodes with cached nodes
     * because it will always return false.
     */
    ReplaceTemporaryContent = 8,
    /**
     * @deprecated
     */
    AddShadowRoot = 9,
    /**
     * @deprecated
     */
    RemoveShadowRoot = 10,
    /**
     * Notify plugins that a new entity state need to be updated to an entity.
     * This is normally happened when user undo/redo the content with an entity snapshot added by a plugin that handles entity
     */
    UpdateEntityState = 11
}

    /**
 * Experimental feature flags
 */
const enum ExperimentalFeatures {
    /**
     * @deprecated This feature is always enabled
     */
    NewIndentation = "NewIndentation",
    /**
     * @deprecated This feature is always enabled
     */
    NewBullet = "NewBullet",
    /**
     * @deprecated This feature is always enabled
     */
    NewNumbering = "NewNumbering",
    /**
     * @deprecated This feature is always enabled
     */
    ListChain = "ListChain",
    /**
     * @deprecated This feature is always enabled
     */
    MergePastedLine = "MergePastedLine",
    /**
     * @deprecated This feature is always enabled
     */
    SingleDirectionResize = "SingleDirectionResize",
    /**
     * @deprecated This feature is always enabled
     */
    PasteWithLinkPreview = "PasteWithLinkPreview",
    /**
     * @deprecated This feature is always enabled
     */
    ImageRotate = "ImageRotate",
    /**
     * @deprecated This feature is always enabled
     */
    ImageCrop = "ImageCrop",
    /**
     * @deprecated This feature is always enabled
     * Check if the element has a style attribute, if not, apply the default format
     */
    AlwaysApplyDefaultFormat = "AlwaysApplyDefaultFormat",
    /**
     * @deprecated This feature can be enabled/disabled using Paste Plugin contructor param
     * Paste the Html instead of the Img when the Html Body only have one IMG Child node
     */
    ConvertSingleImageBody = "ConvertSingleImageBody",
    /**
     * @deprecated This feature is always enabled
     * Align table elements to left, center and right using setAlignment API
     */
    TableAlignment = "TableAlignment",
    /**
     * @deprecated this feature is always enabled
     * Provide a circular resize handles that adaptive the number od handles to the size of the image
     */
    AdaptiveHandlesResizer = "AdaptiveHandlesResizer",
    /**
     * @deprecated this feature is always disabled
     * Automatically transform -- into hyphen, if typed between two words.
     */
    AutoHyphen = "AutoHyphen",
    /**
     * @deprecated this feature is always disabled
     * Use pending format strategy to do style based format, e.g. Font size, Color.
     * With this feature enabled, we don't need to insert temp ZeroWidthSpace character to hold pending format
     * when selection is collapsed. Instead, we will hold the pending format in memory and only apply it when type something
     */
    PendingStyleBasedFormat = "PendingStyleBasedFormat",
    /**
     * @deprecated this feature is always disabled
     * Normalize list to make sure it can be displayed correctly in other client
     * e.g. We will move list items with "display: block" into previous list item and change tag to be DIV
     */
    NormalizeList = "NormalizeList",
    /**
     * @deprecated this feature is always enabled
     * When a html image is selected, the selected image data will be stored by editor core.
     */
    ImageSelection = "ImageSelection",
    /**
     * @deprecated this feature is always enabled
     * Use variable-based dark mode solution rather than dataset-based solution.
     * When enable this feature, need to pass in a DarkModelHandler object to each call of setColor and applyFormat
     * if you need them work for dark mode
     */
    VariableBasedDarkColor = "VariableBasedDarkColor",
    /**
     * @deprecated this feature is always enabled
     * Align list elements elements to left, center and right using setAlignment API
     */
    ListItemAlignment = "ListItemAlignment",
    /**
     * @deprecated
     */
    DefaultFormatInSpan = "DefaultFormatInSpan",
    /**
     * @deprecated
     */
    DefaultFormatOnContainer = "DefaultFormatOnContainer",
    /**
     * @deprecated This feature is always enabled
     * Reuse existing DOM structure if possible when convert Content Model back to DOM tree
     */
    ReusableContentModel = "ReusableContentModel",
    /**
     * @deprecated This feature is always enabled
     * Handle keyboard editing event with Content Model
     */
    EditWithContentModel = "EditWithContentModel",
    /**
     * @deprecated This feature is always enabled
     * Trigger formatting by a especial characters. Ex: (A), 1. i).
     */
    AutoFormatList = "AutoFormatList",
    /**
     * @deprecated This feature is always enabled
     * Add entities around a Read Only  Inline entity to prevent cursor to be hidden when cursor is next of it.
     */
    InlineEntityReadOnlyDelimiters = "InlineEntityReadOnlyDelimiters",
    /**
     * @deprecated This feature is always enabled
     * Paste with Content model
     */
    ContentModelPaste = "ContentModelPaste",
    /**
     * Provide additional Tab Key Features. Requires Text Features Content Editable Features
     */
    TabKeyTextFeatures = "TabKeyTextFeatures",
    /**
     * With this feature enabled, when writing back a list item we will re-use all
     * ancestor list elements, even if they don't match the types currently in the
     * listTypes array for that item. The only list that we will ensure is correct
     * is the one closest to the item.
     */
    ReuseAllAncestorListElements = "ReuseAllAncestorListElements",
    /**
     * Delete table with Backspace key with the whole was selected with table selector
     */
    DeleteTableWithBackspace = "DeleteTableWithBackspace",
    /**
     * Disable list chain functionality
     */
    DisableListChain = "DisableListChain"
}

    /**
 * The enum used for increase or decrease font size
 * Used by setFontSize API
 */
const enum FontSizeChange {
    /**
     * Increase font size
     */
    Increase = 0,
    /**
     * Decrease font size
     */
    Decrease = 1
}

    /**
 * Represents a mode number to indicate what kind of content to retrieve when call Editor.getContent()
 */
const enum GetContentMode {
    /**
     * The clean content without any temporary content only for editor.
     * This is the default value. Call to Editor.getContent() with trigger an ExtractContentWithDom event
     * so that plugins can remove their temporary content, and will return the HTML content
     * which is ready for save to storage.
     */
    CleanHTML = 0,
    /**
     * Retrieve the raw HTML string in current editor. Temporary content will be included.
     */
    RawHTMLOnly = 1,
    /**
     * Retrieve the raw HTML string in current editor with a selection marker. This can be used for
     * save current editor state and call to SetContent with this result can fully restore editor state
     * including current selection
     */
    RawHTMLWithSelection = 2,
    /**
     * Get plain text content only, all format will be ignored
     */
    PlainText = 3,
    /**
     * A fast way to get plain text content, the line-end positions may not be exactly same with HTML content,
     * but the text content should be the same. This is used for quickly retrieve text content and check
     * text only
     */
    PlainTextFast = 4
}

    /**
 * The enum used for increase or decrease indentation of a block
 * Used by setIndentation API
 */
const enum Indentation {
    /**
     * Increase indentation
     */
    Increase = 0,
    /**
     * Decrease indentation
     */
    Decrease = 1
}

    /**
 * The enum used for controlling the capitalization of text.
 * Used by changeCapitalization API
 */
const enum Capitalization {
    /**
     * Transforms the first character after punctuation mark followed by space
     * to uppercase and the rest of characters to lowercase.
     */
    Sentence = "sentence",
    /**
     * Transforms all characters to lowercase
     */
    Lowercase = "lowercase",
    /**
     * Transforms all characters to uppercase
     */
    Uppercase = "uppercase",
    /**
     * Transforms the first character of each word to uppercase
     */
    CapitalizeEachWord = "capitalize"
}

    /**
 * Type of list (numbering or bullet)
 */
const enum ListType {
    /**
     * None list type
     * It means this is not a list
     */
    None = 0,
    /**
     * Ordered List type (numbering list)
     * Represented by "OL" tag
     */
    Ordered = 1,
    /**
     * Unordered List type (bullet list)
     * Represented by "UL" tag
     */
    Unordered = 2
}

    /**
 * Represent the type of a position
 */
const enum PositionType {
    /**
     * At the beginning of a node
     */
    Begin = 0,
    /**
     * At the end of a node
     */
    End = -1,
    /**
     * Before a node
     */
    Before = -2,
    /**
     * After a node
     */
    After = -3
}

    /**
 * Query scope for queryElements() API
 */
const enum QueryScope {
    /**
     * Query from the whole body of root node. This is default value.
     */
    Body = 0,
    /**
     * Query elements on a given selection (intersect)
     * The result element can contain the selection, contain part of selection, or inside selection
     */
    OnSelection = 1,
    /**
     * Query elements inside a given selection only
     */
    InSelection = 2
}

    /**
 * Type of all possible regions. Currently we only support region of Table
 */
const enum RegionType {
    /**
     * Region split by Table
     */
    Table = 0
}

    /**
 * Operations used by editTable() API
 */
const enum TableOperation {
    /**
     * Insert a row above current row
     */
    InsertAbove = 0,
    /**
     * Insert a row below current row
     */
    InsertBelow = 1,
    /**
     * Insert a column on the left of current column
     */
    InsertLeft = 2,
    /**
     * Insert a column on the right of current column
     */
    InsertRight = 3,
    /**
     * Delete the whole table
     */
    DeleteTable = 4,
    /**
     * Delete current column
     */
    DeleteColumn = 5,
    /**
     * Delete current row
     */
    DeleteRow = 6,
    /**
     * Merge current row with the row above
     */
    MergeAbove = 7,
    /**
     * Merge current row with the row below
     */
    MergeBelow = 8,
    /**
     * Merge current column with the column on the left
     */
    MergeLeft = 9,
    /**
     * Merge current column with the column on the right
     */
    MergeRight = 10,
    /**
     * Merge all selected cells
     */
    MergeCells = 11,
    /**
     * Split current table cell horizontally
     */
    SplitHorizontally = 12,
    /**
     * Split current table cell vertically
     */
    SplitVertically = 13,
    /**
     * Align current table at the center
     */
    AlignCenter = 14,
    /**
     * Align current table at the left
     */
    AlignLeft = 15,
    /**
     * Align current table at the right
     */
    AlignRight = 16,
    /**
     * Align current content table cell at the left
     */
    AlignCellLeft = 17,
    /**
     * Align current content table cell at the center
     */
    AlignCellCenter = 18,
    /**
     * Align current content table cell at the right
     */
    AlignCellRight = 19,
    /**
     * Align current content table cell at the top
     */
    AlignCellTop = 20,
    /**
     * Align current table cell at the middle
     */
    AlignCellMiddle = 21,
    /**
     * Align current table cell at the bottom
     */
    AlignCellBottom = 22
}

    /**
 * Operation flags for ImageEdit plugin
 */
const enum ImageEditOperation {
    /**
     * No operation
     */
    None = 0,
    /**
     * Resize image from corner ( horizontal and vertical resize)
     */
    CornerResize = 1,
    /**
     * Resize image from side ( either horizontal or vertical resize)
     */
    SideResize = 2,
    /**
     * Image resize
     */
    Resize = 3,
    /**
     * Image rotate
     */
    Rotate = 4,
    /**
     * Image resize and rotate
     */
    ResizeAndRotate = 7,
    /**
     * Image crop
     */
    Crop = 8,
    /**
     * All operations
     */
    All = 15
}

    /**
 * Represents the strategy to clear the format of the current editor selection
 */
const enum ClearFormatMode {
    /**
     * Inline format. Remove text format.
     */
    Inline = 0,
    /**
     * BLock format. Remove text and structure format of the block.
     */
    Block = 1,
    /**
     * Detect Inline or Block format based on the current editor selector.
     */
    AutoDetect = 2
}

    /**
 * Index of known CreateElementData used by createElement function
 */
const enum KnownCreateElementDataIndex {
    /**
     * Set a none value to help createElement function ignore falsy value
     */
    None = 0,
    /**
     * An empty line without format
     */
    EmptyLine = 1,
    /**
     * Wrapper for blockquote
     */
    BlockquoteWrapper = 2,
    /**
     * Temp DIV for copy/paste
     */
    CopyPasteTempDiv = 3,
    /**
     * ListItem with block style
     */
    BlockListItem = 4,
    /**
     * Wrapper element for context menu
     */
    ContextMenuWrapper = 5,
    /**
     * Wrapper element for image edit
     */
    ImageEditWrapper = 6,
    /**
     * @deprecated
     */
    TableHorizontalResizer = 7,
    /**
     * @deprecated
     */
    TableVerticalResizer = 8,
    /**
     * @deprecated
     */
    TableResizerLTR = 9,
    /**
     * @deprecated
     */
    TableResizerRTL = 10,
    /**
     * @deprecated
     */
    TableSelector = 11,
    /**
     * @deprecated
     */
    EmptyLineFormatInSpan = 12
}

    /**
 * Represent the types of sources to handle in the Paste Plugin
 */
const enum KnownPasteSourceType {
    WordDesktop = 0,
    ExcelDesktop = 1,
    ExcelOnline = 2,
    PowerPointDesktop = 3,
    GoogleSheets = 4,
    WacComponents = 5,
    Default = 6,
    SingleImage = 7
}

    /**
 * Table format border
 */
const enum TableBorderFormat {
    /**
     * All border of the table are displayed
     *  __ __ __
     * |__|__|__|
     * |__|__|__|
     * |__|__|__|
     */
    DEFAULT = 0,
    /**
     * Middle vertical border are not displayed
     *  __ __ __
     * |__ __ __|
     * |__ __ __|
     * |__ __ __|
     */
    LIST_WITH_SIDE_BORDERS = 1,
    /**
     * All borders except header rows borders are displayed
     *  __ __ __
     *  __|__|__
     *  __|__|__
     */
    NO_HEADER_BORDERS = 2,
    /**
     * The left and right border of the table are not displayed
     *  __ __ __
     *  __|__|__
     *  __|__|__
     *  __|__|__
     */
    NO_SIDE_BORDERS = 3,
    /**
     * Only the borders that divides the header row, first column and externals are displayed
     *  __ __ __
     * |__ __ __|
     * |  |     |
     * |__|__ __|
     */
    FIRST_COLUMN_HEADER_EXTERNAL = 4,
    /**
     * The header row has no vertical border, except for the first one
     * The first column has no horizontal border, except for the first one
     *  __ __ __
     * |__ __ __
     * |  |__|__|
     * |  |__|__|
     */
    ESPECIAL_TYPE_1 = 5,
    /**
     * The header row has no vertical border, except for the first one
     * The only horizontal border of the table is the top and bottom of header row
     *  __ __ __
     * |__ __ __
     * |  |     |
     * |  |     |
     */
    ESPECIAL_TYPE_2 = 6,
    /**
     * The only borders are the bottom of header row and the right border of first column
     *  __ __ __
     *    |
     *    |
     */
    ESPECIAL_TYPE_3 = 7,
    /**
     * No border
     */
    CLEAR = 8
}

    /**
 * Editor plugin event type
 */
const enum PluginEventType {
    /**
     * HTML KeyDown event
     */
    KeyDown = 0,
    /**
     * HTML KeyPress event
     */
    KeyPress = 1,
    /**
     * HTML KeyUp event
     */
    KeyUp = 2,
    /**
     * HTML Input / TextInput event
     */
    Input = 3,
    /**
     * HTML CompositionEnd event
     */
    CompositionEnd = 4,
    /**
     * HTML MouseDown event
     */
    MouseDown = 5,
    /**
     * HTML MouseUp event
     */
    MouseUp = 6,
    /**
     * Content changed event
     */
    ContentChanged = 7,
    /**
     * Extract Content with a DOM tree event
     * This event is triggered when getContent() is called with triggerExtractContentEvent = true
     * Plugin can handle this event to remove the UI only markups to return clean HTML
     * by operating on a cloned DOM tree
     */
    ExtractContentWithDom = 8,
    /**
     * Before Paste event, provide a chance to change copied content
     */
    BeforeCutCopy = 9,
    /**
     * Before Paste event, provide a chance to change paste content
     */
    BeforePaste = 10,
    /**
     * Let plugin know editor is ready now
     */
    EditorReady = 11,
    /**
     * Let plugin know editor is about to dispose
     */
    BeforeDispose = 12,
    /**
     * Pending format state (bold, italic, underline, ... with collapsed selection) is changed
     */
    PendingFormatStateChanged = 13,
    /**
     * Scroll event triggered by scroll container
     */
    Scroll = 14,
    /**
     * Operating on an entity. See enum EntityOperation for more details about each operation
     */
    EntityOperation = 15,
    /**
     * HTML ContextMenu event
     */
    ContextMenu = 16,
    /**
     * Editor has entered shadow edit mode
     */
    EnteredShadowEdit = 17,
    /**
     * Editor is about to leave shadow edit mode
     */
    LeavingShadowEdit = 18,
    /**
     * Content of image is being changed from client side
     */
    EditImage = 19,
    /**
     * Content of editor is about to be cleared by SetContent API, handle this event to cache anything you need
     * before it is gone
     */
    BeforeSetContent = 20,
    /**
     * Zoom scale value is changed, triggered by Editor.setZoomScale() when set a different scale number
     */
    ZoomChanged = 21,
    /**
     * EXPERIMENTAL FEATURE
     * Editor changed the selection.
     */
    SelectionChanged = 22,
    /**
     * EXPERIMENTAL FEATURE
     * Editor content is about to be changed by keyboard event.
     * This is only used by Content Model editing
     */
    BeforeKeyboardEditing = 23
}

    /**
 * Types of Selection Ranges that the SelectionRangeEx can return
 */
const enum SelectionRangeTypes {
    /**
     * Normal selection range provided by browser.
     */
    Normal = 0,
    /**
     * Selection made inside of a single table.
     */
    TableSelection = 1,
    /**
     * Selection made in a image.
     */
    ImageSelection = 2
}

    /**
 *  Enum used to control the different types of numbering list
 */
const enum NumberingListType {
    /**
     * Minimum value of the enum
     */
    Min = 1,
    /**
     * Numbering triggered by 1.
     */
    Decimal = 1,
    /**
     * Numbering triggered by 1-
     */
    DecimalDash = 2,
    /**
     * Numbering triggered by 1)
     */
    DecimalParenthesis = 3,
    /**
     * Numbering triggered by (1)
     */
    DecimalDoubleParenthesis = 4,
    /**
     * Numbering triggered by a.
     */
    LowerAlpha = 5,
    /**
     * Numbering triggered by a)
     */
    LowerAlphaParenthesis = 6,
    /**
     * Numbering triggered by (a)
     */
    LowerAlphaDoubleParenthesis = 7,
    /**
     * Numbering triggered by a-
     */
    LowerAlphaDash = 8,
    /**
     * Numbering triggered by A.
     */
    UpperAlpha = 9,
    /**
     * Numbering triggered by A)
     */
    UpperAlphaParenthesis = 10,
    /**
     * Numbering triggered by (A)
     */
    UpperAlphaDoubleParenthesis = 11,
    /**
     * Numbering triggered by A-
     */
    UpperAlphaDash = 12,
    /**
     * Numbering triggered by i.
     */
    LowerRoman = 13,
    /**
     * Numbering triggered by i)
     */
    LowerRomanParenthesis = 14,
    /**
     * Numbering triggered by (i)
     */
    LowerRomanDoubleParenthesis = 15,
    /**
     * Numbering triggered by i-
     */
    LowerRomanDash = 16,
    /**
     * Numbering triggered by I.
     */
    UpperRoman = 17,
    /**
     * Numbering triggered by I)
     */
    UpperRomanParenthesis = 18,
    /**
     * Numbering triggered by (I)
     */
    UpperRomanDoubleParenthesis = 19,
    /**
     * Numbering triggered by I-
     */
    UpperRomanDash = 20,
    /**
     * Maximum value of the enum
     */
    Max = 20
}

    /**
 *  Enum used to control the different types of bullet list
 */
const enum BulletListType {
    /**
     * Minimum value of the enum
     */
    Min = 1,
    /**
     * Bullet triggered by *
     */
    Disc = 1,
    /**
     * Bullet triggered by -
     */
    Dash = 2,
    /**
     * Bullet triggered by --
     */
    Square = 3,
    /**
     * Bullet triggered by >
     */
    ShortArrow = 4,
    /**
     * Bullet triggered by ->
     */
    LongArrow = 5,
    /**
     * Bullet triggered by =>
     */
    UnfilledArrow = 6,
    /**
     * Bullet triggered by —
     */
    Hyphen = 7,
    /**
     * Bullet triggered by -->
     */
    DoubleLongArrow = 8,
    /**
     * Bullet type circle
     */
    Circle = 9,
    /**
     * Maximum value of the enum
     */
    Max = 9
}

    /**
 * Types of definitions, used by Definition type
 */
const enum DefinitionType {
    /**
     * Boolean type definition, represents a boolean type value
     */
    Boolean = 0,
    /**
     * Number type definition, represents a number type value
     */
    Number = 1,
    /**
     * String type definition, represents a string type value
     */
    String = 2,
    /**
     * Array type definition, represents an array with a given item type
     */
    Array = 3,
    /**
     * Object type definition, represents an object with the given property types
     */
    Object = 4,
    /**
     * Customize type definition, represents a customized type with a validator function
     */
    Customize = 5
}

    /**
 * Enum for paste options
 */
const enum PasteType {
    /**
     * Default paste behavior
     */
    Normal = 0,
    /**
     * Paste only the plain text
     */
    AsPlainText = 1,
    /**
     * Apply the current style to pasted content
     */
    MergeFormat = 2,
    /**
     * If there is a image uri in the clipboard, paste the content as image element
     */
    AsImage = 3
}

    /**
 * Provides a chance for plugin to change the content before it is copied from editor.
 */
interface BeforeCutCopyEvent extends BeforeCutCopyEventData, BasePluginEvent<PluginEventType.BeforeCutCopy> {
}

    /**
 * Data of BeforeCutCopyEvent
 */
interface BeforeCutCopyEventData {
    /**
     * Raw DOM event
     */
    rawEvent: ClipboardEvent;
    /**
     * An object contains all related data for pasting
     */
    clonedRoot: HTMLDivElement;
    /**
     * The selection range under cloned root
     */
    range: Range;
    /**
     * Whether this is a cut event
     */
    isCut: boolean;
}

    /**
 * Provides a chance for plugin to change the content before it is copied from editor.
 */
interface CompatibleBeforeCutCopyEvent extends BeforeCutCopyEventData, BasePluginEvent<CompatiblePluginEventType.BeforeCutCopy> {
}

    /**
 * Editor plugin event interface
 */
interface BasePluginEvent<T extends PluginEventType | CompatiblePluginEventType> {
    /**
     * Type of this event
     */
    eventType: T;
    /**
     * An optional event cache.
     * This will be consumed by event cache API to store some expensive calculation result.
     * So that for the same event across plugins, the result doesn't need to be calculated again
     */
    eventDataCache?: {
        [key: string]: any;
    };
}

    /**
 * Provides a chance for plugin to change the content before it is pasted into editor.
 */
interface BeforeDisposeEvent extends BasePluginEvent<PluginEventType.BeforeDispose> {
}

    /**
 * Provides a chance for plugin to change the content before it is pasted into editor.
 */
interface CompatibleBeforeDisposeEvent extends BasePluginEvent<CompatiblePluginEventType.BeforeDispose> {
}

    /**
 * Provides a chance for plugin to change the content before it is pasted into editor.
 */
interface BeforePasteEvent extends BeforePasteEventData, BasePluginEvent<PluginEventType.BeforePaste> {
}

    /**
 * Data of BeforePasteEvent
 */
interface BeforePasteEventData {
    /**
     * An object contains all related data for pasting
     */
    clipboardData: ClipboardData;
    /**
     * HTML Document Fragment which will be inserted into content
     */
    fragment: DocumentFragment;
    /**
     * Html sanitizing options. This will be used for sanitizing the html content before paste
     */
    sanitizingOption: Required<HtmlSanitizerOptions>;
    /**
     * Stripped HTML string before "StartFragment" comment
     */
    htmlBefore: string;
    /**
     * Stripped HTML string after "EndFragment" comment
     */
    htmlAfter: string;
    /**
     * Attributes of the root "HTML" tag
     */
    htmlAttributes: Record<string, string>;
    /**
     * Paste type option (as plain text, merge format, normal, as image)
     */
    readonly pasteType: PasteType | CompatiblePasteType;
}

    /**
 * Provides a chance for plugin to change the content before it is pasted into editor.
 */
interface CompatibleBeforePasteEvent extends BeforePasteEventData, BasePluginEvent<CompatiblePluginEventType.BeforePaste> {
}

    /**
 * The event to be triggered before SetContent API is called.
 * Handle this event to cache anything you need from editor before it is gone.
 */
interface BeforeSetContentEvent extends BeforeSetContentEventData, BasePluginEvent<PluginEventType.BeforeSetContent> {
}

    /**
 * Data of BeforeSetContentEvent
 */
interface BeforeSetContentEventData {
    /**
     * New content HTML that is about to set to editor
     */
    newContent: string;
}

    /**
 * The event to be triggered before SetContent API is called.
 * Handle this event to cache anything you need from editor before it is gone.
 */
interface CompatibleBeforeSetContentEvent extends BeforeSetContentEventData, BasePluginEvent<CompatiblePluginEventType.BeforeSetContent> {
}

    /**
 * Represents a change to the editor made by another plugin
 */
interface ContentChangedEvent extends ContentChangedEventData, BasePluginEvent<PluginEventType.ContentChanged> {
}

    /**
 * Data of ContentChangedEvent
 */
interface ContentChangedEventData {
    /**
     * Source of the change
     */
    source: ChangeSource | CompatibleChangeSource | string;
    /**
     * Optional related data
     */
    data?: any;
    additionalData?: ContentChangedData;
}

    /**
 * Represents a change to the editor made by another plugin
 */
interface CompatibleContentChangedEvent extends ContentChangedEventData, BasePluginEvent<CompatiblePluginEventType.ContentChanged> {
}

    /**
 * Represents an event that will be fired when an inline image is edited by user, and the src
 * attribute of the image is about to be changed
 */
interface EditImageEvent extends EditImageEventData, BasePluginEvent<PluginEventType.EditImage> {
}

    /**
 * Data of EditImageEvent
 */
interface EditImageEventData {
    /**
     * The image element that is being changed
     */
    readonly image: HTMLImageElement;
    /**
     * Original src of the image before all editing in current editor session.
     */
    readonly originalSrc: string;
    /**
     * Src of the image before current batch of editing
     * Plugin can check this value to know which image is not used after the change.
     */
    readonly previousSrc: string;
    /**
     * New src of the changed image, in DataUri format.
     * Plugin can modify this string so that the modified one will be set to the image element
     */
    newSrc: string;
}

    /**
 * Represents an event that will be fired when an inline image is edited by user, and the src
 * attribute of the image is about to be changed
 */
interface CompatibleEditImageEvent extends EditImageEventData, BasePluginEvent<CompatiblePluginEventType.EditImage> {
}

    /**
 * Provides a chance for plugin to change the content before it is pasted into editor.
 */
interface EditorReadyEvent extends BasePluginEvent<PluginEventType.EditorReady> {
}

    /**
 * Provides a chance for plugin to change the content before it is pasted into editor.
 */
interface CompatibleEditorReadyEvent extends BasePluginEvent<CompatiblePluginEventType.EditorReady> {
}

    /**
 * Provide a chance for plugins to handle entity related events.
 * See enum EntityOperation for more details about each operation
 */
interface EntityOperationEvent extends EntityOperationEventData, BasePluginEvent<PluginEventType.EntityOperation> {
}

    /**
 * Data of EntityOperationEvent
 */
interface EntityOperationEventData {
    /**
     * Operation to this entity
     */
    operation: EntityOperation | CompatibleEntityOperation;
    /**
     * The entity that editor is operating on
     */
    entity: Entity;
    /**
     * Optional raw event. Need to do null check before use its value
     */
    rawEvent?: Event;
    /**
     * For EntityOperation.UpdateEntityState, we use this object to pass the new entity state to plugin.
     * For other operation types, it is not used.
     */
    state?: string;
    /**
     * For EntityOperation.NewEntity, plugin can set this property to true then the entity will be persisted.
     * A persisted entity won't be touched during undo/redo, unless it does not exist after undo/redo.
     * For other operation types, this value will be ignored.
     */
    shouldPersist?: boolean;
    /**
     * @deprecated
     */
    contentForShadowEntity?: DocumentFragment;
}

    /**
 * Provide a chance for plugins to handle entity related events.
 * See enum EntityOperation for more details about each operation
 */
interface CompatibleEntityOperationEvent extends EntityOperationEventData, BasePluginEvent<CompatiblePluginEventType.EntityOperation> {
}

    /**
 * Extract Content with a DOM tree event
 * This event is triggered when getContent() is called with triggerExtractContentEvent = true
 * Plugin can handle this event to remove the UI only markups to return clean HTML
 * by operating on a cloned DOM tree
 */
interface ExtractContentWithDomEvent extends ExtractContentWithDomEventData, BasePluginEvent<PluginEventType.ExtractContentWithDom> {
}

    /**
 * Data of ExtractContentWithDomEvent
 */
interface ExtractContentWithDomEventData {
    /**
     * Cloned root element of editor
     * Plugin can change this DOM tree to clean up the markups it added before
     */
    clonedRoot: HTMLElement;
}

    /**
 * Extract Content with a DOM tree event
 * This event is triggered when getContent() is called with triggerExtractContentEvent = true
 * Plugin can handle this event to remove the UI only markups to return clean HTML
 * by operating on a cloned DOM tree
 */
interface CompatibleExtractContentWithDomEvent extends ExtractContentWithDomEventData, BasePluginEvent<CompatiblePluginEventType.ExtractContentWithDom> {
}

    /**
 * An event fired when pending format state (bold, italic, underline, ... with collapsed selection) is changed
 */
interface PendingFormatStateChangedEvent extends BasePluginEvent<PluginEventType.PendingFormatStateChanged> {
    /**
     * The new format state to apply. If null is passed, clear existing pending format state if any
     */
    formatState: PendableFormatState;
    /**
     * A callback to do format change to a temp element. This is used for style-based format such as font and color
     */
    formatCallback?: (element: HTMLElement, isInnerNode?: boolean) => any;
}

    /**
 * An event fired when pending format state (bold, italic, underline, ... with collapsed selection) is changed
 */
interface CompatiblePendingFormatStateChangedEvent extends BasePluginEvent<CompatiblePluginEventType.PendingFormatStateChanged> {
    /**
     * The new format state to apply. If null is passed, clear existing pending format state if any
     */
    formatState: PendableFormatState;
    /**
     * A callback to do format change to a temp element. This is used for style-based format such as font and color
     */
    formatCallback?: (element: HTMLElement, isInnerNode?: boolean) => any;
}

    /**
 * This represents a PluginEvent wrapping native browser event
 */
type PluginDomEvent = PluginCompositionEvent | PluginMouseEvent | PluginKeyboardEvent | PluginInputEvent | PluginScrollEvent;

    /**
 * A base interface of all DOM events
 */
interface PluginDomEventBase<TEventType extends PluginEventType | CompatiblePluginEventType, TRawEvent extends Event> extends BasePluginEvent<TEventType> {
    rawEvent: TRawEvent;
}

    /**
 * This interface represents a PluginEvent wrapping native CompositionEnd event
 */
interface PluginCompositionEvent extends PluginDomEventBase<PluginEventType.CompositionEnd, CompositionEvent> {
}

    /**
 * This interface represents a PluginEvent wrapping native ContextMenu event
 */
interface PluginContextMenuEvent extends PluginContextMenuEventData, PluginDomEventBase<PluginEventType.ContextMenu, MouseEvent> {
}

    /**
 * The interface represents a PluginEvent wrapping native Keyboard event
 */
type PluginKeyboardEvent = PluginKeyDownEvent | PluginKeyPressEvent | PluginKeyUpEvent;

    /**
 * This interface represents a PluginEvent wrapping native KeyDown event
 */
interface PluginKeyDownEvent extends PluginKeyDownEventData, PluginDomEventBase<PluginEventType.KeyDown, KeyboardEvent> {
}

    /**
 * This interface represents a PluginEvent wrapping native KeyPress event
 */
interface PluginKeyPressEvent extends PluginDomEventBase<PluginEventType.KeyPress, KeyboardEvent> {
}

    /**
 * This interface represents a PluginEvent wrapping native KeyUp event
 */
interface PluginKeyUpEvent extends PluginDomEventBase<PluginEventType.KeyUp, KeyboardEvent> {
}

    /**
 * This interface represents a PluginEvent wrapping native Mouse event
 */
type PluginMouseEvent = PluginMouseDownEvent | PluginMouseUpEvent | PluginContextMenuEvent;

    /**
 * This interface represents a PluginEvent wrapping native MouseDown event
 */
interface PluginMouseDownEvent extends PluginDomEventBase<PluginEventType.MouseDown, MouseEvent> {
}

    /**
 * This interface represents a PluginEvent wrapping native MouseUp event
 */
interface PluginMouseUpEvent extends PluginMouseUpEventData, PluginDomEventBase<PluginEventType.MouseUp, MouseEvent> {
}

    /**
 * This interface represents a PluginEvent wrapping native input / textinput event
 */
interface PluginInputEvent extends PluginDomEventBase<PluginEventType.Input, InputEvent> {
}

    /**
 * This interface represents a PluginEvent wrapping native scroll event
 */
interface PluginScrollEvent extends PluginScrollEventData, PluginDomEventBase<PluginEventType.Scroll, Event> {
}

    /**
 * This represents a PluginEvent wrapping native browser event
 */
type CompatiblePluginDomEvent = CompatiblePluginCompositionEvent | CompatiblePluginMouseEvent | CompatiblePluginKeyboardEvent | CompatiblePluginInputEvent | CompatiblePluginScrollEvent;

    /**
 * This interface represents a PluginEvent wrapping native CompositionEnd event
 */
interface CompatiblePluginCompositionEvent extends PluginDomEventBase<CompatiblePluginEventType.CompositionEnd, CompositionEvent> {
}

    /**
 * This interface represents a PluginEvent wrapping native ContextMenu event
 */
interface CompatiblePluginContextMenuEvent extends PluginContextMenuEventData, PluginDomEventBase<CompatiblePluginEventType.ContextMenu, MouseEvent> {
}

    /**
 * The interface represents a PluginEvent wrapping native Keyboard event
 */
type CompatiblePluginKeyboardEvent = CompatiblePluginKeyDownEvent | CompatiblePluginKeyPressEvent | CompatiblePluginKeyUpEvent;

    /**
 * This interface represents a PluginEvent wrapping native KeyDown event
 */
interface CompatiblePluginKeyDownEvent extends PluginKeyDownEventData, PluginDomEventBase<CompatiblePluginEventType.KeyDown, KeyboardEvent> {
}

    /**
 * This interface represents a PluginEvent wrapping native KeyPress event
 */
interface CompatiblePluginKeyPressEvent extends PluginDomEventBase<CompatiblePluginEventType.KeyPress, KeyboardEvent> {
}

    /**
 * This interface represents a PluginEvent wrapping native KeyUp event
 */
interface CompatiblePluginKeyUpEvent extends PluginDomEventBase<CompatiblePluginEventType.KeyUp, KeyboardEvent> {
}

    /**
 * This interface represents a PluginEvent wrapping native Mouse event
 */
type CompatiblePluginMouseEvent = CompatiblePluginMouseDownEvent | CompatiblePluginMouseUpEvent | CompatiblePluginContextMenuEvent;

    /**
 * This interface represents a PluginEvent wrapping native MouseDown event
 */
interface CompatiblePluginMouseDownEvent extends PluginDomEventBase<CompatiblePluginEventType.MouseDown, MouseEvent> {
}

    /**
 * This interface represents a PluginEvent wrapping native MouseUp event
 */
interface CompatiblePluginMouseUpEvent extends PluginMouseUpEventData, PluginDomEventBase<CompatiblePluginEventType.MouseUp, MouseEvent> {
}

    /**
 * This interface represents a PluginEvent wrapping native input / textinput event
 */
interface CompatiblePluginInputEvent extends PluginDomEventBase<CompatiblePluginEventType.Input, InputEvent> {
}

    /**
 * This interface represents a PluginEvent wrapping native scroll event
 */
interface CompatiblePluginScrollEvent extends PluginScrollEventData, PluginDomEventBase<CompatiblePluginEventType.Scroll, UIEvent> {
}

    /**
 * Data of PluginScrollEvent
 */
interface PluginScrollEventData {
    /**
     * Current scroll container that triggers this scroll event
     */
    scrollContainer: HTMLElement;
}

    /**
 * Data of PluginMouseUpEvent
 */
interface PluginMouseUpEventData {
    /**
     * Whether this is a mouse click event (mouse up and down on the same position)
     */
    isClicking?: boolean;
}

    /**
 * Data of PluginContextMenuEvent
 */
interface PluginContextMenuEventData {
    /**
     * A callback array to let editor retrieve context menu item related to this event.
     * Plugins can add their own getter callback to this array,
     * items from each getter will be separated by a splitter item represented by null
     */
    items: any[];
}

    /**
 * Data of PluginKeyDownEvent
 */
interface PluginKeyDownEventData {
    /**
     * Whether this event is handled by edit feature
     */
    handledByEditFeature?: boolean;
}

    /**
 * Editor plugin event interface
 */
type PluginEvent = BeforeCutCopyEvent | BeforePasteEvent | ContentChangedEvent | EntityOperationEvent | ExtractContentWithDomEvent | PluginDomEvent | EditorReadyEvent | BeforeDisposeEvent | PendingFormatStateChangedEvent | EnterShadowEditEvent | LeaveShadowEditEvent | EditImageEvent | BeforeSetContentEvent | ZoomChangedEvent | SelectionChangedEvent | BeforeKeyboardEditingEvent | CompatibleBeforeCutCopyEvent | CompatibleBeforeDisposeEvent | CompatibleBeforePasteEvent | CompatibleBeforeSetContentEvent | CompatibleContentChangedEvent | CompatibleEditImageEvent | CompatibleEditorReadyEvent | CompatibleEntityOperationEvent | CompatibleExtractContentWithDomEvent | CompatiblePendingFormatStateChangedEvent | CompatiblePluginDomEvent | CompatibleEnterShadowEditEvent | CompatibleLeaveShadowEditEvent | CompatibleZoomChangedEvent | CompatibleSelectionChangedEvent | CompatibleBeforeKeyboardEditingEvent;

    /**
 * A type to extract data part of a plugin event type. Data part is the plugin event without eventType field.
 */
type PluginEventData<T extends PluginEventType | CompatiblePluginEventType> = PluginEventDataGeneric<PluginEvent, T>;

    /**
 * A type to extract data part of a plugin event type. Data part is the plugin event without eventType field.
 * This type is a middle result and only used by PluginEventData type
 */
type PluginEventDataGeneric<E extends PluginEvent, T extends PluginEventType | CompatiblePluginEventType> = E extends BasePluginEvent<T> ? Pick<E, Exclude<keyof E, 'eventType'>> : never;

    /**
 * A type to get specify plugin event type from eventType parameter.
 */
type PluginEventFromType<T extends PluginEventType | CompatiblePluginEventType> = PluginEventFromTypeGeneric<PluginEvent, T>;

    /**
 * A type to get specify plugin event type from eventType parameter.
 * This type is a middle result and only used by PluginEventFromType type
 */
type PluginEventFromTypeGeneric<E extends PluginEvent, T extends PluginEventType | CompatiblePluginEventType> = E extends BasePluginEvent<T> ? E : never;

    /**
 * A plugin triggered right after editor has entered Shadow Edit mode
 */
interface EnterShadowEditEvent extends EnterShadowEditEventData, BasePluginEvent<PluginEventType.EnteredShadowEdit> {
}

    /**
 * A plugin triggered right before editor leave Shadow Edit mode
 */
interface LeaveShadowEditEvent extends BasePluginEvent<PluginEventType.LeavingShadowEdit> {
}

    /**
 * Data of EnterShadowEditEvent
 */
interface EnterShadowEditEventData {
    /**
     * The document fragment of original editor content
     */
    fragment: DocumentFragment;
    /**
     * The selection path of original editor content
     */
    selectionPath: SelectionPath | null;
}

    /**
 * A plugin triggered right after editor has entered Shadow Edit mode
 */
interface CompatibleEnterShadowEditEvent extends EnterShadowEditEventData, BasePluginEvent<CompatiblePluginEventType.EnteredShadowEdit> {
}

    /**
 * A plugin triggered right before editor leave Shadow Edit mode
 */
interface CompatibleLeaveShadowEditEvent extends BasePluginEvent<CompatiblePluginEventType.LeavingShadowEdit> {
}

    /**
 * Represents an event object triggered from Editor.setZoomScale() API.
 * Plugins can handle this event when they need to do something for zoom changing.
 *
 */
interface ZoomChangedEvent extends ZoomChangedEventData, BasePluginEvent<PluginEventType.ZoomChanged> {
}

    /**
 * Data of ZoomChangedEvent
 */
interface ZoomChangedEventData {
    /**
     * Zoom scale value before this change
     */
    oldZoomScale: number;
    /**
     * Zoom scale value after this change
     */
    newZoomScale: number;
}

    /**
 * Represents an event object triggered from Editor.setZoomScale() API.
 * Plugins can handle this event when they need to do something for zoom changing.
 *
 */
interface CompatibleZoomChangedEvent extends ZoomChangedEventData, BasePluginEvent<CompatiblePluginEventType.ZoomChanged> {
}

    /**
 * Represents an event that will be fired when the user changed the selection
 */
interface SelectionChangedEvent extends SelectionChangedEventData, BasePluginEvent<PluginEventType.SelectionChanged> {
}

    /**
 * Data of SelectionChangedEvent
 */
interface SelectionChangedEventData {
    /**
     * Information of the selection
     */
    selectionRangeEx: SelectionRangeEx | null;
}

    /**
 * Represents an event that will be fired when the user changed the selection
 */
interface CompatibleSelectionChangedEvent extends SelectionChangedEventData, BasePluginEvent<CompatiblePluginEventType.SelectionChanged> {
}

    /**
 * Provides a chance for plugin to change the content before it is copied from editor.
 */
interface BeforeKeyboardEditingEvent extends BeforeKeyboardEditingData, BasePluginEvent<PluginEventType.BeforeKeyboardEditing> {
}

    /**
 * Data of BeforeKeyboardEditing
 */
interface BeforeKeyboardEditingData {
    /**
     * Raw DOM event
     */
    rawEvent: KeyboardEvent;
}

    /**
 * Provides a chance for plugin to change the content before it is copied from editor.
 */
interface CompatibleBeforeKeyboardEditingEvent extends BeforeKeyboardEditingData, BasePluginEvent<CompatiblePluginEventType.BeforeKeyboardEditing> {
}

    /**
 * Represents data, that can be used to announce text to screen reader.
 */
interface AnnounceData {
    /**
     * @optional Default announce strings built in Rooster
     */
    defaultStrings?: KnownAnnounceStrings | CompatibleKnownAnnounceStrings;
    /**
     * @optional string to announce from this Content Changed event, will be the fallback value if default string
     * is not provided or if it is not found in the strings map.
     */
    text?: string;
    /**
     * @optional if provided, will attempt to replace {n} with each of the values inside of the array.
     */
    formatStrings?: string[];
}

    /**
 * This refers to a "content block" in editor that serves as a content parsing boundary
 * It is most those html block like tags, i.e. &lt;p&gt;, &lt;div&gt;, &lt;li&gt;, &lt;td&gt; etc.
 * but can also be just a text node, followed by a &lt;br&gt;, i.e.
 * for html fragment &lt;div&gt;abc&lt;br&gt;123&lt;/div&gt;, abc&lt;br&gt; is a block, 123 is another block
 */
interface BlockElement {
    /**
     * Collapse this block element to a single DOM element.
     */
    collapseToSingleElement(): HTMLElement;
    /**
     * Get start node of this block element
     */
    getStartNode(): Node;
    /**
     * Get end node of this block element
     */
    getEndNode(): Node;
    /**
     * Check whether this block element equals to the given block element
     */
    equals(blockElement: BlockElement): boolean;
    /**
     * Checks if this block element is after another block element
     */
    isAfter(blockElement: BlockElement): boolean;
    /**
     * Check if the given node is within this block element
     */
    contains(node: Node): boolean;
    /**
     * Get the text content of this block element
     */
    getTextContent(): string;
}

    /**
 * An object contains all related data for pasting
 */
interface ClipboardData {
    /**
     * Available types from clipboard event
     */
    types: string[];
    /**
     * Plain text from clipboard event
     */
    text: string;
    /**
     * HTML string from clipboard event.
     * When set to null, it means there's no HTML from clipboard event.
     * When set to undefined, it means there may be HTML in clipboard event, but fail to retrieve
     */
    rawHtml: string | null | undefined;
    /**
     * Link Preview information provided by Edge
     */
    linkPreview?: EdgeLinkPreview;
    /**
     * Image file from clipboard event
     */
    image: File | null;
    /**
     * General file from clipboard event
     */
    files?: File[];
    /**
     * Html extracted from raw html string and remove content before and after fragment tag
     */
    html?: string;
    /**
     * An editor content snapshot before pasting happens. This is used for changing paste format
     */
    snapshotBeforePaste?: string;
    /**
     * BASE64 encoded data uri of the image if any
     */
    imageDataUri?: string | null;
    /**
     * Array of tag names of the first level child nodes
     */
    htmlFirstLevelChildTags?: string[];
    /**
     * Value of custom paste type. By default it is always empty.
     * To allow custom paste type, pass the allowed types to EditorOptions.allowedCustomPasteType
     */
    customValues: Record<string, string>;
    /**
     * If true, the event was triggered by a native paste event (keyboard or native context menu paste)
     */
    readonly pasteNativeEvent?: boolean;
}

    /**
 * An extended Editor plugin interface which supports providing context menu items
 */
interface ContextMenuProvider<T> extends EditorPlugin  {
    getContextMenuItems: (target: Node) => T[] | null;
}

    /**
 * Custom data stored in editor
 */
interface CustomData {
    /**
     * Value of this custom data
     */
    value: any;
    /**
     * Optional disposer function of the custom data.
     * When a valid value is set, it will be invoked when editor is disposing
     */
    disposer?: (value: any) => void;
}

    /**
 * Property that is going to store additional data related to the Content Changed Event
 */
interface ContentChangedData {
    /**
     * Optional property to store the format api name when using ChangeSource.Format
     */
    formatApiName?: string;
    /**
     * @optional Get entity states related to the snapshot. If it returns entity states, each state will cause
     * an EntityOperation event with operation = EntityOperation.UpdateEntityState when undo/redo to this snapshot
     * @returns Related entity state array
     */
    getEntityState?: () => EntityState[];
    /**
     * @optional
     * Get Announce data from this content changed event.
     * @returns
     */
    getAnnounceData?: () => AnnounceData | undefined;
}

    /**
 * Default format settings
 */
interface DefaultFormat {
    /**
     * Font family
     */
    fontFamily?: string;
    /**
     * Font size
     */
    fontSize?: string;
    /**
     * Single text color (for non dark mode/single mode editor)
     */
    textColor?: string;
    /**
     * Text color light/dark mode pair
     */
    textColors?: ModeIndependentColor;
    /**
     * Single background color (for non dark mode/single mode editor)
     */
    backgroundColor?: string;
    /**
     * Background color light/dark mode pair
     */
    backgroundColors?: ModeIndependentColor;
    /**
     * Whether is bold
     */
    bold?: boolean;
    /**
     * Whether is italic
     */
    italic?: boolean;
    /**
     * Whether has underline
     */
    underline?: boolean;
}

    /**
 * Represents an entity in editor.
 * An entity is an HTML element with special CSS classes and will be treated as an atom from the editor.
 * Editor provides basic operations and will trigger related events to let plugins know that there is
 * operation target to an entity.
 * Plugin can insert/remove an entity, handle entity events, query entities within the editor.
 * Plugin can also set an entity to be readonly so that user can't modify content within an entity.
 */
interface Entity {
    /**
     * Type of this entity. Specified when insert an entity, can be an valid CSS class-like string.
     */
    type: string;
    /**
     * Id of this entity, generated by editor code and will be unique within an editor
     */
    id: string;
    /**
     * The wrapper DOM node of this entity which holds the info CSS classes of this entity
     */
    wrapper: HTMLElement;
    /**
     * Whether this is a readonly entity
     */
    isReadonly: boolean;
}

    /**
 * The format state
 */
interface FormatState extends PendableFormatState, ElementBasedFormatState, StyleBasedFormatState, EditorUndoState  {
    /**
     * Whether editor is in dark mode
     */
    isDarkMode?: boolean;
    /**
     * Current zoom scale of editor
     */
    zoomScale?: number;
}

    /**
 * Format states that can have pending state.
 *
 * e.g., When using execCommand('bold') target to a collapsed selection, browser will enter bold state,
 * but there isn't a &lt;B&gt; tag until user type something, or the state will be rollback if selection
 * is changed.
 */
interface PendableFormatState {
    /**
     * Whether the text is bolded
     */
    isBold?: boolean;
    /**
     * Whether the text is italic
     */
    isItalic?: boolean;
    /**
     * Whether the text has underline
     */
    isUnderline?: boolean;
    /**
     * Whether the text has strike through line
     */
    isStrikeThrough?: boolean;
    /**
     * Whether the text is in subscript mode
     */
    isSubscript?: boolean;
    /**
     * Whether the text is in superscript mode
     */
    isSuperscript?: boolean;
}

    /**
 * Format state represented by DOM element
 */
interface ElementBasedFormatState {
    /**
     * Whether the text is in bullet mode
     */
    isBullet?: boolean;
    /**
     * Whether the text is in numbering mode
     */
    isNumbering?: boolean;
    /**
     * Whether the text is in block quote
     */
    isBlockQuote?: boolean;
    /**
     * Whether the text is in Code element
     */
    isCodeInline?: boolean;
    /**
     * Whether the text is in Code block
     */
    isCodeBlock?: boolean;
    /**
     * Whether unlink command can be called to the text
     */
    canUnlink?: boolean;
    /**
     * Whether the selected text is multiline
     */
    isMultilineSelection?: boolean;
    /**
     * Whether add image alt text command can be called to the text
     */
    canAddImageAltText?: boolean;
    /**
     * Heading level (0-6, 0 means no heading)
     */
    headingLevel?: number;
    /**
     * @deprecated Use headingLevel instead
     * Heading level (0-6, 0 means no heading)
     */
    headerLevel?: number;
    /**
     * Whether the cursor is in table
     */
    isInTable?: boolean;
    /**
     * Format of table, if there is table at cursor position
     */
    tableFormat?: TableFormat;
    /**
     * If there is a table, whether the table has header row
     */
    tableHasHeader?: boolean;
    /**
     * Whether we can execute table cell merge operation
     */
    canMergeTableCell?: boolean;
}

    /**
 * Format states represented by CSS style
 */
interface StyleBasedFormatState {
    /**
     * Font name
     */
    fontName?: string;
    /**
     * Font size
     */
    fontSize?: string;
    /**
     * Background color
     */
    backgroundColor?: string;
    /**
     * Mode independent background color for dark mode
     */
    backgroundColors?: ModeIndependentColor;
    /**
     * Text color
     */
    textColor?: string;
    /**
     * Mode independent background color for dark mode
     */
    textColors?: ModeIndependentColor;
    /**
     * Line height
     */
    lineHeight?: string;
    /**
     * Margin Top
     */
    marginTop?: string;
    /**
     * Margin Bottom
     */
    marginBottom?: string;
    /**
     * Text Align
     */
    textAlign?: string;
    /**
     * Direction of the element ('ltr' or 'rtl')
     */
    direction?: string;
}

    /**
 * Specify if editor can undo/redo an editing operation
 */
interface EditorUndoState {
    /**
     * Whether the content can be undone
     */
    canUndo?: boolean;
    /**
     * Whether the content ca nbe redone
     */
    canRedo?: boolean;
}

    /**
 * @deprecated
 * Options for extractClipboardEvent API
 */
interface ExtractClipboardEventOption extends ExtractClipboardItemsOption, ExtractClipboardItemsForIEOptions  {
}

    /**
 * Options for ExtractClipboardItems API
 */
interface ExtractClipboardItemsOption {
    /**
     * @deprecated This feature is always enabled
     */
    allowLinkPreview?: boolean;
    /**
     * Allowed custom content type when paste besides text/plain, text/html and images
     * Only text types are supported, and do not add "text/" prefix to the type values
     */
    allowedCustomPasteType?: string[];
}

    /**
 * Options for ExtractClipboardItemsForIE API
 */
interface ExtractClipboardItemsForIEOptions {
    /**
     * A callback to help create a temporary DIV for IE to receive pasted content
     */
    getTempDiv?: () => HTMLDivElement;
    /**
     * A callback to help remove the temporary DIV for IE
     */
    removeTempDiv?: (div: HTMLDivElement) => void;
}

    /**
 * Interface of ContentTraverser, provides traversing of content inside editor.
 */
interface IContentTraverser {
    /**
     * Get current block
     */
    currentBlockElement: BlockElement | null;
    /**
     * Get next block element
     */
    getNextBlockElement(): BlockElement | null;
    /**
     * Get previous block element
     */
    getPreviousBlockElement(): BlockElement | null;
    /**
     * Current inline element getter
     */
    currentInlineElement: InlineElement | null;
    /**
     * Get next inline element
     */
    getNextInlineElement(): InlineElement | null;
    /**
     * Get previous inline element
     */
    getPreviousInlineElement(): InlineElement | null;
}

    /**
 * This refers to an inline element (as opposed to block) in editor
 * Inline and block makes the "type" system in editor.
 * An inline element is a maximum resolvable "entity" within the boundary of a block
 * At minimum and also most commonly, it represents a text node.
 * It can represent broader "content" depending on the resolvers that are available, i.e.
 * it can be anchor link, image, emoji, ...
 * Two rules:
 * 1) every inline element must have a container node (text or span, a etc.)
 * 2) inline element cannot be nested
 */
interface InlineElement {
    /**
     * Get the text content of this inline element
     */
    getTextContent(): string;
    /**
     * Get the container node of this inline element
     */
    getContainerNode(): Node;
    /**
     * Get the parent block element of this inline element
     */
    getParentBlock(): BlockElement;
    /**
     * Get the start position of this inline element
     */
    getStartPosition(): NodePosition;
    /**
     * Get the end position of this inline element
     */
    getEndPosition(): NodePosition;
    /**
     * Checks if the given inline element is after this inline element
     */
    isAfter(inlineElement: InlineElement): boolean;
    /**
     * Checks if this inline element is a textual inline element
     */
    isTextualInlineElement(): boolean;
    /**
     * Checks if the given editor position is contained in this inline element
     */
    contains(position: NodePosition): boolean;
    /**
     * Apply inline style to a region of an inline element
     */
    applyStyle(styler: (element: HTMLElement, isInnerNode?: boolean) => any): void;
}

    /**
 * Type definition for the InsertOption, used in the insertNode API.
 * The position parameter defines how the node will be inserted.
 * In a future revision, this will become strongly typed
 * Only parameters applicable to the given position will be accepted.
 */
type InsertOption = InsertOptionRange | InsertOptionBasic;

    /**
 * Shared options for insertNode related APIs
 */
interface InsertOptionBase {
    /**
     * Whether need to update cursor.
     */
    updateCursor?: boolean;
    /**
     * Boolean flag for inserting the content onto a new line.
     * No-op for ContentPosition.Outside
     */
    insertOnNewLine?: boolean;
    /**
     * Boolean flag for inserting the content onto a new line.
     * No-op for ContentPosition.Begin, End, and Outside
     */
    replaceSelection?: boolean;
    /**
     * Boolean flag for inserting the content onto root node of current region.
     * If current position is not at root of region, break parent node until insert can happen at root of region.
     * This option only takes effect when insertOnNewLine is true, otherwise it will be ignored.
     */
    insertToRegionRoot?: boolean;
}

    /**
 * The "basic" insertNode related ContentPositions that require no additional parameters to use.
 */
interface InsertOptionBasic extends InsertOptionBase  {
    position: ContentPosition.Begin | ContentPosition.End | ContentPosition.DomEnd | ContentPosition.Outside | ContentPosition.SelectionStart | CompatibleContentPosition.Begin | CompatibleContentPosition.End | CompatibleContentPosition.DomEnd | CompatibleContentPosition.Outside | CompatibleContentPosition.SelectionStart;
}

    /**
 * The Range variant where insertNode will operate on a range disjointed from the current selection state.
 */
interface InsertOptionRange extends InsertOptionBase  {
    position: ContentPosition.Range | CompatibleContentPosition.Range;
    /**
     * The range to be targeted when insertion happens.
     */
    range: Range;
}

    /**
 * The class that helps search content around a position
 */
interface IPositionContentSearcher {
    /**
     * Get the word before position. The word is determined by scanning backwards till the first white space, the portion
     * between position and the white space is the word before position
     * @returns The word before position
     */
    getWordBefore(): string;
    /**
     * Get the inline element before position
     * @returns The inlineElement before position
     */
    getInlineElementBefore(): InlineElement | null;
    /**
     * Get the inline element after position
     * @returns The inline element after position
     */
    getInlineElementAfter(): InlineElement | null;
    /**
     * Get X number of chars before position
     * The actual returned chars may be less than what is requested.
     * @param length The length of string user want to get, the string always ends at the position,
     * so this length determines the start position of the string
     * @returns The actual string we get as a sub string, or the whole string before position when
     * there is not enough chars in the string
     */
    getSubStringBefore(length: number): string;
    /**
     * Try to get a range matches the given text before the position
     * @param text The text to match against
     * @param exactMatch Whether it is an exact match
     * @returns The range for the matched text, null if unable to find a match
     */
    getRangeFromText(text: string, exactMatch: boolean): Range | null;
    /**
     * Get text section before position till stop condition is met.
     * This offers consumers to retrieve text section by section
     * The section essentially is just an inline element which has Container element
     * so that the consumer can remember it for anchoring popup or verification purpose
     * when position moves out of context etc.
     * @param stopFunc The callback stop function
     */
    forEachTextInlineElement(callback: (textInlineElement: InlineElement) => any): void;
    /**
     * Get first non textual inline element before position
     * @returns First non textual inline element before position or null if no such element exists
     */
    getNearestNonTextInlineElement(): InlineElement | null;
}

    /**
 * LinkData represents a link match result
 */
interface LinkData {
    /**
     * Schema of a hyperlink
     */
    scheme: string;
    /**
     * Original url of a hyperlink
     */
    originalUrl: string;
    /**
     * Normalized url of a hyperlink
     */
    normalizedUrl: string;
}

    /**
 * A color object contains both light mode and dark mode color
 */
interface ModeIndependentColor {
    /**
     * The color to be used in dark mode, if enabled.
     */
    darkModeColor: string;
    /**
     * The color to be used in light mode, or stored as the original color in dark mode.
     */
    lightModeColor: string;
}

    /**
 * Represent a position in DOM tree by the node and its offset index
 */
interface NodePosition {
    readonly node: Node;
    readonly element: HTMLElement;
    readonly offset: number;
    readonly isAtEnd: boolean;
    /**
     * Normalize this position to the leaf node, return the normalize result.
     * If current position is already using leaf node, return this position object itself
     */
    normalize(): NodePosition;
    /**
     * Check if this position is equal to the given position
     * @param position The position to check
     */
    equalTo(position: NodePosition): boolean;
    /**
     * Checks if this position is after the given position
     * @param position The position to check
     */
    isAfter(position: NodePosition): boolean;
    /**
     * Move this position with offset, returns a new position with a valid offset in the same node
     * @param offset Offset to move with
     */
    move(offset: number): NodePosition;
}

    /**
 * This represents a rect inside editor
 */
interface Rect {
    /**
     * Top
     */
    top: number;
    /**
     * Bottom
     */
    bottom: number;
    /**
     * Left
     */
    left: number;
    /**
     * Right
     */
    right: number;
}

    /**
 * Represents a DOM region with selection information
 * For more info about Region, see RegionBase
 */
interface Region extends RegionBase  {
    /**
     * Start position of full selection. It is possible to be out of this region.
     * We use this position and fullSelectionEnd to calculate the selected range inside this region.
     */
    fullSelectionStart: NodePosition;
    /**
     * End position of full selection. It is possible to be out of this region.
     * We use this position and fullSelectionStart to calculate the selected range inside this region.
     */
    fullSelectionEnd: NodePosition;
}

    /**
 * Represents a DOM region.
 * A region is a range under a given node, and possibly after one child and before another child
 * e.g.
 * ```html
 * <div>
 *   <table>...</table>
 *   <span>...</span>
 *   <span>...</span>
 *   <table>...</table>
 * <div>
 * ```
 * We can define a region under DIV before the second TABLE and after the first TABLE
 *
 * This is used when user's selection go across different TD elements or start from TD and end after
 * that TD (or inverse way). Some block operation should not ruin the TABLE structure, so we need to
 * split the selection into several regions.
 */
interface RegionBase {
    /**
     * Root node of this region
     */
    rootNode: HTMLElement;
    /**
     * A node to define the beginning boundary of this region.
     * All nodes after this node before nodeAfter and contained by rootNode will be treated contained by this region.
     */
    nodeBefore?: Node;
    /**
     * A node to define the ending boundary of this region.
     * All nodes before this node after nodeBefore and contained by rootNode will be treated contained by this region.
     */
    nodeAfter?: Node;
    /**
     * Tags that child elements will be skipped
     */
    skipTags: string[];
}

    /**
 * A start and end pair of selection path
 */
interface SelectionPath {
    /**
     * Start path of selection
     */
    start: number[];
    /**
     * End path of selection
     */
    end: number[];
}

    /**
 * Represents a data structure of snapshots, this is usually used for undo snapshots
 */
interface Snapshots<T = string> {
    /**
     * The snapshot array
     */
    snapshots: T[];
    /**
     * Size of all snapshots
     */
    totalSize: number;
    /**
     * Current index
     */
    currentIndex: number;
    /**
     * Index of snapshot added before an auto complete action
     */
    autoCompleteIndex: number;
    /**
     * Max size of all snapshots
     */
    readonly maxSize: number;
}

    /**
 * Common part of NormalContentMetadata and TableContentMetadata
 */
interface ContentMetadataBase<T extends SelectionRangeTypes> {
    isDarkMode: boolean;
    type: T;
}

    /**
 * A content metadata is a data structure storing information other than HTML content,
 * such as dark mode info, selection info, ...
 *
 * NormalContentMetadata is content metadata for normal selection with a start and end selection path.
 *
 * When do any change to this type, also need to fix function isUndoMetadata to make sure
 * the check is correct
 */
interface NormalContentMetadata extends SelectionPath, ContentMetadataBase<SelectionRangeTypes.Normal> {
}

    /**
 * A content metadata is a data structure storing information other than HTML content,
 * such as dark mode info, selection info, ...
 *
 * TableContentMetadata is content metadata for table selection with table id and start and end coordinates
 *
 * When do any change to this type, also need to fix function isUndoMetadata to make sure
 * the check is correct
 */
interface TableContentMetadata extends TableSelection, ContentMetadataBase<SelectionRangeTypes.TableSelection> {
    tableId: string;
}

    /**
 * A content metadata is a data structure storing information other than HTML content,
 * such as dark mode info, selection info, ...
 *
 * ImageContentMetadata is content metadata for image selection with image id
 *
 * When do any change to this type, also need to fix function isUndoMetadata to make sure
 * the check is correct
 */
interface ImageContentMetadata extends ContentMetadataBase<SelectionRangeTypes.ImageSelection> {
    imageId: string;
}

    /**
 * A content metadata is a data structure storing information other than HTML content,
 * such as dark mode info, selection info, ...
 */
type ContentMetadata = NormalContentMetadata | TableContentMetadata | ImageContentMetadata;

    /**
 * A serializable snapshot of editor content, including the html content and metadata
 */
interface Snapshot {
    /**
     * HTML content string
     */
    html: string;
    /**
     * Metadata of the editor content state
     */
    metadata: ContentMetadata | null;
    /**
     * Known colors for dark mode
     */
    knownColors: Readonly<ModeIndependentColor>[];
    /**
     * Entity states related to this undo snapshots. When undo/redo to this snapshot, each entity state will trigger
     * an EntityOperation event with operation = EntityOperation.UpdateEntityState
     */
    entityStates?: EntityState[];
}

    /**
 * State for an entity. This is used for storing entity undo snapshot
 */
interface EntityState {
    /**
     * Type of the entity
     */
    type: string;
    /**
     * Id of the entity
     */
    id: string;
    /**
     * The state of this entity to store into undo snapshot.
     * The state can be any string, or a serialized JSON object.
     * We are using string here instead of a JSON object to make sure the whole state is serializable.
     */
    state: string;
}

    /**
 * Table format
 */
interface TableFormat {
    /**
     * Top border color for each row
     */
    topBorderColor?: string | null;
    /**
     * Bottom border color for each row
     */
    bottomBorderColor?: string | null;
    /**
     * Vertical border color for each row
     */
    verticalBorderColor?: string | null;
    /**
     * Set header row
     */
    hasHeaderRow?: boolean;
    /**
     * Header row background color for even cells
     */
    headerRowColor?: string | null;
    /**
     * Set first column
     */
    hasFirstColumn?: boolean;
    /**
     * Set banded columns
     */
    hasBandedColumns?: boolean;
    /**
     * Set banded rows
     */
    hasBandedRows?: boolean;
    /**
     * Background color for even row or even columns
     */
    bgColorEven?: string | null;
    /**
     * Background color for odd row or odd columns
     */
    bgColorOdd?: string | null;
    /**
     * Table Borders Type
     */
    tableBorderFormat?: TableBorderFormat | CompatibleTableBorderFormat;
    /**
     * If true, the new format will not overlay cells that has color applied
     */
    keepCellShade?: boolean;
    /**
     * Vertical alignment for each row
     */
    verticalAlign?: 'top' | 'middle' | 'bottom' | null;
}

    /**
 * Format of table cell that stored as metadata
 */
type TableCellMetadataFormat = {
    /**
     * Override default background color
     */
    bgColorOverride?: boolean;
    /**
     * Override default vertical align value
     */
    vAlignOverride?: boolean;
};

    /**
 * Represents a selection made inside of a table
 */
interface TableSelection {
    /**
     * First Cell selection x and y position in the table
     */
    firstCell: Coordinates;
    /**
     * Last Cell selection x and y position in the table
     */
    lastCell: Coordinates;
}

    /**
 * Represents 2D Coordinates
 */
interface Coordinates {
    /**
     * X position of the cell
     */
    x: number;
    /**
     * Y position of the cell
     */
    y: number;
}

    /**
 * Options for HtmlSanitizer
 */
interface HtmlSanitizerOptions {
    /**
     * Callbacks for HTML elements
     */
    elementCallbacks?: ElementCallbackMap;
    /**
     * Callbacks for HTML attributes
     */
    attributeCallbacks?: AttributeCallbackMap;
    /**
     * Callbacks for CSS styles
     */
    cssStyleCallbacks?: CssStyleCallbackMap;
    /**
     * Additional tag replacement, to allow replace a tag to another name, or remove it.
     *
     * The value can be:
     * '*': Keep this element with no change
     * '<A valid tag name>: Keep this element but change its tag to the given value
     * null: Remove this element
     *
     * For other unknown tags, we will respect the value of unknownTagReplacement with the same meaning
     */
    additionalTagReplacements?: Record<string, string | null>;
    /**
     * Allowed HTML attributes in addition to default attributes, in lower case
     */
    additionalAllowedAttributes?: string[];
    /**
     * Allowed CSS Class names
     */
    additionalAllowedCssClasses?: string[];
    /**
     * CSS style default values in addition to the default value map, style name should be in lower case
     */
    additionalDefaultStyleValues?: StringMap;
    /**
     * Additional global CSS style nodes
     */
    additionalGlobalStyleNodes?: HTMLStyleElement[];
    /**
     * Additional predefined CSS for element
     */
    additionalPredefinedCssForElement?: PredefinedCssMap;
    /**
     * Preserve HTML comments
     */
    preserveHtmlComments?: boolean;
    /**
     * Define a replacement tag name of unknown tags.
     * A star "*" means keep as it is, no replacement
     * Other valid string means replace the tag name with this string.
     * Empty string, undefined means drop such elements and all its children
     * @default undefined
     */
    unknownTagReplacement?: string | null;
}

    /**
 * @deprecated
 * Options for sanitizeHtml function
 */
interface SanitizeHtmlOptions extends HtmlSanitizerOptions  {
    /**
     * Current HTML element, styles of this element will be used as current style values.
     * Or a string map represents current styles
     */
    currentElementOrStyle?: HTMLElement | StringMap;
    /**
     * When set to true, will only do inline CSS conversion and skip the sanitizing pass
     */
    convertCssOnly?: boolean;
}

    /**
 * Define DOM types of window, used by safeInstanceOf() to check if a given object is of the specified type of its own window
 */
interface TargetWindowBase {
    Range: Range;
    Node: Node;
    ParentNode: ParentNode;
    Element: Element;
    HTMLElement: HTMLElement;
    HTMLUnknownElement: HTMLUnknownElement;
    HTMLUListElement: HTMLUListElement;
    HTMLTrackElement: HTMLTrackElement;
    HTMLTitleElement: HTMLTitleElement;
    HTMLTimeElement: HTMLTimeElement;
    HTMLTextAreaElement: HTMLTextAreaElement;
    HTMLTemplateElement: HTMLTemplateElement;
    HTMLTableSectionElement: HTMLTableSectionElement;
    HTMLTableRowElement: HTMLTableRowElement;
    HTMLTableElement: HTMLTableElement;
    HTMLTableColElement: HTMLTableColElement;
    HTMLTableCellElement: HTMLTableCellElement;
    HTMLTableHeaderCellElement: HTMLTableHeaderCellElement;
    HTMLTableDataCellElement: HTMLTableDataCellElement;
    HTMLTableCaptionElement: HTMLTableCaptionElement;
    HTMLStyleElement: HTMLStyleElement;
    HTMLSpanElement: HTMLSpanElement;
    HTMLSourceElement: HTMLSourceElement;
    HTMLSlotElement: HTMLSlotElement;
    HTMLSelectElement: HTMLSelectElement;
    HTMLScriptElement: HTMLScriptElement;
    HTMLQuoteElement: HTMLQuoteElement;
    HTMLProgressElement: HTMLProgressElement;
    HTMLPreElement: HTMLPreElement;
    HTMLPictureElement: HTMLPictureElement;
    HTMLParamElement: HTMLParamElement;
    HTMLParagraphElement: HTMLParagraphElement;
    HTMLOutputElement: HTMLOutputElement;
    HTMLOptionElement: HTMLOptionElement;
    HTMLOptGroupElement: HTMLOptGroupElement;
    HTMLObjectElement: HTMLObjectElement;
    HTMLOListElement: HTMLOListElement;
    HTMLModElement: HTMLModElement;
    HTMLMeterElement: HTMLMeterElement;
    HTMLMetaElement: HTMLMetaElement;
    HTMLMenuElement: HTMLMenuElement;
    HTMLMediaElement: HTMLMediaElement;
    HTMLVideoElement: HTMLVideoElement;
    HTMLAudioElement: HTMLAudioElement;
    HTMLMarqueeElement: HTMLMarqueeElement;
    HTMLMapElement: HTMLMapElement;
    HTMLLinkElement: HTMLLinkElement;
    HTMLLegendElement: HTMLLegendElement;
    HTMLLabelElement: HTMLLabelElement;
    HTMLLIElement: HTMLLIElement;
    HTMLInputElement: HTMLInputElement;
    HTMLImageElement: HTMLImageElement;
    HTMLIFrameElement: HTMLIFrameElement;
    HTMLHtmlElement: HTMLHtmlElement;
    HTMLHeadingElement: HTMLHeadingElement;
    HTMLHeadElement: HTMLHeadElement;
    HTMLHRElement: HTMLHRElement;
    HTMLFrameSetElement: HTMLFrameSetElement;
    HTMLFrameElement: HTMLFrameElement;
    HTMLFormElement: HTMLFormElement;
    HTMLFontElement: HTMLFontElement;
    HTMLFieldSetElement: HTMLFieldSetElement;
    HTMLEmbedElement: HTMLEmbedElement;
    HTMLDivElement: HTMLDivElement;
    HTMLDirectoryElement: HTMLDirectoryElement;
    HTMLDialogElement: HTMLDialogElement;
    HTMLDetailsElement: HTMLDetailsElement;
    HTMLDataListElement: HTMLDataListElement;
    HTMLDataElement: HTMLDataElement;
    HTMLDListElement: HTMLDListElement;
    HTMLCanvasElement: HTMLCanvasElement;
    HTMLButtonElement: HTMLButtonElement;
    HTMLBodyElement: HTMLBodyElement;
    HTMLBaseElement: HTMLBaseElement;
    HTMLBRElement: HTMLBRElement;
    HTMLAreaElement: HTMLAreaElement;
    HTMLAnchorElement: HTMLAnchorElement;
    DocumentType: DocumentType;
    DocumentFragment: DocumentFragment;
    ShadowRoot: ShadowRoot;
    Document: Document;
    XMLDocument: XMLDocument;
    HTMLDocument: HTMLDocument;
    ChildNode: ChildNode;
    CharacterData: CharacterData;
    Text: Text;
    CDATASection: CDATASection;
    ProcessingInstruction: ProcessingInstruction;
    Comment: Comment;
    Attr: Attr;
}

    /**
 * Define DOM types of window, used by safeInstanceOf() to check if a given object is of the specified type of its own window
 */
interface TargetWindow extends TargetWindowBase  {
    /**
     * @deprecated Just for backward compatibility
     */
    SVGCursorElement: Element;
    /**
     * @deprecated Just for backward compatibility
     */
    HTMLBaseFontElement: Element;
    /**
     * @deprecated Just for backward compatibility
     */
    HTMLAppletElement: Element;
}

    /**
 * Interface of roosterjs editor object
 */
interface IEditor {
    /**
     * Dispose this editor, dispose all plugins and custom data
     */
    dispose(): void;
    /**
     * Get whether this editor is disposed
     * @returns True if editor is disposed, otherwise false
     */
    isDisposed(): boolean;
    /**
     * Insert node into editor
     * @param node The node to insert
     * @param option Insert options. Default value is:
     *  position: ContentPosition.SelectionStart
     *  updateCursor: true
     *  replaceSelection: true
     *  insertOnNewLine: false
     * @returns true if node is inserted. Otherwise false
     */
    insertNode(node: Node, option?: InsertOption): boolean;
    /**
     * Delete a node from editor content
     * @param node The node to delete
     * @returns true if node is deleted. Otherwise false
     */
    deleteNode(node: Node): boolean;
    /**
     * Replace a node in editor content with another node
     * @param existingNode The existing node to be replaced
     * @param toNode node to replace to
     * @param transformColorForDarkMode (optional) Whether to transform new node to dark mode. Default is false
     * @returns true if node is replaced. Otherwise false
     */
    replaceNode(existingNode: Node, toNode: Node, transformColorForDarkMode?: boolean): boolean;
    /**
     * Get BlockElement at given node
     * @param node The node to create InlineElement
     * @returns The BlockElement result
     */
    getBlockElementAtNode(node: Node): BlockElement | null;
    /**
     * Check if the node falls in the editor content
     * @param node The node to check
     * @returns True if the given node is in editor content, otherwise false
     */
    contains(node: Node | null): boolean;
    /**
     * Check if the range falls in the editor content
     * @param range The range to check
     * @returns True if the given range is in editor content, otherwise false
     */
    contains(range: Range): boolean;
    /**
     * Query HTML elements in editor by tag name
     * @param tag Tag name of the element to query
     * @param forEachCallback An optional callback to be invoked on each element in query result
     * @returns HTML Element array of the query result
     */
    queryElements<T extends keyof HTMLElementTagNameMap>(tag: T, forEachCallback?: (node: HTMLElementTagNameMap[T]) => any): HTMLElementTagNameMap[T][];
    /**
     * Query HTML elements in editor by a selector string
     * @param selector Selector string to query
     * @param forEachCallback An optional callback to be invoked on each node in query result
     * @returns HTML Element array of the query result
     */
    queryElements<T extends HTMLElement = HTMLElement>(selector: string, forEachCallback?: (node: T) => any): T[];
    /**
     * Query HTML elements with the given scope by tag name
     * @param tag Tag name of the element to query
     * @param scope The scope of the query, default value is QueryScope.Body
     * @param forEachCallback An optional callback to be invoked on each element in query result
     * @returns HTML Element list of the query result
     */
    queryElements<T extends keyof HTMLElementTagNameMap>(tag: T, scope: QueryScope | CompatibleQueryScope, forEachCallback?: (node: HTMLElementTagNameMap[T]) => any): HTMLElementTagNameMap[T][];
    /**
     * Query HTML elements with the given scope by a selector string
     * @param selector Selector string to query
     * @param scope The scope of the query, default value is QueryScope.Body
     * @param forEachCallback An optional callback to be invoked on each element in query result
     * @returns HTML Element array of the query result
     */
    queryElements<T extends HTMLElement = HTMLElement>(selector: string, scope: QueryScope | CompatibleQueryScope, forEachCallback?: (node: T) => any): T[];
    /**
     * Collapse nodes within the given start and end nodes to their common ancestor node,
     * split parent nodes if necessary
     * @param start The start node
     * @param end The end node
     * @param canSplitParent True to allow split parent node there are nodes before start or after end under the same parent
     * and the returned nodes will be all nodes from start through end after splitting
     * False to disallow split parent
     * @returns When canSplitParent is true, returns all node from start through end after splitting,
     * otherwise just return start and end
     */
    collapseNodes(start: Node, end: Node, canSplitParent: boolean): Node[];
    /**
     * Check whether the editor contains any visible content
     * @param trim Whether trim the content string before check. Default is false
     * @returns True if there's no visible content, otherwise false
     */
    isEmpty(trim?: boolean): boolean;
    /**
     * Get current editor content as HTML string
     * @param mode specify what kind of HTML content to retrieve
     * @returns HTML string representing current editor content
     */
    getContent(mode?: GetContentMode | CompatibleGetContentMode): string;
    /**
     * Set HTML content to this editor. All existing content will be replaced. A ContentChanged event will be triggered
     * @param content HTML content to set in
     * @param triggerContentChangedEvent True to trigger a ContentChanged event. Default value is true
     */
    setContent(content: string, triggerContentChangedEvent?: boolean): void;
    /**
     * Insert HTML content into editor
     * @param HTML content to insert
     * @param option Insert options. Default value is:
     *  position: ContentPosition.SelectionStart
     *  updateCursor: true
     *  replaceSelection: true
     *  insertOnNewLine: false
     */
    insertContent(content: string, option?: InsertOption): void;
    /**
     * Delete selected content
     */
    deleteSelectedContent(): NodePosition | null;
    /**
     * Paste into editor using a clipboardData object
     * @param clipboardData Clipboard data retrieved from clipboard
     * @param pasteAsText Force pasting as plain text. Default value is false
     * @param applyCurrentStyle True if apply format of current selection to the pasted content,
     * false to keep original format.  Default value is false. When pasteAsText is true, this parameter is ignored
     * @param pasteAsImage: When set to true, if the clipboardData contains a imageDataUri will paste the image to the editor
     */
    paste(clipboardData: ClipboardData, pasteAsText?: boolean, applyCurrentFormat?: boolean, pasteAsImage?: boolean): void;
    /**
     * Get current selection range from Editor.
     * It does a live pull on the selection, if nothing retrieved, return whatever we have in cache.
     * @param tryGetFromCache Set to true to retrieve the selection range from cache if editor doesn't own the focus now.
     * Default value is true
     * @returns current selection range, or null if editor never got focus before
     */
    getSelectionRange(tryGetFromCache?: boolean): Range | null;
    /**
     * Get current selection range from Editor.
     * It does a live pull on the selection.
     * @returns current selection range, or null if editor never got focus before
     */
    getSelectionRangeEx(): SelectionRangeEx;
    /**
     * Get current selection in a serializable format
     * It does a live pull on the selection, if nothing retrieved, return whatever we have in cache.
     * @returns current selection path, or null if editor never got focus before
     */
    getSelectionPath(): SelectionPath | null;
    /**
     * Check if focus is in editor now
     * @returns true if focus is in editor, otherwise false
     */
    hasFocus(): boolean;
    /**
     * Focus to this editor, the selection was restored to where it was before, no unexpected scroll.
     */
    focus(): void;
    /**
     * Select content by range
     * @param range The range to select
     * @returns True if content is selected, otherwise false
     */
    select(range: Range): boolean;
    /**
     * Select content by Position and collapse to this position
     * @param position The position to select
     * @returns True if content is selected, otherwise false
     */
    select(position: NodePosition | null): boolean;
    /**
     * Select content by a start and end position
     * @param start The start position to select
     * @param end The end position to select, if this is the same with start, the selection will be collapsed
     * @returns True if content is selected, otherwise false
     */
    select(start: NodePosition, end: NodePosition): boolean;
    /**
     * Select content by node
     * @param node The node to select
     * @returns True if content is selected, otherwise false
     */
    select(node: Node): boolean;
    /**
     * Select content by node and offset, and collapse to this position
     * @param node The node to select
     * @param offset The offset of node to select, can be a number or value of PositionType
     * @returns True if content is selected, otherwise false
     */
    select(node: Node, offset: number | PositionType): boolean;
    /**
     * Select content by start and end nodes and offsets
     * @param startNode The node to select start from
     * @param startOffset The offset to select start from
     * @param endNode The node to select end to
     * @param endOffset The offset to select end to
     * @returns True if content is selected, otherwise false
     */
    select(startNode: Node, startOffset: number | PositionType, endNode: Node, endOffset: number | PositionType): boolean;
    /**
     * Select content by selection path
     * @param path A selection path object
     * @returns True if content is selected, otherwise false
     */
    select(path: SelectionPath): boolean;
    /**
     * Select content using the Table Selection
     * @param table to select
     * @param coordinates first and last cell of the range, if null is provided will remove the selection on the table
     */
    select(table: HTMLTableElement, coordinates: TableSelection | null): boolean;
    /**
     * Select content SelectionRangeEx
     * @param rangeEx SelectionRangeEx object to specify what to select
     */
    select(rangeEx: SelectionRangeEx): boolean;
    /**
     * Get current focused position. Return null if editor doesn't have focus at this time.
     */
    getFocusedPosition(): NodePosition | null;
    /**
     * Get an HTML element from current cursor position.
     * When expectedTags is not specified, return value is the current node (if it is HTML element)
     * or its parent node (if current node is a Text node).
     * When expectedTags is specified, return value is the first ancestor of current node which has
     * one of the expected tags.
     * If no element found within editor by the given tag, return null.
     * @param selector Optional, an HTML selector to find HTML element with.
     * @param startFrom Optional, start search from this node. If not specified, start from current focused position
     * @param event Optional, if specified, editor will try to get cached result from the event object first.
     * If it is not cached before, query from DOM and cache the result into the event object
     */
    getElementAtCursor(selector?: string, startFrom?: Node, event?: PluginEvent): HTMLElement | null;
    /**
     * Check if this position is at beginning of the editor.
     * This will return true if all nodes between the beginning of target node and the position are empty.
     * @param position The position to check
     * @returns True if position is at beginning of the editor, otherwise false
     */
    isPositionAtBeginning(position: NodePosition): boolean;
    /**
     * Get impacted regions from selection
     */
    getSelectedRegions(type?: RegionType | CompatibleRegionType): Region[];
    /**
     * Add a custom DOM event handler to handle events not handled by roosterjs.
     * Caller need to take the responsibility to dispose the handler properly
     * @param eventName DOM event name to handle
     * @param handler Handler callback
     * @returns A dispose function. Call the function to dispose this event handler
     */
    addDomEventHandler(eventName: string, handler: DOMEventHandler): () => void;
    /**
     * Add a bunch of custom DOM event handler to handle events not handled by roosterjs.
     * Caller need to take the responsibility to dispose the handler properly
     * @param handlerMap A event name => event handler map
     * @returns A dispose function. Call the function to dispose all event handlers added by this function
     */
    addDomEventHandler(handlerMap: Record<string, DOMEventHandler>): () => void;
    /**
     * Trigger an event to be dispatched to all plugins
     * @param eventType Type of the event
     * @param data data of the event with given type, this is the rest part of PluginEvent with the given type
     * @param broadcast indicates if the event needs to be dispatched to all plugins
     * True means to all, false means to allow exclusive handling from one plugin unless no one wants that
     * @returns the event object which is really passed into plugins. Some plugin may modify the event object so
     * the result of this function provides a chance to read the modified result
     */
    triggerPluginEvent<T extends PluginEventType | CompatiblePluginEventType>(eventType: T, data: PluginEventData<T>, broadcast?: boolean): PluginEventFromType<T>;
    /**
     * Trigger a ContentChangedEvent
     * @param source Source of this event, by default is 'SetContent'
     * @param data additional data for this event
     */
    triggerContentChangedEvent(source?: ChangeSource | CompatibleChangeSource | string, data?: any): void;
    /**
     * Undo last edit operation
     */
    undo(): void;
    /**
     * Redo next edit operation
     */
    redo(): void;
    /**
     * Add undo snapshot, and execute a format callback function, then add another undo snapshot, then trigger
     * ContentChangedEvent with given change source.
     * If this function is called nested, undo snapshot will only be added in the outside one
     * @param callback The callback function to perform formatting, returns a data object which will be used as
     * the data field in ContentChangedEvent if changeSource is not null.
     * @param changeSource The change source to use when fire ContentChangedEvent. When the value is not null,
     * a ContentChangedEvent will be fired with change source equal to this value
     * @param canUndoByBackspace True if this action can be undone when user presses Backspace key (aka Auto Complete).
     * @param additionalData Optional parameter to provide additional data related to the ContentChanged Event.
     */
    addUndoSnapshot(callback?: (start: NodePosition | null, end: NodePosition | null) => any, changeSource?: ChangeSource | CompatibleChangeSource | string, canUndoByBackspace?: boolean, additionalData?: ContentChangedData): void;
    /**
     * Whether there is an available undo/redo snapshot
     */
    getUndoState(): EditorUndoState;
    /**
     * Get document which contains this editor
     * @returns The HTML document which contains this editor
     */
    getDocument(): Document;
    /**
     * Get the scroll container of the editor
     */
    getScrollContainer(): HTMLElement;
    /**
     * Get custom data related to this editor
     * @param key Key of the custom data
     * @param getter Getter function. If custom data for the given key doesn't exist,
     * call this function to get one and store it if it is specified. Otherwise return undefined
     * @param disposer An optional disposer function to dispose this custom data when
     * dispose editor.
     */
    getCustomData<T>(key: string, getter?: () => T, disposer?: (value: T) => void): T;
    /**
     * Check if editor is in IME input sequence
     * @returns True if editor is in IME input sequence, otherwise false
     */
    isInIME(): boolean;
    /**
     * Get default format of this editor
     * @returns Default format object of this editor
     */
    getDefaultFormat(): DefaultFormat;
    /**
     * Get a content traverser for the whole editor
     * @param startNode The node to start from. If not passed, it will start from the beginning of the body
     */
    getBodyTraverser(startNode?: Node): IContentTraverser;
    /**
     * Get a content traverser for current selection
     * @returns A content traverser, or null if editor never got focus before and no range is provided
     */
    getSelectionTraverser(range?: Range): IContentTraverser | null;
    /**
     * Get a content traverser for current block element start from specified position
     * @param startFrom Start position of the traverser. Default value is ContentPosition.SelectionStart
     * @returns A content traverser, or null if editor never got focus before
     */
    getBlockTraverser(startFrom?: ContentPosition | CompatibleContentPosition): IContentTraverser | null;
    /**
     * Get a text traverser of current selection
     * @param event Optional, if specified, editor will try to get cached result from the event object first.
     * If it is not cached before, query from DOM and cache the result into the event object
     * @returns A content traverser, or null if editor never got focus before
     */
    getContentSearcherOfCursor(event?: PluginEvent | null): IPositionContentSearcher | null;
    /**
     * Run a callback function asynchronously
     * @param callback The callback function to run
     * @returns a function to cancel this async run
     */
    runAsync(callback: (editor: IEditor) => void): () => void;
    /**
     * Set DOM attribute of editor content DIV
     * @param name Name of the attribute
     * @param value Value of the attribute
     */
    setEditorDomAttribute(name: string, value: string | null): void;
    /**
     * Get DOM attribute of editor content DIV, null if there is no such attribute.
     * @param name Name of the attribute
     */
    getEditorDomAttribute(name: string): string | null;
    /**
     * @deprecated Use getVisibleViewport() instead
     *
     * Get current relative distance from top-left corner of the given element to top-left corner of editor content DIV.
     * @param element The element to calculate from. If the given element is not in editor, return value will be null
     * @param addScroll When pass true, The return value will also add scrollLeft and scrollTop if any. So the value
     * may be different than what user is seeing from the view. When pass false, scroll position will be ignored.
     * @returns An [x, y] array which contains the left and top distances, or null if the given element is not in editor.
     */
    getRelativeDistanceToEditor(element: HTMLElement, addScroll?: boolean): number[] | null;
    /**
     * Add a Content Edit feature.
     * @param feature The feature to add
     */
    addContentEditFeature(feature: GenericContentEditFeature<PluginEvent>): void;
    /**
     * Remove a Content Edit feature.
     * @param feature The feature to remove
     */
    removeContentEditFeature(feature: GenericContentEditFeature<PluginEvent>): void;
    /**
     * Get style based format state from current selection, including font name/size and colors
     */
    getStyleBasedFormatState(node?: Node): StyleBasedFormatState;
    /**
     * Get the pendable format state from the current selection, including formats as underline, bold, italics
     * @param forceGetStateFromDOM If set to true, will not consider the cached format and will get the format state directly from DOM tree
     * @return The pending format state of editor.
     */
    getPendableFormatState(forceGetStateFromDOM?: boolean): PendableFormatState;
    /**
     * Ensure user will type into a container element rather than into the editor content DIV directly
     * @param position The position that user is about to type to
     * @param keyboardEvent Optional keyboard event object
     */
    ensureTypeInContainer(position: NodePosition, keyboardEvent?: KeyboardEvent): void;
    /**
     * Set the dark mode state and transforms the content to match the new state.
     * @param nextDarkMode The next status of dark mode. True if the editor should be in dark mode, false if not.
     */
    setDarkModeState(nextDarkMode?: boolean): void;
    /**
     * Check if the editor is in dark mode
     * @returns True if the editor is in dark mode, otherwise false
     */
    isDarkMode(): boolean;
    /**
     * Transform the given node and all its child nodes to dark mode color if editor is in dark mode
     * @param node The node to transform
     * @param direction The transform direction. @default ColorTransformDirection.LightToDark
     */
    transformToDarkColor(node: Node, direction?: ColorTransformDirection | CompatibleColorTransformDirection): void;
    /**
     * Get a darkColorHandler object for this editor.
     */
    getDarkColorHandler(): DarkColorHandler;
    /**
     * Make the editor in "Shadow Edit" mode.
     * In Shadow Edit mode, all format change will finally be ignored.
     * This can be used for building a live preview feature for format button, to allow user
     * see format result without really apply it.
     * This function can be called repeated. If editor is already in shadow edit mode, we can still
     * use this function to do more shadow edit operation.
     */
    startShadowEdit(): void;
    /**
     * Leave "Shadow Edit" mode, all changes made during shadow edit will be discarded
     */
    stopShadowEdit(): void;
    /**
     * Check if editor is in Shadow Edit mode
     */
    isInShadowEdit(): boolean;
    /**
     * Check if the given experimental feature is enabled
     * @param feature The feature to check
     */
    isFeatureEnabled(feature: ExperimentalFeatures | CompatibleExperimentalFeatures): boolean;
    /**
     * Get a function to convert HTML string to trusted HTML string.
     * By default it will just return the input HTML directly. To override this behavior,
     * pass your own trusted HTML handler to EditorOptions.trustedHTMLHandler
     * See https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy/trusted-types
     */
    getTrustedHTMLHandler(): TrustedHTMLHandler;
    /**
     * Get current zoom scale, default value is 1
     * When editor is put under a zoomed container, need to pass the zoom scale number using EditorOptions.zoomScale
     * to let editor behave correctly especially for those mouse drag/drop behaviors
     * @returns current zoom scale number
     */
    getZoomScale(): number;
    /**
     * Set current zoom scale, default value is 1
     * When editor is put under a zoomed container, need to pass the zoom scale number using EditorOptions.zoomScale
     * to let editor behave correctly especially for those mouse drag/drop behaviors
     */
    setZoomScale(scale: number): void;
    /**
     * @deprecated Use getZoomScale() instead
     */
    getSizeTransformer(): SizeTransformer;
    /**
     * Retrieves the rect of the visible viewport of the editor.
     */
    getVisibleViewport(): Rect | null;
}

    /**
 * A handler object for dark color, used for variable-based dark color solution
 */
interface DarkColorHandler {
    /**
     * Given a light mode color value and an optional dark mode color value, register this color
     * so that editor can handle it, then return the CSS color value for current color mode.
     * @param lightModeColor Light mode color value
     * @param isDarkMode Whether current color mode is dark mode
     * @param darkModeColor Optional dark mode color value. If not passed, we will calculate one.
     */
    registerColor(lightModeColor: string, isDarkMode: boolean, darkModeColor?: string): string;
    /**
     * Reset known color record, clean up registered color variables.
     */
    reset(): void;
    /**
     * Parse an existing color value, if it is in variable-based color format, extract color key,
     * light color and query related dark color if any
     * @param color The color string to parse
     * @param isInDarkMode Whether current content is in dark mode. When set to true, if the color value is not in dark var format,
     * we will treat is as a dark mode color and try to find a matched dark mode color.
     */
    parseColorValue(color: string | null | undefined, isInDarkMode?: boolean): ColorKeyAndValue;
    /**
     * Get a copy of known colors
     */
    getKnownColorsCopy(): Readonly<ModeIndependentColor>[];
    /**
     * Find related light mode color from dark mode color.
     * @param darkColor The existing dark color
     */
    findLightColorFromDarkColor(darkColor: string): string | null;
    /**
     * Transform element color, from dark to light or from light to dark
     * @param element The element to transform color
     * @param fromDarkMode Whether this is transforming color from dark mode
     * @param toDarkMode Whether this is transforming color to dark mode
     */
    transformElementColor(element: HTMLElement, fromDarkMode: boolean, toDarkMode: boolean): void;
}

    /**
 * Represents a combination of color key, light color and dark color, parsed from existing color value
 */
interface ColorKeyAndValue {
    /**
     * Key of color, if found, otherwise undefined
     */
    key?: string;
    /**
     * Light mode color value
     */
    lightModeColor: string;
    /**
     * Dark mode color value, if found, otherwise undefined
     */
    darkModeColor?: string;
}

    /**
 * ContentEditFeature interface that handles keyboard event
 */
type ContentEditFeature = GenericContentEditFeature<PluginKeyboardEvent | CompatiblePluginKeyboardEvent>;

    /**
 * Generic ContentEditFeature interface
 */
interface GenericContentEditFeature<TEvent extends PluginEvent> {
    /**
     * Keys of this edit feature to handle
     */
    keys: number[];
    /**
     * Check if the event should be handled by this edit feature
     * @param event The plugin event to check
     * @param editor The editor object
     * @param ctrlOrMeta If Ctrl key (for Windows) or Meta key (for Mac) is pressed
     */
    shouldHandleEvent: (event: TEvent, editor: IEditor, ctrlOrMeta: boolean) => any;
    /**
     * Handle this event
     * @param event The event to handle
     * @param editor The editor object
     */
    handleEvent: (event: TEvent, editor: IEditor) => any;
    /**
     * Whether function keys (Ctrl/Meta or Alt) is allowed for this edit feature, default value is false.
     * When set to false, this edit feature won't be triggered if user has pressed Ctrl/Meta/Alt key
     */
    allowFunctionKeys?: boolean;
}

    /**
 * RoosterJs build in content edit feature
 */
interface BuildInEditFeature<TEvent extends PluginEvent> extends GenericContentEditFeature<TEvent> {
    /**
     * Whether this edit feature is disabled by default
     */
    defaultDisabled?: boolean;
}

    /**
 * Interface of an editor plugin
 */
interface EditorPlugin {
    /**
     * Get a friendly name of this plugin
     */
    getName: () => string;
    /**
     * The first method that editor will call to a plugin when editor is initializing.
     * It will pass in the editor instance, plugin should take this chance to save the
     * editor reference so that it can call to any editor method or format API later.
     * @param editor The editor object
     */
    initialize: (editor: IEditor) => void;
    /**
     * The last method that editor will call to a plugin before it is disposed.
     * Plugin can take this chance to clear the reference to editor. After this method is
     * called, plugin should not call to any editor method since it will result in error.
     */
    dispose: () => void;
    /**
     * Check if the plugin should handle the given event exclusively.
     * Handle an event exclusively means other plugin will not receive this event in
     * onPluginEvent method.
     * If two plugins will return true in willHandleEventExclusively() for the same event,
     * the final result depends on the order of the plugins are added into editor
     * @param event The event to check:
     */
    willHandleEventExclusively?: (event: PluginEvent) => boolean;
    /**
     * Core method for a plugin. Once an event happens in editor, editor will call this
     * method of each plugin to handle the event as long as the event is not handled
     * exclusively by another plugin.
     * @param event The event to handle:
     */
    onPluginEvent?: (event: PluginEvent) => void;
}

    /**
 * An editor plugin which have a state object stored on editor core
 * so that editor and core api can access it
 */
interface PluginWithState<T> extends EditorPlugin  {
    /**
     * Get plugin state object
     */
    getState(): T;
}

    /**
 * An interface for editor core plugins.
 * These plugins are built-in and most of them are not able to be replaced
 */
interface CorePlugins {
    /**
     * Edit plugin handles ContentEditFeatures
     */
    readonly edit: PluginWithState<EditPluginState>;
    /**
     * Undo plugin provides the ability to undo/redo
     */
    readonly undo: PluginWithState<UndoPluginState>;
    /**
     * TypeInContainer plugin makes sure user is always type under a container element under editor DIV
     */
    readonly typeInContainer: EditorPlugin;
    /**
     * DomEvent plugin helps handle additional DOM events such as IME composition, cut, drop.
     */
    readonly domEvent: PluginWithState<DOMEventPluginState>;
    /**
     * PendingFormatStatePlugin handles pending format state management
     */
    readonly pendingFormatState: PluginWithState<PendingFormatStatePluginState>;
    /**
     * MouseUpPlugin help trigger MouseUp event even when mouse up happens outside editor
     * as long as the mouse was pressed within Editor before
     */
    readonly mouseUp: EditorPlugin;
    /**
     * @deprecated after Firefox update
     * TypeAfterLinkPlugin plugin helps workaround a Firefox bug to allow type outside a hyperlink
     */
    readonly typeAfterLink: EditorPlugin;
    /**
     * Copy and paste plugin for handling onCopy and onPaste event
     */
    readonly copyPaste: PluginWithState<CopyPastePluginState>;
    /**
     * Entity Plugin handles all operations related to an entity and generate entity specified events
     */
    readonly entity: PluginWithState<EntityPluginState>;
    /**
     * Image selection Plugin detects image selection and help highlight the image
     */
    readonly imageSelection: EditorPlugin;
    /**
     * NormalizeTable plugin makes sure each table in editor has TBODY/THEAD/TFOOT tag around TR tags
     */
    readonly normalizeTable: EditorPlugin;
    /**
     * Lifecycle plugin handles editor initialization and disposing
     */
    readonly lifecycle: PluginWithState<LifecyclePluginState>;
}

    /**
 * Names of core plugins
 */
type PluginKey = keyof CorePlugins;

    /**
 * Names of the core plugins that have plugin state
 */
type KeyOfStatePlugin<Key extends PluginKey> = CorePlugins[Key] extends PluginWithState<infer U> ? Key : never;

    /**
 * A type map from name of plugin with state to its plugin type
 */
type GenericPluginState<Key extends PluginKey> = {
    [P in StatePluginKeys<Key>]: TypeOfStatePlugin<P>;
};

    /**
 * Auto-calculated State object type for plugin with states
 */
type PluginState = GenericPluginState<PluginKey>;

    /**
 * All names of plugins with plugin state
 */
type StatePluginKeys<Key extends PluginKey> = {
    [P in Key]: KeyOfStatePlugin<P>;
}[Key];

    /**
 * Get type of a plugin with state
 */
type TypeOfStatePlugin<Key extends PluginKey> = CorePlugins[Key] extends PluginWithState<infer U> ? U : never;

    /**
 * Represents the core data structure of an editor
 */
interface EditorCore extends PluginState  {
    /**
     * The content DIV element of this editor
     */
    readonly contentDiv: HTMLDivElement;
    /**
     * An array of editor plugins.
     */
    readonly plugins: EditorPlugin[];
    /**
     * Core API map of this editor
     */
    readonly api: CoreApiMap;
    /**
     * Original API map of this editor. Overridden core API can use API from this map to call the original version of core API.
     */
    readonly originalApi: CoreApiMap;
    /**
     * A handler to convert HTML string to a trust HTML string.
     * By default it will just return the original HTML string directly.
     * To override, pass your own trusted HTML handler to EditorOptions.trustedHTMLHandler
     */
    readonly trustedHTMLHandler: TrustedHTMLHandler;
    zoomScale: number;
    /**
     * @deprecated Use zoomScale instead
     */
    sizeTransformer: SizeTransformer;
    /**
     * Retrieves the Visible Viewport of the editor.
     */
    getVisibleViewport: () => Rect | null;
    /**
     * Color of the border of a selectedImage. Default color: '#DB626C'
     */
    imageSelectionBorderColor?: string;
    /**
     * Dark model handler for the editor, used for variable-based solution.
     * If keep it null, editor will still use original dataset-based dark mode solution.
     */
    darkColorHandler: DarkColorHandler;
    /**
     * A callback to be invoked when any exception is thrown during disposing editor
     * @param plugin The plugin that causes exception
     * @param error The error object we got
     */
    disposeErrorHandler?: (plugin: EditorPlugin, error: Error) => void;
}

    /**
 * Call an editing callback with adding undo snapshots around, and trigger a ContentChanged event if change source is specified.
 * Undo snapshot will not be added if this call is nested inside another addUndoSnapshot() call.
 * @param core The EditorCore object
 * @param callback The editing callback, accepting current selection start and end position, returns an optional object used as the data field of ContentChangedEvent.
 * @param changeSource The ChangeSource string of ContentChangedEvent. @default ChangeSource.Format. Set to null to avoid triggering ContentChangedEvent
 * @param canUndoByBackspace True if this action can be undone when user press Backspace key (aka Auto Complete).
 * @param additionalData Optional parameter to provide additional data related to the ContentChanged Event.
 */
type AddUndoSnapshot = (core: EditorCore, callback: ((start: NodePosition | null, end: NodePosition | null) => any) | null, changeSource: ChangeSource | CompatibleChangeSource | string | null, canUndoByBackspace: boolean, additionalData?: ContentChangedData) => void;

    /**
 * Attach a DOM event to the editor content DIV
 * @param core The EditorCore object
 * @param eventMap A map from event name to its handler
 */
type AttachDomEvent = (core: EditorCore, eventMap: Record<string, DOMEventHandler>) => () => void;

    /**
 * The interface for the map of core API.
 * Editor can call call API from this map under EditorCore object
 */
interface CoreApiMap {
    /**
     * Call an editing callback with adding undo snapshots around, and trigger a ContentChanged event if change source is specified.
     * Undo snapshot will not be added if this call is nested inside another addUndoSnapshot() call.
     * @param core The EditorCore object
     * @param callback The editing callback, accepting current selection start and end position, returns an optional object used as the data field of ContentChangedEvent.
     * @param changeSource The ChangeSource string of ContentChangedEvent. @default ChangeSource.Format. Set to null to avoid triggering ContentChangedEvent
     * @param canUndoByBackspace True if this action can be undone when user presses Backspace key (aka Auto Complete).
     */
    addUndoSnapshot: AddUndoSnapshot;
    /**
     * Attach a DOM event to the editor content DIV
     * @param core The EditorCore object
     * @param eventName The DOM event name
     * @param pluginEventType Optional event type. When specified, editor will trigger a plugin event with this name when the DOM event is triggered
     * @param beforeDispatch Optional callback function to be invoked when the DOM event is triggered before trigger plugin event
     */
    attachDomEvent: AttachDomEvent;
    /**
     * Create a DocumentFragment for paste from a ClipboardData
     * @param core The EditorCore object.
     * @param clipboardData Clipboard data retrieved from clipboard
     * @param position The position to paste to
     * @param pasteAsText True to force use plain text as the content to paste, false to choose HTML or Image if any
     * @param applyCurrentStyle True if apply format of current selection to the pasted content,
     * false to keep original format
     */
    createPasteFragment: CreatePasteFragment;
    /**
     * Ensure user will type into a container element rather than into the editor content DIV directly
     * @param core The EditorCore object.
     * @param position The position that user is about to type to
     * @param keyboardEvent Optional keyboard event object
     * @param deprecated Deprecated parameter, not used
     */
    ensureTypeInContainer: EnsureTypeInContainer;
    /**
     * Focus to editor. If there is a cached selection range, use it as current selection
     * @param core The EditorCore object
     */
    focus: Focus;
    /**
     * Get current editor content as HTML string
     * @param core The EditorCore object
     * @param mode specify what kind of HTML content to retrieve
     * @returns HTML string representing current editor content
     */
    getContent: GetContent;
    /**
     * Get current or cached selection range
     * @param core The EditorCore object
     * @param tryGetFromCache Set to true to retrieve the selection range from cache if editor doesn't own the focus now
     * @returns A Range object of the selection range
     */
    getSelectionRange: GetSelectionRange;
    /**
     * Get current or cached selection range
     * @param core The EditorCore object
     * @param tryGetFromCache Set to true to retrieve the selection range from cache if editor doesn't own the focus now
     * @returns A Range object of the selection range
     */
    getSelectionRangeEx: GetSelectionRangeEx;
    /**
     * Get style based format state from current selection, including font name/size and colors
     * @param core The EditorCore objects
     * @param node The node to get style from
     */
    getStyleBasedFormatState: GetStyleBasedFormatState;
    /**
     * Get the pendable format such as underline and bold
     * @param core The EditorCore object
     *@param forceGetStateFromDOM If set to true, will force get the format state from DOM tree.
     * @return The pending format state of editor.
     */
    getPendableFormatState: GetPendableFormatState;
    /**
     * Check if the editor has focus now
     * @param core The EditorCore object
     * @returns True if the editor has focus, otherwise false
     */
    hasFocus: HasFocus;
    /**
     * Insert a DOM node into editor content
     * @param core The EditorCore object. No op if null.
     * @param option An insert option object to specify how to insert the node
     */
    insertNode: InsertNode;
    /**
     * Restore an undo snapshot into editor
     * @param core The editor core object
     * @param step Steps to move, can be 0, positive or negative
     */
    restoreUndoSnapshot: RestoreUndoSnapshot;
    /**
     * Select content according to the given information.
     * There are a bunch of allowed combination of parameters. See IEditor.select for more details
     * @param core The editor core object
     * @param arg1 A DOM Range, or SelectionRangeEx, or NodePosition, or Node, or Selection Path
     * @param arg2 (optional) A NodePosition, or an offset number, or a PositionType, or a TableSelection, or null
     * @param arg3 (optional) A Node
     * @param arg4 (optional) An offset number, or a PositionType
     */
    select: Select;
    /**
     * Change the editor selection to the given range
     * @param core The EditorCore object
     * @param range The range to select
     * @param skipSameRange When set to true, do nothing if the given range is the same with current selection
     * in editor, otherwise it will always remove current selection range and set to the given one.
     * This parameter is always treated as true in Edge to avoid some weird runtime exception.
     */
    selectRange: SelectRange;
    /**
     * Set HTML content to this editor. All existing content will be replaced. A ContentChanged event will be triggered
     * if triggerContentChangedEvent is set to true
     * @param core The EditorCore object
     * @param content HTML content to set in
     * @param triggerContentChangedEvent True to trigger a ContentChanged event. Default value is true
     */
    setContent: SetContent;
    /**
     * Switch the Shadow Edit mode of editor On/Off
     * @param core The EditorCore object
     * @param isOn True to switch On, False to switch Off
     */
    switchShadowEdit: SwitchShadowEdit;
    /**
     * Edit and transform color of elements between light mode and dark mode
     * @param core The EditorCore object
     * @param rootNode The root HTML element to transform
     * @param includeSelf True to transform the root node as well, otherwise false
     * @param callback The callback function to invoke before do color transformation
     * @param direction To specify the transform direction, light to dark, or dark to light
     * @param forceTransform By default this function will only work when editor core is in dark mode.
     * Pass true to this value to force do color transformation even editor core is in light mode
     * @param fromDarkModel Whether the given content is already in dark mode
     */
    transformColor: TransformColor;
    /**
     * Trigger a plugin event
     * @param core The EditorCore object
     * @param pluginEvent The event object to trigger
     * @param broadcast Set to true to skip the shouldHandleEventExclusively check
     */
    triggerEvent: TriggerEvent;
    /**
     * Select a table and save data of the selected range
     * @param core The EditorCore object
     * @param table table to select
     * @param coordinates first and last cell of the selection, if this parameter is null, instead of
     * selecting, will unselect the table.
     * @param shouldAddStyles Whether need to update the style elements
     * @returns true if successful
     */
    selectTable: SelectTable;
    /**
     * Select a image and save data of the selected range
     * @param core The EditorCore object
     * @param image image to select
     * @param imageId the id of the image element
     * @returns true if successful
     */
    selectImage: SelectImage;
}

    /**
 * Create a DocumentFragment for paste from a ClipboardData
 * @param core The EditorCore object.
 * @param clipboardData Clipboard data retrieved from clipboard
 * @param position The position to paste to
 * @param pasteAsText True to force use plain text as the content to paste, false to choose HTML or Image if any
 * @param applyCurrentStyle True if apply format of current selection to the pasted content,
 * false to keep original format
 */
type CreatePasteFragment = (core: EditorCore, clipboardData: ClipboardData, position: NodePosition | null, pasteAsText: boolean, applyCurrentStyle: boolean, pasteAsImage: boolean) => DocumentFragment | null;

    /**
 * Ensure user will type into a container element rather than into the editor content DIV directly
 * @param core The EditorCore object.
 * @param position The position that user is about to type to
 * @param keyboardEvent Optional keyboard event object
 * @param deprecated Deprecated parameter, not used
 */
type EnsureTypeInContainer = (core: EditorCore, position: NodePosition, keyboardEvent?: KeyboardEvent, deprecated?: boolean) => void;

    /**
 * Focus to editor. If there is a cached selection range, use it as current selection
 * @param core The EditorCore object
 */
type Focus = (core: EditorCore) => void;

    /**
 * Get current editor content as HTML string
 * @param core The EditorCore object
 * @param mode specify what kind of HTML content to retrieve
 * @returns HTML string representing current editor content
 */
type GetContent = (core: EditorCore, mode: GetContentMode | CompatibleGetContentMode) => string;

    /**
 * Get current or cached selection range
 * @param core The EditorCore object
 * @param tryGetFromCache Set to true to retrieve the selection range from cache if editor doesn't own the focus now
 * @returns A Range object of the selection range
 */
type GetSelectionRange = (core: EditorCore, tryGetFromCache: boolean) => Range | null;

    /**
 * Get current selection range
 * @param core The EditorCore object
 * @returns A Range object of the selection range
 */
type GetSelectionRangeEx = (core: EditorCore) => SelectionRangeEx;

    /**
 * Get style based format state from current selection, including font name/size and colors
 * @param core The EditorCore objects
 * @param node The node to get style from
 */
type GetStyleBasedFormatState = (core: EditorCore, node: Node | null) => StyleBasedFormatState;

    /**
 * Get the pendable format such as underline and bold
 * @param core The EditorCore object
 * @param forceGetStateFromDOM If set to true, will force get the format state from DOM tree.
 * @return The pending format state of editor.
 */
type GetPendableFormatState = (core: EditorCore, forceGetStateFromDOM: boolean) => PendableFormatState;

    /**
 * Check if the editor has focus now
 * @param core The EditorCore object
 * @returns True if the editor has focus, otherwise false
 */
type HasFocus = (core: EditorCore) => boolean;

    /**
 * Insert a DOM node into editor content
 * @param core The EditorCore object. No op if null.
 * @param option An insert option object to specify how to insert the node
 */
type InsertNode = (core: EditorCore, node: Node, option: InsertOption | null) => boolean;

    /**
 * Restore an undo snapshot into editor
 * @param core The editor core object
 * @param step Steps to move, can be 0, positive or negative
 */
type RestoreUndoSnapshot = (core: EditorCore, step: number) => void;

    /**
 * Select content according to the given information.
 * There are a bunch of allowed combination of parameters. See IEditor.select for more details
 * @param core The editor core object
 * @param arg1 A DOM Range, or SelectionRangeEx, or NodePosition, or Node, or Selection Path
 * @param arg2 (optional) A NodePosition, or an offset number, or a PositionType, or a TableSelection, or null
 * @param arg3 (optional) A Node
 * @param arg4 (optional) An offset number, or a PositionType
 */
type Select = (core: EditorCore, arg1: Range | SelectionRangeEx | NodePosition | Node | SelectionPath | null, arg2?: NodePosition | number | PositionType | TableSelection | null, arg3?: Node, arg4?: number | PositionType) => boolean;

    /**
 * Change the editor selection to the given range
 * @param core The EditorCore object
 * @param range The range to select
 * @param skipSameRange When set to true, do nothing if the given range is the same with current selection
 * in editor, otherwise it will always remove current selection range and set to the given one.
 * This parameter is always treated as true in Edge to avoid some weird runtime exception.
 */
type SelectRange = (core: EditorCore, range: Range, skipSameRange?: boolean) => boolean;

    /**
 * Set HTML content to this editor. All existing content will be replaced. A ContentChanged event will be triggered
 * if triggerContentChangedEvent is set to true
 * @param core The EditorCore object
 * @param content HTML content to set in
 * @param triggerContentChangedEvent True to trigger a ContentChanged event. Default value is true
 */
type SetContent = (core: EditorCore, content: string, triggerContentChangedEvent: boolean, metadata?: ContentMetadata) => void;

    /**
 * Switch the Shadow Edit mode of editor On/Off
 * @param core The EditorCore object
 * @param isOn True to switch On, False to switch Off
 */
type SwitchShadowEdit = (core: EditorCore, isOn: boolean) => void;

    /**
 * Edit and transform color of elements between light mode and dark mode
 * @param core The EditorCore object
 * @param rootNode The root HTML node to transform
 * @param includeSelf True to transform the root node as well, otherwise false
 * @param callback The callback function to invoke before do color transformation
 * @param direction To specify the transform direction, light to dark, or dark to light
 * @param forceTransform By default this function will only work when editor core is in dark mode.
 * Pass true to this value to force do color transformation even editor core is in light mode
 * @param fromDarkModel Whether the given content is already in dark mode
 */
type TransformColor = (core: EditorCore, rootNode: Node | null, includeSelf: boolean, callback: (() => void) | null, direction: ColorTransformDirection | CompatibleColorTransformDirection, forceTransform?: boolean, fromDarkMode?: boolean) => void;

    /**
 * Trigger a plugin event
 * @param core The EditorCore object
 * @param pluginEvent The event object to trigger
 * @param broadcast Set to true to skip the shouldHandleEventExclusively check
 */
type TriggerEvent = (core: EditorCore, pluginEvent: PluginEvent, broadcast: boolean) => void;

    /**
 * Select a table and save data of the selected range
 * @param core The EditorCore object
 * @param table table to select
 * @param coordinates first and last cell of the selection, if this parameter is null, instead of
 * selecting, will unselect the table.
 * @returns true if successful
 */
type SelectTable = (core: EditorCore, table: HTMLTableElement | null, coordinates?: TableSelection) => TableSelectionRange | null;

    /**
 * Select a table and save data of the selected range
 * @param core The EditorCore object
 * @param image image to select
 * @returns true if successful
 */
type SelectImage = (core: EditorCore, image: HTMLImageElement | null) => ImageSelectionRange | null;

    /**
 * The options to specify parameters customizing an editor, used by ctor of Editor class
 */
interface EditorOptions {
    /**
     * List of plugins.
     * The order of plugins here determines in what order each event will be dispatched.
     * Plugins not appear in this list will not be added to editor, including built-in plugins.
     * Default value is empty array.
     */
    plugins?: EditorPlugin[];
    /**
     * Default format of editor content. This will be applied to empty content.
     * If there is already content inside editor, format of existing content will not be changed.
     * Default value is the computed style of editor content DIV
     */
    defaultFormat?: DefaultFormat;
    /**
     * @deprecated Use undoMetadataSnapshotService instead
     * Undo snapshot service. Use this parameter to customize the undo snapshot service.
     */
    undoSnapshotService?: UndoSnapshotsService<string>;
    /**
     * Undo snapshot service based on content metadata. Use this parameter to customize the undo snapshot service.
     * When this property is set, value of undoSnapshotService will be ignored.
     */
    undoMetadataSnapshotService?: UndoSnapshotsService<Snapshot>;
    /**
     * Initial HTML content
     * Default value is whatever already inside the editor content DIV
     */
    initialContent?: string;
    /**
     * A function map to override default core API implementation
     * Default value is null
     */
    coreApiOverride?: Partial<CoreApiMap>;
    /**
     * A plugin map to override default core Plugin implementation
     * Default value is null
     */
    corePluginOverride?: Partial<CorePlugins>;
    /**
     * If the editor is currently in dark mode
     */
    inDarkMode?: boolean;
    /**
     * @deprecated
     * RoosterJS provides an experimental "external content handler" that transforms text
     * This is used when content is pasted or inserted via a method we can hook into.
     * This transform is currently "lossy" and will eliminate color information.
     * If you want to change this behavior, you may define a different function here.
     * It takes in the impacted HTMLElement
     */
    onExternalContentTransform?: (element: HTMLElement, fromDarkMode: boolean, toDarkMode: boolean, darkColorHandler: DarkColorHandler) => void;
    /**
     * A util function to transform light mode color to dark mode color
     * Default value is to return the original light color
     */
    getDarkColor?: (lightColor: string) => string;
    /**
     * Whether to skip the adjust editor process when for light/dark mode
     */
    doNotAdjustEditorColor?: boolean;
    /**
     * The scroll container to get scroll event from.
     * By default, the scroll container will be the same with editor content DIV
     */
    scrollContainer?: HTMLElement;
    /**
     * Specify the enabled experimental features
     */
    experimentalFeatures?: (ExperimentalFeatures | CompatibleExperimentalFeatures)[];
    /**
     * By default, we will stop propagation of a printable keyboard event
     * (a keyboard event which is caused by printable char input).
     * Set this option to true to override this behavior in case you still need the event
     * to be handled by ancestor nodes of editor.
     */
    allowKeyboardEventPropagation?: boolean;
    /**
     * Allowed custom content type when paste besides text/plain, text/html and images
     * Only text types are supported, and do not add "text/" prefix to the type values
     */
    allowedCustomPasteType?: string[];
    /**
     * Customized trusted type handler used for sanitizing HTML string before assign to DOM tree
     * This is required when trusted-type Content-Security-Policy (CSP) is enabled.
     * See https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy/trusted-types
     */
    trustedHTMLHandler?: TrustedHTMLHandler;
    /**
     * Current zoom scale, @default value is 1
     * When editor is put under a zoomed container, need to pass the zoom scale number using this property
     * to let editor behave correctly especially for those mouse drag/drop behaviors
     */
    zoomScale?: number;
    /**
     * @deprecated Use zoomScale instead
     */
    sizeTransformer?: SizeTransformer;
    /**
     * Retrieves the visible viewport of the Editor. The default viewport is the Rect of the scrollContainer.
     */
    getVisibleViewport?: () => Rect | null;
    /**
     * Color of the border of a selectedImage. Default color: '#DB626C'
     */
    imageSelectionBorderColor?: string;
    /**
     * A callback to be invoked when any exception is thrown during disposing editor
     * @param plugin The plugin that causes exception
     * @param error The error object we got
     */
    disposeErrorHandler?: (plugin: EditorPlugin, error: Error) => void;
}

    /**
 * A list to specify whether each of the listed content edit features is enabled
 */
interface ContentEditFeatureSettings extends ListFeatureSettings, QuoteFeatureSettings, TableFeatureSettings, StructuredNodeFeatureSettings, AutoLinkFeatureSettings, ShortcutFeatureSettings, CursorFeatureSettings, MarkdownFeatureSettings, EntityFeatureSettings, TextFeatureSettings, CodeFeatureSettings  {
}

    /**
 * Settings for auto link features
 */
interface AutoLinkFeatureSettings {
    /**
     * When press Space or Enter after a hyperlink-like string, convert the string to a hyperlink
     * @default true
     */
    autoLink: boolean;
    /**
     * Unlink when backspace right after a hyperlink
     * @default false
     */
    unlinkWhenBackspaceAfterLink: boolean;
}

    /**
 * Settings for cursor features
 */
interface CursorFeatureSettings {
    /**
     * Chrome may make the cursor move the then end of document if press Ctrl+Left at the beginning of document
     * Let's disable this behavior
     * @default true
     */
    noCycleCursorMove: boolean;
}

    /**
 * Settings for entity features
 */
interface EntityFeatureSettings {
    /**
     * A content edit feature to trigger EntityOperation event with operation "Click" when user
     * clicks on a readonly entity.
     */
    clickOnEntity: boolean;
    /**
     * A content edit feature to trigger EntityOperation event with operation "Escape" when user
     * presses ESC on a readonly entity.
     */
    escapeFromEntity: boolean;
    /**
     * A content edit feature to split current line into two lines at the cursor when user presses
     * ENTER right before a readonly entity.
     * Browser's default behavior will insert an extra BR tag before the entity which causes an extra
     * empty line. So we override the default behavior here.
     */
    enterBeforeReadonlyEntity: boolean;
    /**
     * A content edit feature to trigger EntityOperation event with operation "RemoveFromEnd" when user
     * press BACKSPACE right after an entity
     */
    backspaceAfterEntity: boolean;
    /**
     * A content edit feature to trigger EntityOperation event with operation "RemoveFromStart" when user
     * press DELETE right after an entity
     */
    deleteBeforeEntity: boolean;
    /**
     * Content edit feature to move the cursor from Delimiters around Entities when using Right or Left Arrow Keys
     */
    moveBetweenDelimitersFeature: boolean;
    /**
     * Content edit Feature to trigger a Delete Entity Operation when one of the Delimiter is about to be removed with DELETE or Backspace
     */
    removeEntityBetweenDelimiters: boolean;
}

    /**
 * Settings for list features
 */
interface ListFeatureSettings {
    /**
     * When press space after an asterisk or number in an empty line, toggle bullet/numbering
     * @default true
     */
    autoBullet: boolean;
    /**
     * When press Tab in a list, indent current list item
     * @default true
     */
    indentWhenTab: boolean;
    /**
     * When press Shift+Tab in a list, outdent current list item
     * @default true
     */
    outdentWhenShiftTab: boolean;
    /**
     * When press BaskSpace on empty line which is the first item of a list, outdent current list item
     * @default true
     */
    outdentWhenBackspaceOnEmptyFirstLine: boolean;
    /**
     * When press Enter on empty line in a list, outdent current list item
     * @default true for IE, false for other browsers since they have already had the behavior
     */
    outdentWhenEnterOnEmptyLine: boolean;
    /**
     * When press Backspace on first char in a list, make current item a new line of previous list item
     * @default false
     */
    mergeInNewLineWhenBackspaceOnFirstChar: boolean;
    /**
     * When edit with lists, maintain the list numbers of list chain
     */
    maintainListChain: boolean;
    /**
     * When delete key is pressed before the first item, indent the correct list of numbers
     */
    maintainListChainWhenDelete: boolean;
    /**
     * When press space after *, -, --, ->, -->, >, =>  in an empty line, toggle bullet
     * @default true
     */
    autoBulletList: boolean;
    /**
     * When press space after an number, a letter or roman number followed by ), ., -, or between parenthesis in an empty line, toggle numbering
     * @default true
     */
    autoNumberingList: boolean;
    /**
     * MergeListOnBackspaceAfterList edit feature, provides the ability to merge list on backspace on block after a list.
     * @default true
     */
    mergeListOnBackspaceAfterList: boolean;
    /**
     * indentWhenAltShiftRight edit feature, provides the ability to indent or outdent current list when user press Alt+shift+Right
     * @default when browser is in Mac it is default disabled, else it is enabled
     */
    indentWhenAltShiftRight: boolean;
    /**
     * outdentWhenAltShiftLeft edit feature, provides the ability to indent or outdent current list when user press Alt+shift+Left
     * @default when browser is in Mac it is default disabled, else it is enabled
     */
    outdentWhenAltShiftLeft: boolean;
}

    /**
 * Settings for mark down features
 */
interface MarkdownFeatureSettings {
    /**
     * When typing text surrounded by '*', the symbols will be removed and the text will be bolded.
     * @default true
     */
    markdownBold: boolean;
    /**
     * When typing text surrounded by '_', the symbols will be removed and the text will be italicized.
     * @default true
     */
    markdownItalic: boolean;
    /**
     * When typing text surrounded by '~', the symbols will be removed and the text will be strike through.
     * @default true
     */
    markdownStrikethru: boolean;
    /**
     * When typing text surrounded by '`', the symbols will be removed and the text will be wrapped in a code tag.
     * @default true
     */
    markdownInlineCode: boolean;
}

    /**
 * Settings for quote features
 */
interface QuoteFeatureSettings {
    /**
     * When press BAckspace on empty line which is the first line of a blockquote, unquote current line
     * @default true
     */
    unquoteWhenBackspaceOnEmptyFirstLine: boolean;
    /**
     * When press Enter on empty line in a blockquote, unquote current line
     * @default true
     */
    unquoteWhenEnterOnEmptyLine: boolean;
}

    /**
 * Settings for shortcut features
 */
interface ShortcutFeatureSettings {
    /**
     * Respond to default common keyboard short, i.e. Ctrl+B, Ctrl+I, Ctrl+U, Ctrl+Z, Ctrl+Y
     * @default true
     */
    defaultShortcut: boolean;
}

    /**
 * Settings for structured node features
 */
interface StructuredNodeFeatureSettings {
    /**
     * When press Enter at the beginning of first structured element (table, list) and there isn't line before the position
     * we create a new line before so that user got a chance to enter content before the table or list
     * @default false
     */
    insertLineBeforeStructuredNodeFeature: boolean;
}

    /**
 * Settings for table features
 */
interface TableFeatureSettings {
    /**
     * When press TAB or SHIFT+TAB key in table cell, jump to next/previous table cell
     * @default true
     */
    tabInTable: boolean;
    /**
     * When press Up or Down in table cell, jump to the table cell above/below
     * @default true for Chrome and safari, false for other browsers since they already have correct behavior
     */
    upDownInTable: boolean;
    /**
     * IndentTableOnTab edit feature, provides the ability to indent the table if it is all cells are selected.
     */
    indentTableOnTab: boolean;
    /**
     * Requires @see ExperimentalFeatures.DeleteTableWithBackspace
     * Delete a table selected with the table selector pressing  Backspace key
     */
    deleteTableWithBackspace: boolean;
}

    /**
 * Settings for text features
 */
interface TextFeatureSettings {
    /**
     * Requires @see ExperimentalFeatures.TabKeyTextFeatures to be enabled
     * When press Tab:
     *      If Whole Paragraph selected, indent paragraph,
     *      If range is collapsed, add spaces
     *      If range is not collapsed but not all the paragraph is selected, remove selection and add
     *          spaces
     */
    indentWhenTabText: boolean;
    /**
     * Requires @see ExperimentalFeatures.TabKeyTextFeatures to be enabled
     * When press Tab:
     *      If Whole Paragraph selected, outdent paragraph
     */
    outdentWhenTabText: boolean;
    /**
     * @deprecated
     * Requires @see ExperimentalFeatures.AutoHyphen to be enabled
     * Automatically transform -- into hyphen, if typed between two words.
     */
    autoHyphen: boolean;
}

    /**
 * Settings for code features
 */
interface CodeFeatureSettings {
    /**
     * When inside a code block, exit the code block by pressing Enter twice, or once on an empty line
     * @default true
     */
    removeCodeWhenEnterOnEmptyLine: boolean;
    /**
     * When inside an empty code block (or an empty first line), exit the code block by pressing Backspace
     * @default true
     */
    removeCodeWhenBackspaceOnEmptyFirstLine: boolean;
}

    /**
 * An interface to define a replacement rule for CustomReplace plugin
 */
interface CustomReplacement {
    /**
     * Source string to replace from
     */
    sourceString: string;
    /**
     * HTML string to replace to
     */
    replacementHTML: string;
    /**
     * Whether the matching should be case sensitive
     */
    matchSourceCaseSensitive: boolean;
    /**
     * A callback to check if the string should be replaced
     * @param content the content where the string is
     * @param replacement string to be replaced
     * @param sourceEditor reference to the editor, allows for more complex replacement rules
     * @return true, if the string should be replaced, else return false
     */
    shouldReplace?: (replacement: CustomReplacement, content: string, sourceEditor?: IEditor) => boolean;
}

    /**
 * Represent an interface to provide functionalities for Undo Snapshots
 */
interface UndoSnapshotsService<T = string> {
    /**
     * Check whether can move current undo snapshot with the given step
     * @param step The step to check, can be positive, negative or 0
     * @returns True if can move current snapshot with the given step, otherwise false
     */
    canMove(step: number): boolean;
    /**
     * Move current snapshot with the given step if can move this step. Otherwise no action and return null
     * @param step The step to move
     * @returns If can move with the given step, returns the snapshot after move, otherwise null
     */
    move(step: number): T | null;
    /**
     * Add a new undo snapshot
     * @param snapshot The snapshot to add
     */
    addSnapshot(snapshot: T, isAutoCompleteSnapshot: boolean): void;
    /**
     * Clear all undo snapshots after the current one
     */
    clearRedo(): void;
    /**
     * Whether there is a snapshot added before auto complete and it can be undone now
     */
    canUndoAutoComplete(): boolean;
}

    /**
 * Data provider for PickerPlugin
 */
interface PickerDataProvider {
    /**
     * Function called when the plugin is initialized to register two callbacks with the
     * data provider and to provide a reference to the Editor. The first callback, insertNodeCallback,
     * may be called in order to "commit" a new element to the editor body that isn't handled
     * automatically by the editor plugin. The second callback, setIsSuggestingCallback, sets
     * the isSuggesting value in the plugin. The isSuggesting value determines whether the picker
     * is shown (true) or hidden (false). In most cases, this value is managed by the plugin, but
     * the setIsSuggestingCallback may be used for situations where the UX needs to manipulate the
     * suggesting state on its own.
     */
    onInitalize: (insertNodeCallback: (nodeToInsert: HTMLElement) => void, setIsSuggestingCallback: (isSuggesting: boolean) => void, editor?: IEditor) => void;
    /**
     * Function called when the plugin is disposed for the data provider to do any cleanup.
     */
    onDispose: () => void;
    /**
     * Function called when the picker changes suggesting state
     * (isSuggesting - true: Plugin is being shown; false: Plugin is being hidden).
     */
    onIsSuggestingChanged: (isSuggesting: boolean) => void;
    /**
     * Function called when the query string (text after the trigger symbol) is updated.
     */
    queryStringUpdated: (queryString: string, isExactMatch: boolean) => void;
    /**
     * Function called when a keypress is issued that would "select" a currently highlighted option.
     */
    selectOption?: () => void;
    /**
     * Function called when a keypress is issued that would move the highlight on any picker UX.
     */
    shiftHighlight?: (isIncrement: boolean) => void;
    /**
     * Function that is called when a delete command is issued.
     * Returns the intended replacement node (if partial delete) or null (if full delete)
     */
    onRemove: (nodeRemoved: Node, isBackwards: boolean) => Node;
    /**
     * Function that is called by the plugin to set the current cursor position as an
     * anchor point for where to show UX.
     */
    setCursorPoint?: (targetPoint: {
        x: number;
        y: number;
    }, buffer: number) => void;
    /**
     * Function that is called when the plugin detects the editor's content has changed.
     * Provides a list of current picker placed elements in the document.
     */
    onContentChanged?: (elementIds: string[]) => void;
    /**
     * Function that returns the index of the option currently selected in the picker.
     */
    getSelectedIndex?: () => number;
    /**
     * Handler of scroll event from scroll container of editor
     */
    onScroll?: (scrollContainer: HTMLElement) => void;
}

    /**
 * Options for PickerPlugin
 */
interface PickerPluginOptions {
    /**
     * Constant that defines the element ID prefix to look for.
     * If it matches, this element should be handled by the plugin
     */
    elementIdPrefix: string;
    /**
     * When apply the selected item in picker, a ContentChangedEvent will be broadcasted.
     * This value will be used as the ChangeSource of this event.
     */
    changeSource: string;
    /**
     * Constant that defines the character(s) that will trigger the suggesting state in the plugin.
     */
    triggerCharacter: string;
    /**
     * Option for using the picker in the horizontal state:
     * Vertical (the default, when this is false), will call shiftHighlight with up (false) and down (true).
     * Horizontal (when this is true), will call shiftHighlight with left (false) and right (true).
     */
    isHorizontal?: boolean;
    /**
     * When apply the selected item in picker, perform as an auto-complete behavior (can be undone by BACKSPACE key)
     * if this option is set to true
     */
    handleAutoComplete?: boolean;
    /**
     * Constant that defines the ID label for the picker.
     * Used for setting the ariaOwns attribute of the editor when a picker is open.
     */
    suggestionsLabel?: string;
    /**
     * Constant that defines the prefix of the ID label for the picker's options.
     * Used for setting the ariaActiveDescendant attribute of the editor when a picker option is selected.
     */
    suggestionLabelPrefix?: string;
}

    /**
 * Represent a virtual cell of a virtual table
 */
interface VCell {
    /**
     * The table cell object. The value will be null if this is an expanded virtual cell
     */
    td?: HTMLTableCellElement | null;
    /**
     * Whether this cell is spanned from left
     */
    spanLeft?: boolean;
    /**
     * Whether this cell is spanned from above
     */
    spanAbove?: boolean;
    /**
     * The width in pixel of the actual td (including border and padding).
     * If the table is under a zoomed container, this value is the visible pixel count after zoom.
     */
    width?: number;
    /**
     * The height in pixel of the actual td (including border and padding)
     * If the table is under a zoomed container, this value is the visible pixel count after zoom.
     */
    height?: number;
}

    /**
 * Options for ImageEdit plugin
 */
interface ImageEditOptions {
    /**
     * Color of resize/rotate border, handle and icon
     * @default #DB626C
     */
    borderColor?: string | ModeIndependentColor;
    /**
     * Minimum resize/crop width
     * @default 10
     */
    minWidth?: number;
    /**
     * Minimum resize/crop height
     * @default 10
     */
    minHeight?: number;
    /**
     * Whether preserve width/height ratio when resize
     * Pressing SHIFT key when resize will for preserve ratio even this value is set to false
     * @default false
     */
    preserveRatio?: boolean;
    /**
     * Minimum degree increase/decrease when rotate image.
     * Pressing SHIFT key when rotate will ignore this value and rotate by any degree with mouse moving
     * @default 5
     */
    minRotateDeg?: number;
    /**
     * Selector of the image that allows editing
     * @default img
     */
    imageSelector?: string;
    /**
     * @deprecated
     * HTML for the rotate icon
     * @default A predefined SVG icon
     */
    rotateIconHTML?: string;
    /**
     * Whether side resizing (single direction resizing) is disabled. @default false
     */
    disableSideResize?: boolean;
    /**
     * Whether image rotate is disabled. @default false
     */
    disableRotate?: boolean;
    /**
     * Whether image crop is disabled. @default false
     */
    disableCrop?: boolean;
    /**
     * Which operations will be executed when image is selected
     * @default ImageEditOperation.ResizeAndRotate
     */
    onSelectState?: ImageEditOperation | CompatibleImageEditOperation;
}

    /**
 * An interface represents the data for creating element used by createElement()
 */
interface CreateElementData {
    /**
     * Tag name of this element.
     * It can be just a tag, or in format "namespace:tag"
     */
    tag: string;
    /**
     * Namespace of this tag
     */
    namespace?: string;
    /**
     * CSS class name
     */
    className?: string;
    /**
     * CSS style
     */
    style?: string;
    /**
     * Dataset of this element
     */
    dataset?: Record<string, string>;
    /**
     * Additional attributes of this element
     */
    attributes?: Record<string, string>;
    /**
     * Child nodes of this element, can be another CreateElementData, or a string which represents a text node
     */
    children?: (CreateElementData | string)[];
}

    /**
 * Represents normal selection
 */
interface SelectionRangeExBase<T extends SelectionRangeTypes | CompatibleSelectionRangeTypes> {
    /**
     * Selection Type definition
     */
    type: T;
    /**
     * Ranges
     */
    ranges: Range[];
    /**
     * Whether all the ranges are collapsed
     */
    areAllCollapsed: boolean;
}

    /**
 * Represents normal selection
 */
interface NormalSelectionRange extends SelectionRangeExBase<SelectionRangeTypes.Normal | CompatibleSelectionRangeTypes.Normal> {
}

    /**
 * Represents the selection made inside of a table.
 */
interface TableSelectionRange extends SelectionRangeExBase<SelectionRangeTypes.TableSelection | CompatibleSelectionRangeTypes.TableSelection> {
    /**
     * Table that has cells selected
     */
    table: HTMLTableElement;
    /**
     * Coordinates of first and last Cell
     */
    coordinates: TableSelection | undefined;
    /**
     * Whether the whole table is selected
     */
    isWholeTableSelected?: boolean;
}

    /**
 * Represents a selected image.
 */
interface ImageSelectionRange extends SelectionRangeExBase<SelectionRangeTypes.ImageSelection | CompatibleSelectionRangeTypes.ImageSelection> {
    /**
     * Selected Image
     */
    image: HTMLImageElement;
}

    /**
 * Types of ranges used in editor api getSelectionRangeEx
 */
type SelectionRangeEx = NormalSelectionRange | TableSelectionRange | ImageSelectionRange;

    /**
 * Represents all info of a known entity, including its DOM element, whether it is deleted and if it can be persisted
 */
interface KnownEntityItem {
    /**
     * The HTML element of entity wrapper
     */
    element: HTMLElement;
    /**
     * Whether this entity is deleted.
     */
    isDeleted?: boolean;
    /**
     * Whether we want to persist this entity element during undo/redo
     */
    canPersist?: boolean;
}

    /**
 * Attribute callback, will be called when HtmlSanitizer process an attribute with given name
 * @param value Value of the attribute
 * @param element The HTML element contains this attribute
 * @param context A context object to store values which can used for communicating among callbacks
 * @returns null to remove this attribute, otherwise keep the attribute with the value returned
 */
type AttributeCallback = (value: string, element: HTMLElement, context: Object) => string;

    /**
 * A map of attribute callbacks. Tag name should be in lower case
 */
type AttributeCallbackMap = Record<string, AttributeCallback>;

    /**
 * Style callback, will be called when HtmlSanitizer process an inline CSS style with given name
 * @param value Value of the CSS rule
 * @param element The HTML element contains this CSS style
 * @param thisStyle Current values of known inheritable styles
 * @param context A context object to store values which can used for communicating among callbacks
 * @returns True if this rule should be kept, otherwise false
 */
type CssStyleCallback = (value: string, element: HTMLElement, thisStyle: StringMap, context: Object) => boolean;

    /**
 * A map of style callbacks. Style name should be in lower case
 */
type CssStyleCallbackMap = Record<string, CssStyleCallback>;

    /**
 * Element callback, will be called when HtmlSanitizer process an element with the given tag
 * @param element The HTML element
 * @param context A context object to store values which can used for communicating among callbacks
 * @returns True if this element should be kept, otherwise false
 */
type ElementCallback = (element: HTMLElement, context: Object) => boolean;

    /**
 * A map of attribute callbacks. Tag name should be in lower case
 */
type StringMap = Record<string, string>;

    /**
 * A map of elements callback. Tag name should be in upper case
 */
type ElementCallbackMap = Record<string, ElementCallback>;

    /**
 * A map of predefined CSS styles for elements
 */
type PredefinedCssMap = Record<string, StringMap>;

    /**
 * Handler function type of DOM event
 */
type DOMEventHandlerFunction<E = Event> = (event: E) => void;

    /**
 * DOM event handler object with mapped plugin event type and handler function
 */
interface DOMEventHandlerObject {
    /**
     * Type of plugin event. The DOM event will be mapped with this plugin event type
     */
    pluginEventType: PluginEventType | CompatiblePluginEventType | null;
    /**
     * Handler function. Besides the mapped plugin event type, this function will also be triggered
     * when correlated DOM event is fired
     */
    beforeDispatch: DOMEventHandlerFunction | null;
}

    /**
 * Combined event handler type with all 3 possibilities
 */
type DOMEventHandler<E = Event> = PluginEventType | CompatiblePluginEventType | DOMEventHandlerFunction<E> | DOMEventHandlerObject;

    /**
 * A handler type to convert HTML string to a trust HTML string
 */
type TrustedHTMLHandler = (html: string) => string;

    /**
 * A transformer function. It transform the size changes according to current situation.
 * A typical scenario to use this function is when editor is located under a scaled container, so we need to
 * calculate the scaled size change according to current zoom rate.
 * @param size Original delta size from mouse event
 * @returns Calculated delta size. By default it should just return original value
 */
type SizeTransformer = (size: number) => number;

    /**
 * A type template to get item type of an array
 */
type ArrayItemType<T extends any[]> = T extends (infer U)[] ? U : never;

    /**
 * Base interface of property definition
 */
interface DefinitionBase<T extends DefinitionType | CompatibleDefinitionType> {
    /**
     * Type of this property
     */
    type: T;
    /**
     * Whether this property is optional
     */
    isOptional?: boolean;
    /**
     * Whether this property is allowed to be null
     */
    allowNull?: boolean;
}

    /**
 * String property definition. This definition can also be used for string based enum property
 */
interface StringDefinition extends DefinitionBase<DefinitionType.String | CompatibleDefinitionType.String> {
    /**
     * An optional value of this property. When specified, the given property must have exactly same value of this value
     */
    value?: string;
}

    /**
 * Number property definition. This definition can also be used for number based enum property
 */
interface NumberDefinition extends DefinitionBase<DefinitionType.Number | CompatibleDefinitionType.Number> {
    /**
     * An optional value of this property. When specified, the given property must have same value of this value
     */
    value?: number;
    /**
     * An optional minimum value of this property. When specified, the given property must be greater or equal to this value
     */
    minValue?: number;
    /**
     * An optional maximum value of this property. When specified, the given property must be less or equal to this value
     */
    maxValue?: number;
}

    /**
 * Boolean property definition
 */
interface BooleanDefinition extends DefinitionBase<DefinitionType.Boolean | CompatibleDefinitionType.Boolean> {
    /**
     * An optional value of this property. When specified, the given property must have same value of this value
     */
    value?: boolean;
}

    /**
 * Array property definition.
 */
interface ArrayDefinition<T extends any[]> extends DefinitionBase<DefinitionType.Array | CompatibleDefinitionType.Array> {
    /**
     * Definition of each item of this array. All items of the given array must have the same type. Otherwise, use CustomizeDefinition instead.
     */
    itemDef: Definition<ArrayItemType<T>>;
    /**
     * An optional minimum length of this array. When specified, the given array must have at least this value of items
     */
    minLength?: number;
    /**
     * An optional maximum length of this array. When specified, the given array must have at most this value of items
     */
    maxLength?: number;
}

    /**
 * Object property definition.
 */
interface ObjectDefinition<T extends Object> extends DefinitionBase<DefinitionType.Object | CompatibleDefinitionType.Object> {
    /**
     * A key-value map to specify the definition of each possible property of this object
     */
    propertyDef: ObjectPropertyDefinition<T>;
}

    /**
 * Object property definition type used by Object Definition
 */
type ObjectPropertyDefinition<T extends Object> = {
    [Key in keyof T]: Definition<T[Key]>;
};

    /**
 * Customize property definition. When all other property definition type cannot satisfy your requirement,
 * use this definition with a customized validator function to do property validation.
 */
interface CustomizeDefinition extends DefinitionBase<DefinitionType.Customize | CompatibleDefinitionType.Customize> {
    /**
     * The customized validator function to do customized validation
     * @param input The value to validate
     * @returns True means the given value is of the specified type, otherwise false
     */
    validator: (input: any) => boolean;
}

    /**
 * A combination of all definition types
 */
type Definition<T> = CustomizeDefinition | (T extends any[] ? ArrayDefinition<T> : T extends Record<string, any> ? ObjectDefinition<T> : T extends String ? StringDefinition : T extends Number ? NumberDefinition : T extends Boolean ? BooleanDefinition : never);

    /**
 * Type of Editor Core Creator
 * @param contentDiv The DIV HTML element which will be the container element of editor
 * @param options An optional options object to customize the editor
 */
type CoreCreator<TEditorCore extends EditorCore, TEditorOptions extends EditorOptions> = (contentDiv: HTMLDivElement, options: TEditorOptions) => TEditorCore;

    /**
 * Experimental feature flags
 */
enum CompatibleExperimentalFeatures {
    /**
     * @deprecated This feature is always enabled
     */
    NewIndentation = "NewIndentation",
    /**
     * @deprecated This feature is always enabled
     */
    NewBullet = "NewBullet",
    /**
     * @deprecated This feature is always enabled
     */
    NewNumbering = "NewNumbering",
    /**
     * @deprecated This feature is always enabled
     */
    ListChain = "ListChain",
    /**
     * @deprecated This feature is always enabled
     */
    MergePastedLine = "MergePastedLine",
    /**
     * @deprecated This feature is always enabled
     */
    SingleDirectionResize = "SingleDirectionResize",
    /**
     * @deprecated This feature is always enabled
     */
    PasteWithLinkPreview = "PasteWithLinkPreview",
    /**
     * @deprecated This feature is always enabled
     */
    ImageRotate = "ImageRotate",
    /**
     * @deprecated This feature is always enabled
     */
    ImageCrop = "ImageCrop",
    /**
     * @deprecated This feature is always enabled
     * Check if the element has a style attribute, if not, apply the default format
     */
    AlwaysApplyDefaultFormat = "AlwaysApplyDefaultFormat",
    /**
     * @deprecated This feature can be enabled/disabled using Paste Plugin contructor param
     * Paste the Html instead of the Img when the Html Body only have one IMG Child node
     */
    ConvertSingleImageBody = "ConvertSingleImageBody",
    /**
     * @deprecated This feature is always enabled
     * Align table elements to left, center and right using setAlignment API
     */
    TableAlignment = "TableAlignment",
    /**
     * @deprecated this feature is always enabled
     * Provide a circular resize handles that adaptive the number od handles to the size of the image
     */
    AdaptiveHandlesResizer = "AdaptiveHandlesResizer",
    /**
     * @deprecated this feature is always disabled
     * Automatically transform -- into hyphen, if typed between two words.
     */
    AutoHyphen = "AutoHyphen",
    /**
     * @deprecated this feature is always disabled
     * Use pending format strategy to do style based format, e.g. Font size, Color.
     * With this feature enabled, we don't need to insert temp ZeroWidthSpace character to hold pending format
     * when selection is collapsed. Instead, we will hold the pending format in memory and only apply it when type something
     */
    PendingStyleBasedFormat = "PendingStyleBasedFormat",
    /**
     * @deprecated this feature is always disabled
     * Normalize list to make sure it can be displayed correctly in other client
     * e.g. We will move list items with "display: block" into previous list item and change tag to be DIV
     */
    NormalizeList = "NormalizeList",
    /**
     * @deprecated this feature is always enabled
     * When a html image is selected, the selected image data will be stored by editor core.
     */
    ImageSelection = "ImageSelection",
    /**
     * @deprecated this feature is always enabled
     * Use variable-based dark mode solution rather than dataset-based solution.
     * When enable this feature, need to pass in a DarkModelHandler object to each call of setColor and applyFormat
     * if you need them work for dark mode
     */
    VariableBasedDarkColor = "VariableBasedDarkColor",
    /**
     * @deprecated this feature is always enabled
     * Align list elements elements to left, center and right using setAlignment API
     */
    ListItemAlignment = "ListItemAlignment",
    /**
     * @deprecated
     */
    DefaultFormatInSpan = "DefaultFormatInSpan",
    /**
     * @deprecated
     */
    DefaultFormatOnContainer = "DefaultFormatOnContainer",
    /**
     * @deprecated This feature is always enabled
     * Reuse existing DOM structure if possible when convert Content Model back to DOM tree
     */
    ReusableContentModel = "ReusableContentModel",
    /**
     * @deprecated This feature is always enabled
     * Handle keyboard editing event with Content Model
     */
    EditWithContentModel = "EditWithContentModel",
    /**
     * @deprecated This feature is always enabled
     * Trigger formatting by a especial characters. Ex: (A), 1. i).
     */
    AutoFormatList = "AutoFormatList",
    /**
     * @deprecated This feature is always enabled
     * Add entities around a Read Only  Inline entity to prevent cursor to be hidden when cursor is next of it.
     */
    InlineEntityReadOnlyDelimiters = "InlineEntityReadOnlyDelimiters",
    /**
     * @deprecated This feature is always enabled
     * Paste with Content model
     */
    ContentModelPaste = "ContentModelPaste",
    /**
     * Provide additional Tab Key Features. Requires Text Features Content Editable Features
     */
    TabKeyTextFeatures = "TabKeyTextFeatures",
    /**
     * With this feature enabled, when writing back a list item we will re-use all
     * ancestor list elements, even if they don't match the types currently in the
     * listTypes array for that item. The only list that we will ensure is correct
     * is the one closest to the item.
     */
    ReuseAllAncestorListElements = "ReuseAllAncestorListElements",
    /**
     * Delete table with Backspace key with the whole was selected with table selector
     */
    DeleteTableWithBackspace = "DeleteTableWithBackspace",
    /**
     * Disable list chain functionality
     */
    DisableListChain = "DisableListChain"
}

    /**
 * Editor plugin event type
 */
enum CompatiblePluginEventType {
    /**
     * HTML KeyDown event
     */
    KeyDown = 0,
    /**
     * HTML KeyPress event
     */
    KeyPress = 1,
    /**
     * HTML KeyUp event
     */
    KeyUp = 2,
    /**
     * HTML Input / TextInput event
     */
    Input = 3,
    /**
     * HTML CompositionEnd event
     */
    CompositionEnd = 4,
    /**
     * HTML MouseDown event
     */
    MouseDown = 5,
    /**
     * HTML MouseUp event
     */
    MouseUp = 6,
    /**
     * Content changed event
     */
    ContentChanged = 7,
    /**
     * Extract Content with a DOM tree event
     * This event is triggered when getContent() is called with triggerExtractContentEvent = true
     * Plugin can handle this event to remove the UI only markups to return clean HTML
     * by operating on a cloned DOM tree
     */
    ExtractContentWithDom = 8,
    /**
     * Before Paste event, provide a chance to change copied content
     */
    BeforeCutCopy = 9,
    /**
     * Before Paste event, provide a chance to change paste content
     */
    BeforePaste = 10,
    /**
     * Let plugin know editor is ready now
     */
    EditorReady = 11,
    /**
     * Let plugin know editor is about to dispose
     */
    BeforeDispose = 12,
    /**
     * Pending format state (bold, italic, underline, ... with collapsed selection) is changed
     */
    PendingFormatStateChanged = 13,
    /**
     * Scroll event triggered by scroll container
     */
    Scroll = 14,
    /**
     * Operating on an entity. See enum EntityOperation for more details about each operation
     */
    EntityOperation = 15,
    /**
     * HTML ContextMenu event
     */
    ContextMenu = 16,
    /**
     * Editor has entered shadow edit mode
     */
    EnteredShadowEdit = 17,
    /**
     * Editor is about to leave shadow edit mode
     */
    LeavingShadowEdit = 18,
    /**
     * Content of image is being changed from client side
     */
    EditImage = 19,
    /**
     * Content of editor is about to be cleared by SetContent API, handle this event to cache anything you need
     * before it is gone
     */
    BeforeSetContent = 20,
    /**
     * Zoom scale value is changed, triggered by Editor.setZoomScale() when set a different scale number
     */
    ZoomChanged = 21,
    /**
     * EXPERIMENTAL FEATURE
     * Editor changed the selection.
     */
    SelectionChanged = 22,
    /**
     * EXPERIMENTAL FEATURE
     * Editor content is about to be changed by keyboard event.
     * This is only used by Content Model editing
     */
    BeforeKeyboardEditing = 23
}

    /**
 * Enum for paste options
 */
enum CompatiblePasteType {
    /**
     * Default paste behavior
     */
    Normal = 0,
    /**
     * Paste only the plain text
     */
    AsPlainText = 1,
    /**
     * Apply the current style to pasted content
     */
    MergeFormat = 2,
    /**
     * If there is a image uri in the clipboard, paste the content as image element
     */
    AsImage = 3
}

    /**
 * Possible change sources. Here are the predefined sources.
 * It can also be other string if the change source can't fall into these sources.
 */
enum CompatibleChangeSource {
    /**
     * Content changed by auto link
     */
    AutoLink = "AutoLink",
    /**
     * Content changed by create link
     */
    CreateLink = "CreateLink",
    /**
     * Content changed by format
     */
    Format = "Format",
    /**
     * Content changed by image resize
     */
    ImageResize = "ImageResize",
    /**
     * Content changed by paste
     */
    Paste = "Paste",
    /**
     * Content changed by setContent API
     */
    SetContent = "SetContent",
    /**
     * Content changed by cut operation
     */
    Cut = "Cut",
    /**
     * Content changed by drag & drop operation
     */
    Drop = "Drop",
    /**
     * Insert a new entity into editor
     */
    InsertEntity = "InsertEntity",
    /**
     * Editor is switched to dark mode, content color is changed
     */
    SwitchToDarkMode = "SwitchToDarkMode",
    /**
     * Editor is switched to light mode, content color is changed
     */
    SwitchToLightMode = "SwitchToLightMode",
    /**
     * List chain reorganized numbers of lists
     */
    ListChain = "ListChain",
    /**
     * Keyboard event, used by Content Model.
     * Data of this event will be the key code number
     */
    Keyboard = "Keyboard"
}

    /**
 * Define possible operations to an entity
 */
enum CompatibleEntityOperation {
    /**
     * Notify plugins that there is a new plugin was added into editor.
     * Plugin can handle this event to entity hydration.
     * This event will be only fired once for each entity DOM node.
     * After undo, or copy/paste, since new DOM nodes were added, this event will be fired
     * for those entities represented by newly added nodes.
     */
    NewEntity = 0,
    /**
     * Notify plugins that user is clicking target to an entity
     */
    Click = 1,
    /**
     * Notify plugins that user is triggering context menu to an entity
     */
    ContextMenu = 2,
    /**
     * Notify plugins that user is escaping from an entity by pressing ESC key
     */
    Escape = 3,
    /**
     * Notify plugins that user is removing an entity from its start position using DELETE key
     */
    RemoveFromStart = 4,
    /**
     * Notify plugins that user is remove an entity from its end position using BACKSPACE key
     */
    RemoveFromEnd = 5,
    /**
     * Notify plugins that an entity is being overwritten.
     * This can be caused by key in, cut, paste, delete, backspace ... on a selection
     * which contains some entities.
     */
    Overwrite = 6,
    /**
     * Notify plugins that an entity is being partially overwritten.
     * This happens when user selected part of the entity then do key press, or cut, paste, delete, backspace, ...
     */
    PartialOverwrite = 7,
    /**
     * Notify plugins that editor is generating HTML content for save.
     * Plugin should use this event to remove any temporary content, and only leave DOM nodes that
     * should be saved as HTML string.
     * This event will provide a cloned DOM tree for each entity, do NOT compare the DOM nodes with cached nodes
     * because it will always return false.
     */
    ReplaceTemporaryContent = 8,
    /**
     * @deprecated
     */
    AddShadowRoot = 9,
    /**
     * @deprecated
     */
    RemoveShadowRoot = 10,
    /**
     * Notify plugins that a new entity state need to be updated to an entity.
     * This is normally happened when user undo/redo the content with an entity snapshot added by a plugin that handles entity
     */
    UpdateEntityState = 11
}

    /**
 * Known announce strings
 */
enum CompatibleKnownAnnounceStrings {
    /**
     * String announced for a list item in a OL List
     * @example
     * Auto corrected, &lcub;0&rcub;
     * Where &lcub0&rcub is the new list item bullet
     */
    AnnounceListItemNumbering = 1,
    /**
     * String announced for a list item in a UL List
     * @example
     * Auto corrected bullet
     */
    AnnounceListItemBullet = 2,
    /**
     * String announced when cursor is moved to the last cell in a table
     */
    AnnounceOnFocusLastCell = 3
}

    /**
 * The position. Mostly used for content insertion and traversing
 * On insertion, we will need to specify where we want the content to be placed (begin, end, selection or outside)
 * On content traversing, we will need to specify the start position of traversing
 */
enum CompatibleContentPosition {
    /**
     * Begin of the container
     */
    Begin = 0,
    /**
     * End of the container
     */
    End = 1,
    /**
     * End of the content div domain.
     */
    DomEnd = 2,
    /**
     * Selection start
     */
    SelectionStart = 3,
    /**
     * Outside of editor
     */
    Outside = 4,
    /**
     * Manually defined range
     */
    Range = 5
}

    /**
 * Table format border
 */
enum CompatibleTableBorderFormat {
    /**
     * All border of the table are displayed
     *  __ __ __
     * |__|__|__|
     * |__|__|__|
     * |__|__|__|
     */
    DEFAULT = 0,
    /**
     * Middle vertical border are not displayed
     *  __ __ __
     * |__ __ __|
     * |__ __ __|
     * |__ __ __|
     */
    LIST_WITH_SIDE_BORDERS = 1,
    /**
     * All borders except header rows borders are displayed
     *  __ __ __
     *  __|__|__
     *  __|__|__
     */
    NO_HEADER_BORDERS = 2,
    /**
     * The left and right border of the table are not displayed
     *  __ __ __
     *  __|__|__
     *  __|__|__
     *  __|__|__
     */
    NO_SIDE_BORDERS = 3,
    /**
     * Only the borders that divides the header row, first column and externals are displayed
     *  __ __ __
     * |__ __ __|
     * |  |     |
     * |__|__ __|
     */
    FIRST_COLUMN_HEADER_EXTERNAL = 4,
    /**
     * The header row has no vertical border, except for the first one
     * The first column has no horizontal border, except for the first one
     *  __ __ __
     * |__ __ __
     * |  |__|__|
     * |  |__|__|
     */
    ESPECIAL_TYPE_1 = 5,
    /**
     * The header row has no vertical border, except for the first one
     * The only horizontal border of the table is the top and bottom of header row
     *  __ __ __
     * |__ __ __
     * |  |     |
     * |  |     |
     */
    ESPECIAL_TYPE_2 = 6,
    /**
     * The only borders are the bottom of header row and the right border of first column
     *  __ __ __
     *    |
     *    |
     */
    ESPECIAL_TYPE_3 = 7,
    /**
     * No border
     */
    CLEAR = 8
}

    /**
 * Represents a mode number to indicate what kind of content to retrieve when call Editor.getContent()
 */
enum CompatibleGetContentMode {
    /**
     * The clean content without any temporary content only for editor.
     * This is the default value. Call to Editor.getContent() with trigger an ExtractContentWithDom event
     * so that plugins can remove their temporary content, and will return the HTML content
     * which is ready for save to storage.
     */
    CleanHTML = 0,
    /**
     * Retrieve the raw HTML string in current editor. Temporary content will be included.
     */
    RawHTMLOnly = 1,
    /**
     * Retrieve the raw HTML string in current editor with a selection marker. This can be used for
     * save current editor state and call to SetContent with this result can fully restore editor state
     * including current selection
     */
    RawHTMLWithSelection = 2,
    /**
     * Get plain text content only, all format will be ignored
     */
    PlainText = 3,
    /**
     * A fast way to get plain text content, the line-end positions may not be exactly same with HTML content,
     * but the text content should be the same. This is used for quickly retrieve text content and check
     * text only
     */
    PlainTextFast = 4
}

    /**
 * Query scope for queryElements() API
 */
enum CompatibleQueryScope {
    /**
     * Query from the whole body of root node. This is default value.
     */
    Body = 0,
    /**
     * Query elements on a given selection (intersect)
     * The result element can contain the selection, contain part of selection, or inside selection
     */
    OnSelection = 1,
    /**
     * Query elements inside a given selection only
     */
    InSelection = 2
}

    /**
 * Type of all possible regions. Currently we only support region of Table
 */
enum CompatibleRegionType {
    /**
     * Region split by Table
     */
    Table = 0
}

    /**
 * Represents the mode of color transformation
 */
enum CompatibleColorTransformDirection {
    /**
     * Transform from light to dark
     */
    LightToDark = 0,
    /**
     * Transform from dark to light
     */
    DarkToLight = 1
}

    /**
 * Operation flags for ImageEdit plugin
 */
enum CompatibleImageEditOperation {
    /**
     * No operation
     */
    None = 0,
    /**
     * Resize image from corner ( horizontal and vertical resize)
     */
    CornerResize = 1,
    /**
     * Resize image from side ( either horizontal or vertical resize)
     */
    SideResize = 2,
    /**
     * Image resize
     */
    Resize = 3,
    /**
     * Image rotate
     */
    Rotate = 4,
    /**
     * Image resize and rotate
     */
    ResizeAndRotate = 7,
    /**
     * Image crop
     */
    Crop = 8,
    /**
     * All operations
     */
    All = 15
}

    /**
 * Types of Selection Ranges that the SelectionRangeEx can return
 */
enum CompatibleSelectionRangeTypes {
    /**
     * Normal selection range provided by browser.
     */
    Normal = 0,
    /**
     * Selection made inside of a single table.
     */
    TableSelection = 1,
    /**
     * Selection made in a image.
     */
    ImageSelection = 2
}

    /**
 * Types of definitions, used by Definition type
 */
enum CompatibleDefinitionType {
    /**
     * Boolean type definition, represents a boolean type value
     */
    Boolean = 0,
    /**
     * Number type definition, represents a number type value
     */
    Number = 1,
    /**
     * String type definition, represents a string type value
     */
    String = 2,
    /**
     * Array type definition, represents an array with a given item type
     */
    Array = 3,
    /**
     * Object type definition, represents an object with the given property types
     */
    Object = 4,
    /**
     * Customize type definition, represents a customized type with a validator function
     */
    Customize = 5
}

    /**
 * enum for setting block alignment, used by setAlignment API
 */
enum CompatibleAlignment {
    /**
     * Align left
     */
    Left = 0,
    /**
     * Align center
     */
    Center = 1,
    /**
     * Align right
     */
    Right = 2
}

    /**
 *  Enum used to control the different types of bullet list
 */
enum CompatibleBulletListType {
    /**
     * Minimum value of the enum
     */
    Min = 1,
    /**
     * Bullet triggered by *
     */
    Disc = 1,
    /**
     * Bullet triggered by -
     */
    Dash = 2,
    /**
     * Bullet triggered by --
     */
    Square = 3,
    /**
     * Bullet triggered by >
     */
    ShortArrow = 4,
    /**
     * Bullet triggered by ->
     */
    LongArrow = 5,
    /**
     * Bullet triggered by =>
     */
    UnfilledArrow = 6,
    /**
     * Bullet triggered by —
     */
    Hyphen = 7,
    /**
     * Bullet triggered by -->
     */
    DoubleLongArrow = 8,
    /**
     * Bullet type circle
     */
    Circle = 9,
    /**
     * Maximum value of the enum
     */
    Max = 9
}

    /**
 * The enum used for controlling the capitalization of text.
 * Used by changeCapitalization API
 */
enum CompatibleCapitalization {
    /**
     * Transforms the first character after punctuation mark followed by space
     * to uppercase and the rest of characters to lowercase.
     */
    Sentence = "sentence",
    /**
     * Transforms all characters to lowercase
     */
    Lowercase = "lowercase",
    /**
     * Transforms all characters to uppercase
     */
    Uppercase = "uppercase",
    /**
     * Transforms the first character of each word to uppercase
     */
    CapitalizeEachWord = "capitalize"
}

    /**
 * Represents the strategy to clear the format of the current editor selection
 */
enum CompatibleClearFormatMode {
    /**
     * Inline format. Remove text format.
     */
    Inline = 0,
    /**
     * BLock format. Remove text and structure format of the block.
     */
    Block = 1,
    /**
     * Detect Inline or Block format based on the current editor selector.
     */
    AutoDetect = 2
}

    /**
 * Prefix of content types
 */
enum CompatibleContentTypePrefix {
    /**
     * Text type prefix
     */
    Text = "text/",
    /**
     * Image type prefix
     */
    Image = "image/"
}

    /**
 * Known content types
 */
enum CompatibleContentType {
    /**
     * Plain text content type
     */
    PlainText = "text/plain",
    /**
     * HTML content type
     */
    HTML = "text/html"
}

    /**
 * @deprecated
 * Constants string for dataset names used by dark mode
 */
enum CompatibleDarkModeDatasetNames {
    /**
     * Original style text color
     */
    OriginalStyleColor = "ogsc",
    /**
     * Original style background color
     */
    OriginalStyleBackgroundColor = "ogsb",
    /**
     * Original attribute text color
     */
    OriginalAttributeColor = "ogac",
    /**
     * Original attribute background color
     */
    OriginalAttributeBackgroundColor = "ogab"
}

    /**
 * Class names for Delimiter
 */
enum CompatibleDelimiterClasses {
    /**
     * Class name to specify this delimiter is before an entity
     */
    DELIMITER_BEFORE = "entityDelimiterBefore",
    /**
     *  Class name to specify this delimiter is after an entity
     */
    DELIMITER_AFTER = "entityDelimiterAfter"
}

    /**
 * enum for setting block direction, used by setDirection API
 */
enum CompatibleDirection {
    /**
     * Left to right
     */
    LeftToRight = 0,
    /**
     * Right to left
     */
    RightToLeft = 1
}

    /**
 * Command strings for Document.execCommand() API
 * https://developer.mozilla.org/en-US/docs/Web/API/Document/execCommand
 */
enum CompatibleDocumentCommand {
    /**
     * Changes the browser auto-link behavior (Internet Explorer only)
     */
    AutoUrlDetect = "AutoUrlDetect",
    /**
     * Changes the document background color. In styleWithCss mode, it affects the background color of the containing block instead.
     * This requires a &lt;color&gt; value string to be passed in as a value argument. Note that Internet Explorer uses this to set the
     * text background color.
     */
    BackColor = "backColor",
    /**
     * Toggles bold on/off for the selection or at the insertion point. Internet Explorer uses the &lt;strong&gt; tag instead of &lt;b&gt;.
     */
    Bold = "bold",
    /**
     * Clears all authentication credentials from the cache.
     */
    ClearAuthenticationCache = "ClearAuthenticationCache",
    /**
     * Makes the content document either read-only or editable. This requires a boolean true/false as the value argument.
     * (Not supported by Internet Explorer.)
     */
    ContentReadOnly = "contentReadOnly",
    /**
     * Copies the current selection to the clipboard. Conditions of having this behavior enabled vary from one browser to another,
     * and have evolved over time. Check the compatibility table to determine if you can use it in your case.
     */
    Copy = "copy",
    /**
     * Creates an hyperlink from the selection, but only if there is a selection. Requires a URI string as a value argument for the
     * hyperlink's href. The URI must contain at least a single character, which may be whitespace.
     * (Internet Explorer will create a link with a null value.)
     */
    CreateLink = "createLink",
    /**
     * Removes the current selection and copies it to the clipboard. When this behavior is enabled varies between browsers,
     * and its conditions have evolved over time. Check the compatibility table for usage details.
     */
    Cut = "cut",
    /**
     * Adds a &lt;small&gt; tag around the selection or at the insertion point. (Not supported by Internet Explorer.)
     */
    DecreaseFontSize = "decreaseFontSize",
    /**
     * Changes the paragraph separator used when new paragraphs are created in editable text regions. See Differences in markup
     * generation for more details.
     */
    DefaultParagraphSeparator = "defaultParagraphSeparator",
    /**
     * Deletes the current selection.
     */
    Delete = "delete",
    /**
     * Enables or disables the table row/column insertion and deletion controls. (Not supported by Internet Explorer.)
     */
    EnableInlineTableEditing = "enableInlineTableEditing",
    /**
     * Enables or disables the resize handles on images and other resizable objects. (Not supported by Internet Explorer.)
     */
    EnableObjectResizing = "enableObjectResizing",
    /**
     * Changes the font name for the selection or at the insertion point. This requires a font name string (like "Arial")
     * as a value argument.
     */
    FontName = "fontName",
    /**
     * Changes the font size for the selection or at the insertion point. This requires an integer from 1-7 as a value argument.
     */
    FontSize = "fontSize",
    /**
     * Changes a font color for the selection or at the insertion point. This requires a hexadecimal color value string
     * as a value argument.
     */
    ForeColor = "foreColor",
    /**
     * Adds an HTML block-level element around the line containing the current selection, replacing the block element containing
     * the line if one exists (in Firefox, &lt;blockquote&gt; is the exception — it will wrap any containing block element).
     * Requires a tag-name string as a value argument. Virtually all block-level elements can be used.
     * (Internet Explorer supports only heading tags H1–H6, ADDRESS, and PRE, which must be wrapped in angle brackets, such as "&lt;H1&gt;".)
     */
    FormatBlock = "formatBlock",
    /**
     * Deletes the character ahead of the cursor's position, identical to hitting the Delete key on a Windows keyboard.
     */
    ForwardDelete = "forwardDelete",
    /**
     * Adds a heading element around a selection or insertion point line. Requires the tag-name strings a value argument (i.e. "H1", "H6").
     * (Not supported by Internet Explorer and Safari.)
     */
    Heading = "heading",
    /**
     * Changes the background color for the selection or at the insertion point. Requires a color value string as a value argument.
     * useCSS must be true for this to function. (Not supported by Internet Explorer.)
     */
    HiliteColor = "hiliteColor",
    /**
     * Adds a &lt;big&gt; tag around the selection or at the insertion point. (Not supported by Internet Explorer.)
     */
    IncreaseFontSize = "increaseFontSize",
    /**
     * Indents the line containing the selection or insertion point. In Firefox, if the selection spans multiple lines at different
     * levels of indentation, only the least indented lines in the selection will be indented.
     */
    Indent = "indent",
    /**
     * Controls whether the Enter key inserts a &lt;br&gt; element, or splits the current block element into two.
     * (Not supported by Internet Explorer.)
     */
    InsertBrOnReturn = "insertBrOnReturn",
    /**
     * Inserts a &lt;hr&gt; element at the insertion point, or replaces the selection with it.
     */
    InsertHorizontalRule = "insertHorizontalRule",
    /**
     * Inserts an HTML string at the insertion point (deletes selection). Requires a valid HTML string as a value argument.
     * (Not supported by Internet Explorer.)
     */
    InsertHTML = "insertHTML",
    /**
     * Inserts an image at the insertion point (deletes selection). Requires a URL string for the image's src as a value argument.
     * The requirements for this string are the same as createLink.
     */
    InsertImage = "insertImage",
    /**
     * Creates a numbered ordered list for the selection or at the insertion point.
     */
    InsertOrderedList = "insertOrderedList",
    /**
     * Creates a bulleted unordered list for the selection or at the insertion point.
     */
    InsertUnorderedList = "insertUnorderedList",
    /**
     * Inserts a paragraph around the selection or the current line.
     * (Internet Explorer inserts a paragraph at the insertion point and deletes the selection.)
     */
    InsertParagraph = "insertParagraph",
    /**
     * Inserts the given plain text at the insertion point (deletes selection).
     */
    InsertText = "insertText",
    /**
     * Toggles italics on/off for the selection or at the insertion point.
     * (Internet Explorer uses the &lt;em&gt; element instead of &lt;i&gt;.)
     */
    Italic = "italic",
    /**
     * Centers the selection or insertion point.
     */
    JustifyCenter = "justifyCenter",
    /**
     * Justifies the selection or insertion point.
     */
    JustifyFull = "justifyFull",
    /**
     * Justifies the selection or insertion point to the left.
     */
    JustifyLeft = "justifyLeft",
    /**
     * Right-justifies the selection or the insertion point.
     */
    JustifyRight = "justifyRight",
    /**
     * Outdents the line containing the selection or insertion point.
     */
    Outdent = "outdent",
    /**
     * Pastes the clipboard contents at the insertion point (replaces current selection). Disabled for web content. See [1].
     */
    Paste = "paste",
    /**
     * Redoes the previous undo command.
     */
    Redo = "redo",
    /**
     * Removes all formatting from the current selection.
     */
    RemoveFormat = "removeFormat",
    /**
     * Selects all of the content of the editable region.
     */
    SelectAll = "selectAll",
    /**
     * Toggles strikethrough on/off for the selection or at the insertion point.
     */
    StrikeThrough = "strikeThrough",
    /**
     * Toggles subscript on/off for the selection or at the insertion point.
     */
    Subscript = "subscript",
    /**
     * Toggles superscript on/off for the selection or at the insertion point.
     */
    Superscript = "superscript",
    /**
     * Toggles underline on/off for the selection or at the insertion point.
     */
    Underline = "underline",
    /**
     * Undoes the last executed command.
     */
    Undo = "undo",
    /**
     * Removes the anchor element from a selected hyperlink.
     */
    Unlink = "unlink",
    /**
     * Replaces the useCSS command. true modifies/generates style attributes in markup, false generates presentational elements.
     */
    StyleWithCSS = "styleWithCSS"
}

    /**
 * The is essentially an enum representing result from browser compareDocumentPosition API
 * https://developer.mozilla.org/en-US/docs/Web/API/Node/compareDocumentPosition
 */
enum CompatibleDocumentPosition {
    /**
     * Same node
     */
    Same = 0,
    /**
     * Node is disconnected from document
     */
    Disconnected = 1,
    /**
     * Node is preceding the comparing node
     */
    Preceding = 2,
    /**
     * Node is following the comparing node
     */
    Following = 4,
    /**
     * Node contains the comparing node
     */
    Contains = 8,
    /**
     * Node is contained by the comparing node
     */
    ContainedBy = 16
}

    /**
 * CSS Class names for Entity
 */
enum CompatibleEntityClasses {
    /**
     * Class name to specify this is an entity
     */
    ENTITY_INFO_NAME = "_Entity",
    /**
     * Class name to specify the type of an entity
     */
    ENTITY_TYPE_PREFIX = "_EType_",
    /**
     * Class name to specify the ID of an entity
     */
    ENTITY_ID_PREFIX = "_EId_",
    /**
     * Class name to specify if the entity is readonly
     */
    ENTITY_READONLY_PREFIX = "_EReadonly_"
}

    /**
 * The enum used for increase or decrease font size
 * Used by setFontSize API
 */
enum CompatibleFontSizeChange {
    /**
     * Increase font size
     */
    Increase = 0,
    /**
     * Decrease font size
     */
    Decrease = 1
}

    /**
 * The enum used for increase or decrease indentation of a block
 * Used by setIndentation API
 */
enum CompatibleIndentation {
    /**
     * Increase indentation
     */
    Increase = 0,
    /**
     * Decrease indentation
     */
    Decrease = 1
}

    /**
 * Key numbers common used keys
 */
enum CompatibleKeys {
    NULL = 0,
    BACKSPACE = 8,
    TAB = 9,
    ENTER = 13,
    SHIFT = 16,
    CTRL_LEFT = 17,
    ALT = 18,
    ESCAPE = 27,
    SPACE = 32,
    PAGEUP = 33,
    END = 35,
    HOME = 36,
    LEFT = 37,
    UP = 38,
    RIGHT = 39,
    DOWN = 40,
    PRINT_SCREEN = 44,
    DELETE = 46,
    /**
     * @deprecated Just for backward compatibility
     */
    EIGHT_ASTIRISK = 56,
    EIGHT_ASTERISK = 56,
    B = 66,
    I = 73,
    U = 85,
    Y = 89,
    Z = 90,
    META_LEFT = 91,
    COMMA = 188,
    DASH_UNDERSCORE = 189,
    PERIOD = 190,
    /**
     * @deprecated Just for backward compatibility
     */
    FORWARDSLASH = 191,
    FORWARD_SLASH = 191,
    GRAVE_TILDE = 192,
    /**
     * Keys below are non-standard, and should be used in ContentEditFeatures only
     */
    CONTENTCHANGED = 257,
    RANGE = 258,
    Ctrl = 4096,
    Meta = 8192,
    Shift = 16384
}

    /**
 * Index of known CreateElementData used by createElement function
 */
enum CompatibleKnownCreateElementDataIndex {
    /**
     * Set a none value to help createElement function ignore falsy value
     */
    None = 0,
    /**
     * An empty line without format
     */
    EmptyLine = 1,
    /**
     * Wrapper for blockquote
     */
    BlockquoteWrapper = 2,
    /**
     * Temp DIV for copy/paste
     */
    CopyPasteTempDiv = 3,
    /**
     * ListItem with block style
     */
    BlockListItem = 4,
    /**
     * Wrapper element for context menu
     */
    ContextMenuWrapper = 5,
    /**
     * Wrapper element for image edit
     */
    ImageEditWrapper = 6,
    /**
     * @deprecated
     */
    TableHorizontalResizer = 7,
    /**
     * @deprecated
     */
    TableVerticalResizer = 8,
    /**
     * @deprecated
     */
    TableResizerLTR = 9,
    /**
     * @deprecated
     */
    TableResizerRTL = 10,
    /**
     * @deprecated
     */
    TableSelector = 11,
    /**
     * @deprecated
     */
    EmptyLineFormatInSpan = 12
}

    /**
 * Represent the types of sources to handle in the Paste Plugin
 */
enum CompatibleKnownPasteSourceType {
    WordDesktop = 0,
    ExcelDesktop = 1,
    ExcelOnline = 2,
    PowerPointDesktop = 3,
    GoogleSheets = 4,
    WacComponents = 5,
    Default = 6,
    SingleImage = 7
}

    /**
 * Type of list (numbering or bullet)
 */
enum CompatibleListType {
    /**
     * None list type
     * It means this is not a list
     */
    None = 0,
    /**
     * Ordered List type (numbering list)
     * Represented by "OL" tag
     */
    Ordered = 1,
    /**
     * Unordered List type (bullet list)
     * Represented by "UL" tag
     */
    Unordered = 2
}

    /**
 * The is essentially an enum represents the type of the node
 * https://developer.mozilla.org/en-US/docs/Web/API/Node/nodeType
 * Values not listed here are deprecated.
 */
enum CompatibleNodeType {
    /**
     * An Element node such as &lt;p&gt; or &lt;div&gt;.
     */
    Element = 1,
    /**
     * An Attribute node such as name="value".
     */
    Attribute = 2,
    /**
     * The actual Text of Element or Attr.
     */
    Text = 3,
    /**
     * A ProcessingInstruction of an XML document such as &lt;?xml-stylesheet ... ?&gt; declaration.
     */
    ProcessingInstruction = 7,
    /**
     * A Comment node.
     */
    Comment = 8,
    /**
     * A Document node.
     */
    Document = 9,
    /**
     * A DocumentType node e.g. &lt;!DOCTYPE html&gt; for HTML5 documents.
     */
    DocumentType = 10,
    /**
     * A DocumentFragment node.
     */
    DocumentFragment = 11
}

    /**
 *  Enum used to control the different types of numbering list
 */
enum CompatibleNumberingListType {
    /**
     * Minimum value of the enum
     */
    Min = 1,
    /**
     * Numbering triggered by 1.
     */
    Decimal = 1,
    /**
     * Numbering triggered by 1-
     */
    DecimalDash = 2,
    /**
     * Numbering triggered by 1)
     */
    DecimalParenthesis = 3,
    /**
     * Numbering triggered by (1)
     */
    DecimalDoubleParenthesis = 4,
    /**
     * Numbering triggered by a.
     */
    LowerAlpha = 5,
    /**
     * Numbering triggered by a)
     */
    LowerAlphaParenthesis = 6,
    /**
     * Numbering triggered by (a)
     */
    LowerAlphaDoubleParenthesis = 7,
    /**
     * Numbering triggered by a-
     */
    LowerAlphaDash = 8,
    /**
     * Numbering triggered by A.
     */
    UpperAlpha = 9,
    /**
     * Numbering triggered by A)
     */
    UpperAlphaParenthesis = 10,
    /**
     * Numbering triggered by (A)
     */
    UpperAlphaDoubleParenthesis = 11,
    /**
     * Numbering triggered by A-
     */
    UpperAlphaDash = 12,
    /**
     * Numbering triggered by i.
     */
    LowerRoman = 13,
    /**
     * Numbering triggered by i)
     */
    LowerRomanParenthesis = 14,
    /**
     * Numbering triggered by (i)
     */
    LowerRomanDoubleParenthesis = 15,
    /**
     * Numbering triggered by i-
     */
    LowerRomanDash = 16,
    /**
     * Numbering triggered by I.
     */
    UpperRoman = 17,
    /**
     * Numbering triggered by I)
     */
    UpperRomanParenthesis = 18,
    /**
     * Numbering triggered by (I)
     */
    UpperRomanDoubleParenthesis = 19,
    /**
     * Numbering triggered by I-
     */
    UpperRomanDash = 20,
    /**
     * Maximum value of the enum
     */
    Max = 20
}

    /**
 * Represent the type of a position
 */
enum CompatiblePositionType {
    /**
     * At the beginning of a node
     */
    Begin = 0,
    /**
     * At the end of a node
     */
    End = -1,
    /**
     * Before a node
     */
    Before = -2,
    /**
     * After a node
     */
    After = -3
}

    /**
 * Operations used by editTable() API
 */
enum CompatibleTableOperation {
    /**
     * Insert a row above current row
     */
    InsertAbove = 0,
    /**
     * Insert a row below current row
     */
    InsertBelow = 1,
    /**
     * Insert a column on the left of current column
     */
    InsertLeft = 2,
    /**
     * Insert a column on the right of current column
     */
    InsertRight = 3,
    /**
     * Delete the whole table
     */
    DeleteTable = 4,
    /**
     * Delete current column
     */
    DeleteColumn = 5,
    /**
     * Delete current row
     */
    DeleteRow = 6,
    /**
     * Merge current row with the row above
     */
    MergeAbove = 7,
    /**
     * Merge current row with the row below
     */
    MergeBelow = 8,
    /**
     * Merge current column with the column on the left
     */
    MergeLeft = 9,
    /**
     * Merge current column with the column on the right
     */
    MergeRight = 10,
    /**
     * Merge all selected cells
     */
    MergeCells = 11,
    /**
     * Split current table cell horizontally
     */
    SplitHorizontally = 12,
    /**
     * Split current table cell vertically
     */
    SplitVertically = 13,
    /**
     * Align current table at the center
     */
    AlignCenter = 14,
    /**
     * Align current table at the left
     */
    AlignLeft = 15,
    /**
     * Align current table at the right
     */
    AlignRight = 16,
    /**
     * Align current content table cell at the left
     */
    AlignCellLeft = 17,
    /**
     * Align current content table cell at the center
     */
    AlignCellCenter = 18,
    /**
     * Align current content table cell at the right
     */
    AlignCellRight = 19,
    /**
     * Align current content table cell at the top
     */
    AlignCellTop = 20,
    /**
     * Align current table cell at the middle
     */
    AlignCellMiddle = 21,
    /**
     * Align current table cell at the bottom
     */
    AlignCellBottom = 22
}

    /**
 * An editor plugin to handle content edit event.
 * The following cases are included:
 * 1. Auto increase/decrease indentation on Tab, Shift+tab
 * 2. Enter, Backspace on empty list item
 * 3. Enter, Backspace on empty blockquote line
 * 4. Auto bullet/numbering
 * 5. Auto link
 * 6. Tab in table
 * 7. Up/Down in table
 * 8. Manage list style
 */
class ContentEdit implements EditorPlugin  {
    private settingsOverride?;
    private additionalFeatures?;
    private editor;
    private features;
    /**
     * Create instance of ContentEdit plugin
     * @param settingsOverride An optional feature set to override default feature settings
     * @param additionalFeatures Optional. More features to add
     */
    constructor(settingsOverride?: Partial<ContentEditFeatureSettings> | undefined, additionalFeatures?: GenericContentEditFeature<PluginEvent>[] | undefined);
    /**
     * Get a friendly name of  this plugin
     */
    getName(): string;
    /**
     * Initialize this plugin
     * @param editor The editor instance
     */
    initialize(editor: IEditor): void;
    private disposeFeatures;
    /**
     * Dispose this plugin
     */
    dispose(): void;
}

    /**
 * Get all content edit features provided by roosterjs
 */
function getAllFeatures(): Record<keyof ContentEditFeatureSettings, BuildInEditFeature<PluginEvent>>;

    /**
 * An editor plugin that support showing a context menu using render() function from options parameter
 */
class ContextMenu<T> implements EditorPlugin  {
    private options;
    private container;
    private editor;
    private isMenuShowing;
    /**
     * Create a new instance of ContextMenu class
     * @param options An options object to determine how to show/hide the context menu
     */
    constructor(options: ContextMenuOptions<T>);
    /**
     * Get a friendly name of  this plugin
     */
    getName(): string;
    /**
     * Initialize this plugin
     * @param editor The editor instance
     */
    initialize(editor: IEditor): void;
    /**
     * Dispose this plugin
     */
    dispose(): void;
    /**
     * Handle events triggered from editor
     * @param event PluginEvent object
     */
    onPluginEvent(event: PluginEvent): void;
    private initContainer;
    private onDismiss;
}

    /**
 * Context Menu options for ContextMenu plugin
 */
interface ContextMenuOptions<T> {
    /**
     * Render function for the context menu
     * @param container The container HTML element, it will be located at the mouse click position,
     * so the callback just need to render menu content into this container
     * @param onDismiss The onDismiss callback, some menu render need to know this callback so that
     * it can handle the dismiss event
     */
    render: (container: HTMLElement, items: (T | null)[], onDismiss: () => void) => void;
    /**
     * Dismiss function for the context menu, it will be called when user wants to dismiss this context menu
     * e.g. user click away so the menu should be dismissed
     * @param container The container HTML element
     */
    dismiss?: (container: HTMLElement) => void;
    /**
     * Whether the default context menu is allowed. @default false
     */
    allowDefaultMenu?: boolean;
}

    /**
 * Wrapper for CustomReplaceContentEditFeature that provides an API for updating the
 * content edit feature
 */
class CustomReplace implements EditorPlugin  {
    private longestReplacementLength;
    private editor;
    private replacements;
    private replacementEndCharacters;
    /**
     * Create instance of CustomReplace plugin
     * @param replacements Replacement rules. If not passed, a default replacement rule set will be applied
     */
    constructor(replacements?: CustomReplacement[]);
    /**
     * Set the replacements that this plugin is looking for.
     * @param newReplacements new set of replacements for this plugin
     */
    updateReplacements(newReplacements: CustomReplacement[]): void;
    /**
     * Get a friendly name of this plugin
     */
    getName(): string;
    /**
     * Initialize this plugin
     * @param editor The editor instance
     */
    initialize(editor: IEditor): void;
    /**
     * Dispose this plugin
     */
    dispose(): void;
    /**
     * Handle events triggered from editor
     * @param event PluginEvent object
     */
    onPluginEvent(event: PluginEvent): void;
    private getMatchingReplacement;
}

    /**
 * Maintain list numbers of list chain when content is modified by cut/paste/drag&drop
 */
class CutPasteListChain implements EditorPlugin  {
    private chains;
    private expectedChangeSource;
    private editor;
    private disposer;
    /**
     * Get a friendly name of this plugin
     */
    getName(): string;
    /**
     * Initialize this plugin
     * @param editor The editor instance
     */
    initialize(editor: IEditor): void;
    /**
     * Dispose this plugin
     */
    dispose(): void;
    /**
     * Handle events triggered from editor
     * @param event PluginEvent object
     */
    onPluginEvent(event: PluginEvent): void;
    private onDrop;
    private cacheListChains;
}

    /**
 * An editor plugin that show a tooltip for existing link
 */
class HyperLink implements EditorPlugin  {
    private getTooltipCallback;
    private target?;
    private onLinkClick?;
    private originalHref;
    private trackedLink;
    private editor;
    private disposer;
    /**
     * Create a new instance of HyperLink class
     * @param getTooltipCallback A callback function to get tooltip text for an existing hyperlink.
     * Default value is to return the href itself. If null, there will be no tooltip text.
     * @param target (Optional) Target window name for hyperlink. If null, will use "_blank"
     * @param onLinkClick (Optional) Open link callback (return false to use default behavior)
     */
    constructor(getTooltipCallback?: (href: string, a: HTMLAnchorElement) => string, target?: string | undefined, onLinkClick?: ((anchor: HTMLAnchorElement, mouseEvent: MouseEvent) => boolean | void) | undefined);
    /**
     * Get a friendly name of  this plugin
     */
    getName(): string;
    /**
     * Initialize this plugin
     * @param editor The editor instance
     */
    initialize(editor: IEditor): void;
    protected onMouse: (e: MouseEvent) => void;
    protected onBlur: (e: FocusEvent) => void;
    /**
     * Dispose this plugin
     */
    dispose(): void;
    /**
     * Handle events triggered from editor
     * @param event PluginEvent object
     */
    onPluginEvent(event: PluginEvent): void;
    /**
     * Try get href from an anchor element
     * The reason this is put in a try-catch is that
     * it has been seen that accessing href may throw an exception, in particular on IE/Edge
     */
    private tryGetHref;
    /**
     * Determines if KeyboardEvent is meant to edit content
     */
    private isContentEditValue;
    /**
     * Updates the href of the tracked link if the display text doesn't match href anymore
     */
    private updateLinkHrefIfShouldUpdate;
    /**
     * Clears the tracked link and its original href value so that it's back to default state
     */
    private resetLinkTracking;
    /**
     * Compares the normalized URL of inner text of element to its href to see if they match.
     */
    private doesLinkDisplayMatchHref;
    /**
     * Update href of an element in place to new display text if it's a valid URL
     */
    private updateLinkHref;
}

    /**
 * ImageEdit plugin provides the ability to edit an inline image in editor, including image resizing, rotation and cropping
 */
class ImageEdit implements EditorPlugin  {
    private onShowResizeHandle?;
    protected editor: IEditor | null;
    protected options: ImageEditOptions;
    private disposer;
    private allowedOperations;
    private image;
    private clonedImage;
    private wrapper;
    private editInfo;
    private lastSrc;
    private dndHelpers;
    /**
     * Identify if the image was resized by the user.
     */
    private wasResized;
    /**
     * The span element that wraps the image and opens shadow dom
     */
    private shadowSpan;
    /**
     * The span element that wraps the image and opens shadow dom
     */
    private isCropping;
    /**
     * If the image is a gif, this is the png source of the gif image
     */
    private pngSource;
    /**
     * Create a new instance of ImageEdit
     * @param options Image editing options
     * @param onShowResizeHandle An optional callback to allow customize resize handle element of image resizing.
     * To customize the resize handle element, add this callback and change the attributes of elementData then it
     * will be picked up by ImageEdit code
     */
    constructor(options?: ImageEditOptions, onShowResizeHandle?: OnShowResizeHandle | undefined);
    /**
     * Get a friendly name of this plugin
     */
    getName(): string;
    /**
     * Initialize this plugin. This should only be called from Editor
     * @param editor Editor instance
     */
    initialize(editor: IEditor): void;
    /**
     * Dispose this plugin
     */
    dispose(): void;
    /**
     * Handle events triggered from editor
     * @param e PluginEvent object
     */
    onPluginEvent(e: PluginEvent): void;
    /**
     * Check if the given image edit operation is allowed by this plugin
     * @param operation The image edit operation to check
     * @returns True means it is allowed, otherwise false
     */
    isOperationAllowed(operation: ImageEditOperation): boolean;
    /**
     * Set current image for edit. If there is already image in editing, it will quit editing mode and any pending editing
     * operation will be submitted
     * @param image The image to edit
     * @param operation The editing operation
     */
    setEditingImage(image: HTMLImageElement, operation: ImageEditOperation | CompatibleImageEditOperation): void;
    /**
     * Stop editing image. If there is already image in editing, it will quit editing mode and any pending editing
     * operation will be submitted
     * @param image The image to edit
     * @param selectImage True to select this image after quit editing mode
     */
    setEditingImage(image: null, selectImage?: boolean): void;
    /**
     * Flip the image.
     * @param image The image to be flipped
     * @param direction
     */
    flipImage(image: HTMLImageElement, direction: 'vertical' | 'horizontal'): void;
    /**
     * Rotate the image in radian angle.
     * @param image The image to be rotated
     * @param angleRad The angle in radian that the image must be rotated.
     */
    rotateImage(image: HTMLImageElement, angleRad: number): void;
    /**
     * quit editing mode when editor lose focus
     */
    private onBlur;
    /**
     * Create editing wrapper for the image
     */
    private createWrapper;
    private insertImageWrapper;
    /**
     * Remove the temp wrapper of the image
     */
    private removeWrapper;
    /**
     * Update image edit elements to reflect current editing result
     * @param context
     */
    private updateWrapper;
    /**
     * Create drag and drop helpers
     * @param wrapper
     * @param elementClass
     * @param dragAndDrop
     */
    private createDndHelpers;
    /**
     * Clean up drag and drop helpers
     */
    private clearDndHelpers;
}

    /**
 * Check if we can regenerate edited image from the source image.
 * An image can't regenerate result when there is CORS issue of the source content.
 * @param img The image element to test
 * @returns True when we can regenerate the edited image, otherwise false
 */
function canRegenerateImage(img: HTMLImageElement): boolean;

    /**
 * Resize the image by percentage of its natural size. If the image is cropped or rotated,
 * the final size will also calculated with crop and rotate info.
 * @param editor The editor that contains the image
 * @param image The image to resize
 * @param percentage Percentage to resize to
 * @param minWidth Minimum width
 * @param minHeight Minimum height
 */
function resizeByPercentage(editor: IEditor, image: HTMLImageElement, percentage: number, minWidth: number, minHeight: number): void;

    /**
 * Check if the image is already resized to the given percentage
 * @param image The image to check
 * @param percentage The percentage to check
 */
function isResizedTo(image: HTMLImageElement, percentage: number): boolean;

    /**
 * Remove explicit width & height attributes on the image element.
 * @param editor The editor that contains the image
 * @param image The image to remove w/h from
 */
function resetImage(editor: IEditor, image: HTMLImageElement): void;

    /**
 * An optional callback to allow customize resize handle element of image resizing.
 * To customize the resize handle element, add this callback and change the attributes of elementData then it
 * will be picked up by ImageEdit code
 */
interface OnShowResizeHandle {
    (elementData: CreateElementData, x: DNDDirectionX, y: DnDDirectionY): void;
}

    /**
 * Horizontal direction types for image edit
 */
type DNDDirectionX = 'w' | '' | 'e';

    /**
 * Vertical direction types for image edit
 */
type DnDDirectionY = 'n' | '' | 's';

    /**
 * @deprecated Use ImageEdit plugin instead
 */
class ImageResize extends ImageEdit  {
    /**
     * Create a new instance of ImageResize
     * @param minWidth Minimum width of image when resize in pixel, default value is 10
     * @param minHeight Minimum height of image when resize in pixel, default value is 10
     * @param selectionBorderColor Color of resize border and handles, default value is #DB626C
     * @param forcePreserveRatio Whether always preserve width/height ratio when resize, default value is false
     * @param resizableImageSelector Selector for picking which image is resizable (e.g. for all images not placeholders), note
     * that the tag must be IMG regardless what the selector is
     */
    constructor(minWidth?: number, minHeight?: number, selectionBorderColor?: string, forcePreserveRatio?: boolean, resizableImageSelector?: string);
    /**
     * @deprecated
     */
    showResizeHandle(img: HTMLImageElement): void;
    /**
     * @deprecated
     */
    hideResizeHandle(selectImageAfterUnSelect?: boolean): void;
}

    /**
 * Paste plugin, handles BeforePaste event and reformat some special content, including:
 * 1. Content copied from Word
 * 2. Content copied from Excel
 * 3. Content copied from Word Online or OneNote Online
 */
class Paste implements EditorPlugin  {
    private unknownTagReplacement;
    private convertSingleImageBody;
    private editor;
    /**
     * Construct a new instance of Paste class
     * @param unknownTagReplacement Replace solution of unknown tags, default behavior is to replace with SPAN
     * @param convertSingleImageBody When enabled, if clipboard HTML contains a single image, we reuse the image without modifying the src attribute.
     *                               When disabled, pasted image src attribute will use the dataUri from clipboard data -- By Default disabled.
     */
    constructor(unknownTagReplacement?: string, convertSingleImageBody?: boolean);
    /**
     * Get a friendly name of  this plugin
     */
    getName(): string;
    /**
     * Initialize this plugin. This should only be called from Editor
     * @param editor Editor instance
     */
    initialize(editor: IEditor): void;
    /**
     * Dispose this plugin
     */
    dispose(): void;
    /**
     * Handle events triggered from editor
     * @param event PluginEvent object
     */
    onPluginEvent(event: PluginEvent): void;
}

    /**
 * PickerPlugin represents a plugin of editor which can handle picker related behaviors, including
 * - Show picker when special trigger key is pressed
 * - Hide picker
 * - Change selection in picker by Up/Down/Left/Right
 * - Apply selected item in picker
 *
 * PickerPlugin doesn't provide any UI, it just wraps related DOM events and invoke callback functions.
 */
class PickerPlugin<T extends PickerDataProvider = PickerDataProvider> implements EditorPlugin  {
    readonly dataProvider: T;
    private pickerOptions;
    private editor;
    private eventHandledOnKeyDown;
    private blockSuggestions;
    private isSuggesting;
    private lastKnownRange;
    private isPendingInputEventHandling;
    private currentInputLength;
    private newInputLength;
    constructor(dataProvider: T, pickerOptions: PickerPluginOptions);
    /**
     * Get a friendly name
     */
    getName(): string;
    /**
     * Initialize this plugin. This should only be called from Editor
     * @param editor Editor instance
     */
    initialize(editor: IEditor): void;
    /**
     * Dispose this plugin
     */
    dispose(): void;
    /**
     * Check if the plugin should handle the given event exclusively.
     * Handle an event exclusively means other plugin will not receive this event in
     * onPluginEvent method.
     * If two plugins will return true in willHandleEventExclusively() for the same event,
     * the final result depends on the order of the plugins are added into editor
     * @param event The event to check
     */
    willHandleEventExclusively(event: PluginEvent): boolean;
    /**
     * Handle events triggered from editor
     * @param event PluginEvent object
     */
    onPluginEvent(event: PluginEvent): void;
    private setLastKnownRange;
    private setIsSuggesting;
    private cancelDefaultKeyDownEvent;
    private getIdValue;
    private getWordBeforeCursor;
    private replaceNode;
    private getRangeUntilAt;
    private shouldHandleKeyUpEvent;
    private onKeyUpDomEvent;
    private onKeyDownEvent;
    private getParentNodeIfTextNode;
    private onAndroidInputEvent;
    private calcInputLength;
    private tryRemoveNode;
    private getWord;
    private setRangeStart;
    private setAriaOwns;
    private setAriaActiveDescendant;
    private getInlineElementBeforeCursor;
    private isAndroidKeyboardEvent;
}

    /**
 * TableResize plugin, provides the ability to resize a table by drag-and-drop
 */
class TableResize implements EditorPlugin  {
    private onShowHelperElement?;
    private anchorContainerSelector?;
    private editor;
    private onMouseMoveDisposer;
    private tableRectMap;
    private tableEditor;
    /**
     * Construct a new instance of TableResize plugin
     * @param onShowHelperElement An optional callback to allow customize helper element of table resizing.
     * To customize the helper element, add this callback and change the attributes of elementData then it
     * will be picked up by TableResize code
     * @param anchorContainerSelector An optional selector string to specify the container to host the plugin.
     * The container must not be affected by transform: scale(), otherwise the position calculation will be wrong.
     * If not specified, the plugin will be inserted in document.body
     */
    constructor(onShowHelperElement?: ((elementData: CreateElementData, helperType: 'CellResizer' | 'TableInserter' | 'TableResizer' | 'TableSelector') => void) | undefined, anchorContainerSelector?: string | undefined);
    /**
     * Get a friendly name of  this plugin
     */
    getName(): string;
    /**
     * Initialize this plugin. This should only be called from Editor
     * @param editor Editor instance
     */
    initialize(editor: IEditor): void;
    private onMouseOut;
    /**
     * Dispose this plugin
     */
    dispose(): void;
    /**
     * Handle events triggered from editor
     * @param event PluginEvent object
     */
    onPluginEvent(e: PluginEvent): void;
    private onMouseMove;
    private setTableEditor;
    private invalidateTableRects;
    private disposeTableEditor;
    private ensureTableRects;
}

    /**
 * A watermark plugin to manage watermark string for roosterjs
 */
class Watermark implements EditorPlugin  {
    private watermark;
    private customClass?;
    private editor;
    private disposer;
    private format;
    /**
     * Create an instance of Watermark plugin
     * @param watermark The watermark string
     */
    constructor(watermark: string, format?: DefaultFormat, customClass?: string | undefined);
    /**
     * Get a friendly name of  this plugin
     */
    getName(): string;
    /**
     * Initialize this plugin. This should only be called from Editor
     * @param editor Editor instance
     */
    initialize(editor: IEditor): void;
    /**
     * Dispose this plugin
     */
    dispose(): void;
    /**
     * Handle events triggered from editor
     * @param event PluginEvent object
     */
    onPluginEvent(event: PluginEvent): void;
    private showHideWatermark;
    private removeWatermark;
}

    /**
 * TableCellSelectionPlugin help highlight table cells
 */
class TableCellSelection implements EditorPlugin  {
    private editor;
    private state;
    private shadowEditCoordinatesBackup;
    constructor();
    /**
     * Get a friendly name of  this plugin
     */
    getName(): string;
    /**
     * Initialize this plugin. This should only be called from Editor
     * @param editor Editor instance
     */
    initialize(editor: IEditor): void;
    /**
     * Dispose this plugin
     */
    dispose(): void;
    /**
     * Handle events triggered from editor
     * @param event PluginEvent object
     */
    onPluginEvent(event: PluginEvent): void;
    private handleLeavingShadowEdit;
    private handleEnteredShadowEdit;
}

    /**
 * Automatically transform -- into hyphen, if typed between two words.
 */
class AutoFormat implements EditorPlugin  {
    private editor;
    private lastKeyTyped;
    /**
     * Get a friendly name of this plugin
     */
    getName(): string;
    /**
     * Initialize this plugin
     * @param editor The editor instance
     */
    initialize(editor: IEditor): void;
    /**
     * Dispose this plugin
     */
    dispose(): void;
    /**
     * Handle events triggered from editor
     * @param event PluginEvent object
     */
    onPluginEvent(event: PluginEvent): void;
}

    /**
 * Announce feature keys
 */
type AnnounceFeatureKey = 'announceNewListItem' | 'announceWarningOnLastTableCell';

    /**
 * Represents a Announce feature used in Announce Plugin.
 * If the Should Handle Callback returns announce data, it will be announced by using a aria-live region.
 */
interface AnnounceFeature {
    /**
     * Whether to handle this feature, if returns Announce Data, will be announced, otherwise will do nothing.
     * @returns
     */
    shouldHandle: (editor: IEditor, lastFocusedElement: HTMLElement | null) => AnnounceData | false;
    /**
     * Keys handled in the current event
     */
    keys: number[];
}

    /**
 * Announce messages to screen reader by using aria live element.
 */
class Announce implements EditorPlugin  {
    private stringsMapOrGetter?;
    private ariaLiveElement;
    private editor;
    private features;
    private lastFocusedElement;
    constructor(stringsMapOrGetter?: Map<KnownAnnounceStrings | CompatibleKnownAnnounceStrings, string> | ((key: KnownAnnounceStrings | CompatibleKnownAnnounceStrings) => string) | undefined, skipAnnounceFeatures?: AnnounceFeatureKey[], additionalFeatures?: AnnounceFeature[]);
    /**
     * Get a friendly name of this plugin
     */
    getName(): string;
    /**
     * Initialize this plugin
     * @param editor The editor instance
     */
    initialize(editor: IEditor): void;
    /**
     * Dispose this plugin
     */
    dispose(): void;
    /**
     * Handle events triggered from editor
     * @param event PluginEvent object
     */
    onPluginEvent(ev: PluginEvent): void;
    private handleFeatures;
    protected announce(announceData: AnnounceData, editor: IEditor): void;
    private getString;
    /**
     * @internal
     * Public only for unit testing.
     * @returns
     */
    getAriaLiveElement(): HTMLDivElement | undefined;
}

}